#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Control/Icon.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc: 

package SAPDB::Install::Control::Icon;

use Wx::Event qw (
	EVT_LEFT_DOWN EVT_LEFT_UP
	EVT_LEFT_DCLICK EVT_SET_FOCUS);

use Wx qw (wxICON_ERROR wxEVT_COMMAND_BUTTON_CLICKED);

use SAPDB::Install::Control::StaticText;
use SAPDB::Install::Control::StaticImage;

our @ISA = qw (Wx::Panel);

sub new{
	my($this ) = shift->SUPER::new(@_);
	#$this->SetWindowStyle(wxWANTS_CHARS);
	$this->SetBackgroundColour($this->GetParent->GetBackgroundColour);
	$this->{'selected'} = 0;
	EVT_LEFT_DOWN($this,\&OnLeftDown);
	EVT_LEFT_UP($this,\&OnLeftUp);
	EVT_LEFT_DCLICK($this,\&OnDClick);
	$this->SetNextHandler($this->GetParent);
	EVT_SET_FOCUS($this,\&OnSetFocus);
	return $this;
}

sub OnPaint{
	my ($self,$event) = @_;
	if(defined $self->{'simg'}){
		$self->{'simg'}->Refresh;
	}
	if(defined $self->{'txtctrl'}){
		$self->{'txtctrl'}->Refresh;
	}				
}

sub OnLeftDown{
	my ($self,$event) = @_;
	$self->{'down'} = 1;	
}

sub OnDClick{
	my ($self,$event) = @_;
	$self->Call;
}

sub OnLeftUp{
	my ($self,$event) = @_;
	if($self->{'down'}){
		unless(defined $self->{'button_event'}){
			$self->{'button_event'} = Wx::CommandEvent->new(wxEVT_COMMAND_BUTTON_CLICKED,$self->GetId);
		}
		$self->GetParent->AddPendingEvent($self->{'button_event'});
	}
	$self->{'down'} = 0;
}

sub setText{
	my ($self,$text) = @_;
	$self->{'txtctrl'} = SAPDB::Install::Control::StaticText->new($self,-1,[0,64],[64,20]);
	$self->{'txtctrl'}->SetNextHandler($self);
	$self->{'txtctrl'}->setText($text);
	my ($textwidth,$textheight) = @{$self->{'txtctrl'}->{'current_textextent'}};
	$self->{'txtctrl'}->SetSize([$textwidth,$textheight]);
	if(defined $self->{'simg'}){
		my $iconheight = $self->{'img'}->GetHeight + $textheight;
		my $iconwidth =  $self->{'img'}->GetWidth;
		if($textwidth > $self->{'img'}->GetWidth){
			my $img_pos = ($textwidth - $iconwidth ) / 2;
			$self->{'simg'}->Move([$img_pos,0]); 
			$iconwidth = $textwidth;
			
		}
		$self->SetSize([$iconwidth,$iconheight]);  
		$self->{'txtctrl'}->Move([0,$self->{'img'}->GetHeight]);
	}
	$self->{'txtctrl'}->setText($text,1);

}

sub select{
	my ($self) = @_;
	$self->{'txtctrl'}->setInvers;
}

sub unselect{
	my ($self) = @_;
	$self->{'txtctrl'}->setInvers;
}




sub setImg{
	my ($self,$img) = @_;
	$self->{'img'} = $img;
	my $width = $img->GetWidth;
	if(defined $self->{'txtctrl'}){
		my ($twidth,$theight) = $self->{'txtctrl'}->GetSizeWH;
		$width = $twidth if $twidth > $width;	
	}
	$self->SetSize([$img->GetWidth,$img->GetHeight]);
	$self->{'simg'} = SAPDB::Install::Control::StaticImage->new($self,-1,[0,0],[64,100]);
	$self->{'simg'}->SetData([$img->GetData,$img->GetWidth,$img->GetHeight]);
	$self->{'simg'}->Render;
}

sub Call{
	my ($self,$args) = @_;
	if(defined $self->{'cmd_func'}){
		my @argv = ();
		if(defined $args){
			push @argv,@$args;
		}
		if(defined $self->{'cmd_args'}){
			push @argv,@{$self->{'cmd_args'}};
		}
		&{$self->{'cmd_func'}}(@argv);
	}
	else{
		my $msgbox = Wx::MessageDialog->new($self,"Icon \"".$self->{'txtctrl'}->{'text'}."\" has no Cmd Function Reference!\n",
						"Error: Icon Call", wxICON_ERROR | wxCENTRE);
		$msgbox->ShowModal();
	}	
}

sub SetToolTip{
	my ($self,$text) = @_;
	$self->{'simg'}->SetToolTip($text);
	$self->{'txtctrl'}->SetToolTip($text);
	$self->SUPER::SetToolTip($text);

		
}


sub OnSetFocus{
	my ($self,$event) = @_;
	$self->GetParent->SetFocus;
	if(defined $self->{'simg'}){
		$self->{'simg'}->Destroy;
		delete $self->{'simg'};
	}
}

sub Destroy{
	my ($self) = @_;
	if(defined $self->{'txtctrl'}){
		$self->{'txtctrl'}->Destroy;
		delete $self->{'txtctrl'};
	}
	$self->SUPER::Destroy;

}

sub DESTROY{
	my ($self) = @_;
	print ">>>> DESTROY $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}



1;