#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Instance/Check/Finalize.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#

package SAPDB::Install::Instance::Check::Finalize;

use SAPDB::Install::Instance::Base;
use SAPDB::Install::Instance::Check::Common;
use SAPDB::Install::Instance::Check::Conditions;
use SAPDB::Install::Instance::Check::Finalize::Generic;
use SAPDB::Install::Instance::Check::Finalize::ConData;
use SAPDB::Install::Instance::Check::Finalize::ExternalBackup;
use SAPDB::Install::Instance::Check::Finalize::ExtLVCCons;

push @ISA, 
	'SAPDB::Install::Instance::Base',
	'SAPDB::Install::Instance::Check::Common',
	'SAPDB::Install::Instance::Check::Conditions';

#
# run any neccessary finalize check
# it depends on migration strategy
#
sub finalizecheck {
	my ($self) = @_; 

	#
	# find out instance type if not already known
	#
	unless (defined $self->{'instancetype'}) {
		my $rc = $self->conditions ();
		unless (defined $rc && $rc == 0) {
			$self->set_errorstate ('ERROR');
			return undef;
		}
	}

	$self->msgbegin ("running finalize check");
	$self->msg0 ("running finalize check...\n");

	my $migration_strategy = $self->{migration_strategy};
	unless (defined $migration_strategy && $migration_strategy ne 'NONE') {
		$self->msg0 ("unknown migration strategy\n");
		$self->msgend ();
		$self->set_errorstate ('ERROR');
		return undef;
	}
	
	#
	# find required check method
	# 	
	my $class = 'SAPDB::Install::Instance::Check::Finalize::';
	if ($migration_strategy eq 'EXTERNAL_CONSISTENT_BACKUP') {
		$class .= 'ExternalBackup';
	} elsif ($migration_strategy eq 'EXTERNAL_LIVECACHE_CONSISTENCE') {
		$class .= 'ExtLVCCons';
	} elsif ($migration_strategy eq 'CONSISTENT_DATA') {
		$class .= 'ConData';
	} else {
		$class .= 'Generic';
	}

	# save prev class before setting new class
	my ($prev) = ref ($self);

	bless ($self, $class);
	my $rc = $self->finalizecheck ();

	# restore prev class before returning
	bless ($self, $prev);
	return $rc;
}

1;
