#!/usr/bin/perl
#
# $Header$
# $DateTime$
# $Change$
#
# Desc: Resource module of SDBINST / SDBUPD / sdbunist / sdbregview / sdbverify
#

package SAPDB::Install::Resources;

use Wx;
use SAPDB::Install::StdIO;
use SAPDB::Install::Image;	

our %resource_cache;

sub new {
	my ($type,$archive) = @_;
	return $resource_cache{$archive} if (exists $resource_cache{$archive});

	-f $archive or
	print2stderr("cannot open archive $archive: $!\n")
	and return;

	my %members;
	$members{'archive'} = $archive;
	$members{'tar'} = SDB::Common::Archive::Tar::new ($archive);
	unless (defined $members{'tar'}) {
		print2stderr("cannot open archive: $archive\n");
		return;
	}

	$resource_cache{$archive} = bless(\%members,$type);
	return $resource_cache{$archive};
}

sub readAll {
	my ($self) = @_;

	my $ah = $self->{'tar'};
	$ah->rewind ();

	my $bmp_hnd;
	my $png_hnd;
	my $jpg_hnd;

	for(;;){
		my $name = $ah->read ();
		last unless (defined $name && $name ne '');

		if($name =~ /\.bmp$|\.png$|\.jpg$|\.jepg$/i) {
			my $fh = $ah->open ($name);
			my $img = SAPDB::Install::Image->new ();

			if ($name =~ /\.png$/i){
				unless(defined $png_hnd){
					$png_hnd = Wx::PNGHandler->new(); 
				}
				$png_hnd->LoadFile ($img, $fh);
			} elsif ($name =~ /\.bmp$/i) {
				unless(defined $bmp_hnd){
					$bmp_hnd = Wx::BMPHandler->new();
				}
				$bmp_hnd->LoadFile ($img, $fh);
			} elsif ($name =~ /\.jpg$/i || $name =~ /\.jepg$/i) {
				unless(defined $jpg_hnd){
					$jpg_hnd = Wx::JPEGHandler->new();
				}
				$jpg_hnd->LoadFile ($img, $fh);
			}
			$self->{$name} = $img;			
		}
	}
}

sub getImages{
	my ($self,@list) = @_;
	my %return_value;
	my @missing;

	foreach my $image (@list){
		if( defined $self->{$image}){
			$return_value{$image} = $self->{$image};
		} else {
			push @missing, $image;
		}
	}

	if ($#missing >= 0) {
		my $ah = $self->{'tar'};
		$ah->rewind ();

		my $bmp_hnd;
		my $png_hnd;
		my $jpg_hnd;

		for (;;) {
			my $name = $ah->read ();
			last unless (defined $name && $name ne '');

			foreach my $mem (@missing){
				next unless ($name eq $mem);
				my $fh = $ah->open ($name);
				my $img = SAPDB::Install::Image->new ();
				if($name =~ /\.png$/i){
					unless(defined $png_hnd){
						$png_hnd = Wx::PNGHandler->new(); 
					}
					$png_hnd->LoadFile ($img, $fh);
				} elsif ($name =~ /\.jpg$/i || $name =~ /\.jepg$/i) {
					unless(defined $jpg_hnd){
						$jpg_hnd = Wx::JPEGHandler->new();
					}
					$jpg_hnd->LoadFile ($img, $fh);
				} 
				else{
					unless(defined $bmp_hnd){
						$bmp_hnd = Wx::BMPHandler->new();
					}
					$bmp_hnd->LoadFile ($img, $fh);
				}
				$self->{$name} = $img;
				$return_value{$name} = $img;				
			}
		}
	}

	return \%return_value;
}

sub getData {
	my ($self,@list) = @_;
	$self->{'tar'}->rewind();
	my %return_value;

	for(;;){
		my $name = $self->{'tar'}->read ();
		last unless (defined $name && $name ne '');

		foreach my $mem (@list){
			next unless ($name eq $mem);

			my $fh = $ah->open ($name);
			my $img = SAPDB::Install::Image->new ();

			my $imgdata = $img->GetData ();
			my $height = $img->GetHeight ();
			my $width = $img->GetWidth ();
			$return_value{$mem} = [$imgdata,$width,$height];
		}
	}
	return \%return_value;
}

1;
