#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Service/DBM.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc: 

package SAPDB::Install::Service::DBM;

use SAPDB::Install::Registry;
use SAPDB::Install::System;
use SAPDB::Install::Tools;

our @ISA = ();

sub new{
	my ($class,$dbname,$host) = @_;
	my $members = {
		'dbname' => $dbname,
		'host' => $host,
		'dbroot' => undef,
		'state' => undef,
		'last_error' => undef,
		'exec_rc' => undef,
		'command_out' => undef,
		'log' => undef,
		'dbmcli_image' => undef,
		'xstart_image' => undef,
		'xstop_image' => undef,
		'xcons_image' => undef,
		'dbmreg_image' => undef,
		'dbmunreg_image' => undef,
		'dbmstart_image' => undef,
		'dbmstop_image' => undef,
		'dbmsrv_image' => undef,
		'connected' => undef,
		'dbm' => undef
	};
	my $self = bless ($members,$class);
	$self->get_dbroot;
	return $self;	
}

sub connect{
	my ($self,$user,$passwd) = @_;
	return 1 if ($self->{'connected'});
	if (defined $user and defined $passwd){
		require SAPDB::Install::DBMCmd;
		$self->{'dbm'} = 
			SAPDB::Install::DBMCmd->new($self->{host},
										$self->{dbname});
		$self->{'connected'} = 1;
		return 1;
	}
	return undef;
}

sub disconnect{
	my ($self,$user,$passwd) = @_;
	if($self->{'connected'}){
		$self->{'dbm'} = undef;
		$self->{'connected'} = undef;
	}	
	return 1;
}

sub reset_dbroot{
	my ($self) = @_;
	$self->{'dbroot'} = undef;
	$self->{'registered'} = undef;
	foreach my $prog ('dbmstart','dbmstop','dbmreg','dbmunreg','dbmsrv'){
		if (defined $self->{$prog.'_image'}){
			$self->{$prog.'_image'} = undef;
		}
	}
	
}

sub get_dbroot{
	my ($self) = @_;
	return $self->{'dbroot'} if defined $self->{'dbroot'};
	my %dbs = getInstances();
	unless (exists $dbs{$self->{'dbname'}}){
		$self->{'last_error'} = 'database '.$self->{'dbname'}.' not found';
		return undef;
	}
	$self->{'dbroot'} = $dbs{$self->{'dbname'}};
	$self->{'registered'} = 1;
}

sub get_prog_image{
	my ($self,$prog) = @_;
	return undef unless exists $self->{$prog.'_image'};
	
	if ($prog =~ /^dbm/ and $prog !~ /dbmcli/){
		
		#
		# version dependent programs
		#
	
		unless (defined $self->{'dbroot'}){
			
			#
			# find dbroot
			#
			
			unless (defined $self->get_dbroot()){
				return undef;
			}
			
		}
		
		unless (-d $self->{'dbroot'}){
			$self->{'last_error'} = 'dependent path '.$self->{'dbroot'}.' does not exist';
			return undef;
		}
		my $path = $self->{'dbroot'}.'/pgm/'.$prog;
		$path .= '.exe' if $^O =~ /mswin/i;
		unless (-f $path){
			$self->{'last_error'} = 'cannot find program $prog';
			 return undef;
		}
		$self->{$prog.'_image'} = $path;			
	
	}
	else{
	
		#
		# version independent programs
		#
		
		my ($indep_data,$indep_prog) = readIndepPath();
		my $prog_image = $prog;
		$prog_image =~ s/^x/x_/;
		if ($^O =~ /mswin/i){
			my $path = $indep_prog.($prog =~ /dbmcli/ ? 
										'/pgm/' :
										'/bin/').$prog_image.'.exe';
			if (-f $path){
				$path =~ s/\//\\/g;
				$self->{$prog.'_image'} = $path;
			}
			else{
			  $self->{'last_error'} = 'cannot find program '.$prog_image;
			  return undef;
			}
		}
		else{
			my $path = $indep_prog.'/bin/'.$prog_image;
			if (-x $path){
				$self->{$prog.'_image'} = $path;
			}
			else{
			  $self->{'last_error'} = 'cannot find program '.$prog_image; 	
			  return undef;
			}
		}
	}
	return 1;
}

sub db_offline{
	my ($self) = @_;
	if ($self->{'connected'}){
		return $self->{'dbm'}->db_offline;
	}
	
	unless (defined $self->{'dbname'}){
		$self->{last_error} = 'database name unknown';
		return undef;
	}

	if (defined $host && $host ne 'localhost'){
		$self->{last_error} = 'operation is denied '.
				'on remote host, you have to login first';
		return undef;
	}
	
	unless ($self->{'registered'}){
		$self->{'last_error'} = 'database '.$self->{'dbname'}." is unknown\n";
		return undef;
	}	
	
	unless (defined $self->db_state){
		$self->{'last_error'} = 'cannot get database state: '.
								 $self->{'last_error'};
		return undef;
	}
	
	if ($self->{'state'} eq 'OFFLINE'){
		return 1;
	}
	
	my $stop_prog;
	
	if ($^O =~ /mswin/i){
		$self->get_prog_image('dbmstop')
			unless defined $self->{'dbmstop_image'};
		return undef unless defined $self->{'dbmstop_image'};
		$stop_prog = $self->{'dbmstop_image'};
		$stop_prog =~ s/\//\\/g
	}
	else{
		$self->get_prog_image('xstop')
			unless defined $self->{'xstop_image'};
		return undef unless defined $self->{'xstop_image'};
		$stop_prog = $self->{'xstop_image'};
	}
	
	my $cmd = '"'.$stop_prog.'" '.
				$self->{dbname};
	my $out;
	my $rc = callsystem ($cmd,sub {$out = $_[0];1;},1);
	if (not defined $rc || $rc != 0){
		$self->{last_error} = 'x_start failed (return code $rc): '.
								$out;
		return undef;
	}
	return 1;
}

sub db_online{
	my ($self) = @_;
	if ($self->{'connected'}){
		return $self->{'dbm'}->db_online;
	}
	
	unless (defined $self->{'dbname'}){
		$self->{last_error} = 'database name unknown';
		return undef;
	}

	if (defined $host && $host ne 'localhost'){
		$self->{last_error} = 'operation is denied '.
			'on remote host, you have to login first';
		return undef;
	}

	unless ($self->{'registered'}){
		$self->{'last_error'} = 'database '.$self->{'dbname'}." is unknown\n";
		return undef;
	}
	
	unless (defined $self->db_state){
		$self->{'last_error'} = 'cannot get database state: '.
								 $self->{'last_error'};
		return undef;
	}
	
	if ($self->{'state'} eq 'ONLINE'){
		return 1;
	}
	
	if ($self->{'state'} eq 'ADMIN'){
		unless (defined $self->db_offline){
			$self->{'last_error'} = 'cannot switch '.
						'to offline to prepare restart: '.
						$self->{'last_error'};
			return undef;
		}
	}
	
	my $start_prog;
	
	if ($^O =~ /mswin/i){
		unless (defined $self->{'dbmstart_image'}){
			unless ($self->get_prog_image('dbmstart')){
				return undef;
			}
		}
		$start_prog = $self->{'dbmstart_image'};
		$start_prog =~ s/\//\\/g;
	}
	else{
		unless (defined $self->{'xstart_image'}){
			unless ($self->get_prog_image('xstart')){
				return undef;
			}
		}
		$start_prog = $self->{'xstart_image'}
	}
	
	my $cmd = '"'.$start_prog.'" '.$self->{dbname};
	my $out;
	my $rc = callsystem ($cmd,sub {$out = $_[0]},1);
	if ($rc != 0){
		$self->{last_error} = 'command \''.$cmd.
			'\' ended with error code '.$rc.": \n".$out;
		return undef;
	}
	return 1;
}

sub db_drop{
	my ($self) = @_;
	if ($self->{'connected'}){
		return $self->{'dbm'}->db_drop();
	}
	$self->{last_error} = 'not yet implemeted';
	return undef;
}

sub db_state{
	my ($self) = @_;
	if ($self->{'connected'}){
		return $self->{'dbm'}->db_state();
	}
	
	unless ($self->{'registered'}){
		$self->{'last_error'} = 'database '.$self->{'dbname'}." is unknown\n";
		return undef;
	}
	
	unless (defined $self->{'xcons_image'}){
		$self->get_prog_image('xcons');
		unless (defined $self->{'xcons_image'}){
			return undef;
		}
	}
	my $cmd = '"'.$self->{'xcons_image'}.'" '.
				$self->{'dbname'}.' SHOW STATE';
	my $out;
	my $rc = callsystem ($cmd,sub {($out) = @_; 1;},1);
	
	if (not defined $rc || $rc != 0){
		$self->{'last_error'} = 'command \''.$cmd.
			'\' ended with error code '.$rc.": \n".$out;
		return undef;
	}
	my $state;
	
	foreach my $line (split("\n",$out)){
		if ($line =~ /Server\sDB\s.*\shas\snot\sbeen\sstarted/){
			$state = 'OFFLINE';
			last;
		}
		($state) = ($line =~ /state\s+is\s+(\S+)/i);
		last if $state =~ /\S/; 
	}
	unless ($state =~ /\S/){
		$self->{'last_error'} = 'xcons out: '.$out;
		return undef; 
	}
	$self->{'state'} = $state;
	return $state;	
}

sub db_reg{
	my ($self,$dbroot) = @_;
	
	unless ($dbroot =~ /\S/){
		$self->{'last_error'} = 'dependent path is unknown';
		return undef;
	}
	
	unless (-d $dbroot){
		$self->{'last_error'} = 'dependent path "'.$dbroot.'" doesn\'t exist';
		return undef;
	}
		
	my $dbmsrv = "$dbroot/pgm/dbmsrv";
	$dbmsrv .= '.exe' if $^O =~ /mswin/i;
	
	unless (-f $dbmsrv){
		$self->{'last_error'} = '"'.$dbroot.'" is no valid instroot: dbmsrv not found';
		return undef;
	}
	
	
	if ($self->{'connected'}){
		return $self->{'dbm'}->db_reg($dbroot);
	}
	
	my ($indep_data,$indep_prog) = readIndepPath();
	
	unless (-f $indep_data.'/config/'.$self->{'dbname'}){
		$self->{last_error} = 'database paramfile not found';
		return undef;
	} 
		
	my %dbs = getInstances();
	
	if (exists $dbs{$self->{'dbname'}}){
		my $depend = $dbs{$self->{'dbname'}};
		if (normalizePath ($depend) eq normalizePath ($dbroot)){
			return 1;
		}
		$self->{last_error} = 'database is already registered'.
							 ' with other dependent path ('.$depend.')';
		return undef;
	}
	
	my %kernels = getInstalledReleases();
	
	my $found = 0;
	foreach my $depend (keys(%kernels)){
		if (normalizePath ($depend) eq normalizePath ($dbroot)){
			$found = 1;
			last;
		}
	}
	
	unless ($found){
		$self->{last_error} = "dependent path \"$dbroot\" ".
							"isn\'t registered";
 		return undef;
	}
	
	$self->reset_dbroot();
	
	$self->{'dbroot'} = $dbroot;
		
	unless (defined $self->{'dbmreg_image'}){
		unless ($self->get_prog_image('dbmreg')){
			if ($self->{last_error} =~ /^cannot\sfind/){
				$self->{last_error} = 'database version doesn\'t support this feature';
				return undef;	
			}
			
			$self->{'last_error'} = 'cannot find program dbmreg: '.$self->{'last_error'};
			return undef;
		}
	}
	
	my $dbmreg = $self->{'dbmreg_image'};
	
			
	$dbmreg =~ s/\//\\/g if $^O =~ /mswin/i;
	
	my $cmd = '"'.$dbmreg.'" '.$self->{'dbname'};
	
	my $out;
	my $rc = callsystem ($cmd,sub {($out) = @_; 1;},1);
	
	if (not defined $rc || $rc != 0){
		$self->{'last_error'} = 'command \''.$cmd.
					\' ended with error code '.$rc.": \n".$out;
		return undef;
	}
	my $ok = 0;	
	foreach my $line (split("\n",$out)){
		if ($line =~ /^OK/){
			$ok = 1;
			last;
		}
	}
	unless($ok){
		$self->{'last_error'} = $out;
		return undef;
	}
	$self->{'registered'} = 1;
	return 1;
}

sub db_unreg{
	my ($self) = @_;
	if ($self->{'connected'}){
		return $self->{'dbm'}->db_unreg();
	}
	my $dbmunreg;
	
	unless (defined $self->{'dbmunreg_image'}){
		
		unless ($self->get_prog_image('dbmunreg')){
			
			if ($self->{last_error} =~ /^cannot\sfind/){
				$self->{last_error} = 'database version doesn\'t support this feature';
				return undef;	
			}
			
			$self->{'last_error'} = 'cannot find program dbmunreg: '.$self->{'last_error'};
			return undef;
		}
	}
	
	$dbmunreg = $self->{'dbmunreg_image'};
	
	if ($^O =~ /mswin/i){
		$dbmunreg =~ s/\//\\/g;
	}
	
	
	my $cmd = '"'.$dbmunreg.
			'" '.$self->{'dbname'};
	
	my $out;
	my $rc = callsystem ($cmd,sub {($out) = @_; 1;},1);
	
	$self->reset_dbroot();
	
	if (not defined $rc || $rc != 0){
		$self->{'last_error'} = 'command \''.$cmd.
			'\' ended with error code '.$rc.": \n".$out;
		return undef;
	}
	my $ok = 0;	
	foreach my $line (split("\n",$out)){
		if ($line =~ /^OK/){
			$ok = 1;
			last;
		}
	}
	unless($ok){
		$self->{'last_error'} = $out;
		return undef;
	}
	return 1;
}


sub DESTROY{
	my ($self) = @_;
	$self->{'dbm'} = undef;
}

sub AUTOLOAD{
	my $self = shift;
	my $method = $AUTOLOAD;
	$method =~ s/.*://;
	unless (defined $self->{'dbm'} && 
			$self->{'dbm'}->can($method)){
		
		die ("method \"$method\" isn\'t defined in class ".
			__PACKAGE__."\n");
	}
	return $self->{'dbm'}->{$method}(@_);
}

1;