#!/usr/bin/perl

package SAPDB::Install::Templates::Base;

#ifdef SCRIPTING
#use strict;
#use SAPDB::Install::DBMCmd;
#use SAPDB::Install::BackupPipe;
#use SAPDB::Install::Untgz;
#use SAPDB::Install::InstInfo;
#use SAPDB::Install::Registry;
#use SAPDB::Install::System;
#use SAPDB::Install::Tools;
#use SAPDB::Install::MachInfo;
#use SAPDB::Install::Templates::Util;
#use SAPDB::Install::FSUsage;
#require Exporter;
#our @ISA = qw( Exporter );
#our @EXPORT = qw( createThis 
#                  getGeneric
#                  mymkdir 
#                );
#endif

#because SDBINST

require SDB::Common::Exporter;

our @ISA = (SDB::Common::Exporter);
our @EXPORT = qw (createThis
                  getGeneric
                  mymkdir);

use SAPDB::Install::DBMCmd;
use SAPDB::Install::BackupPipe;
use SAPDB::Install::Untgz;
use SAPDB::Install::StdIO;
use SAPDB::Install::InstInfo;
use SAPDB::Install::Registry;
use SAPDB::Install::Values;
use SAPDB::Install::System;
use SAPDB::Install::MachInfo;
use SAPDB::Install::Templates::Util;
use SAPDB::Install::Tools;
use SAPDB::Install::FSUsage;

#endbecause



#ifdef doc
#<h1>SAPDB::Install::Templates::Base</h1>
#  <p>
#  Base class for MaxDB database instances.
#  Configures, creates and deletes MaxDB Database Instances.
#  (Uses  <code>SAPDB::Install::DBMCmd</code>)
#  </p>
#  <h2>Usage</h2>
#  <pre><![CDATA[my $testData = SAPDB::Install::Templates::Base::new();
#                $testData->set...();
#                $testData->createInstance();
#               ]]></pre>
#  <h2>Details</h2>
#  <p>The module basically contains:</p>
#  <ol>
#  <li>
#  A straightforward data model in PERL for MaxDB database instance configuration data.
#  </li>
#  <li>
#  The PERL code to create and delete such database instances.
#  </li>
#  </ol>
#  <p>
#  As such the module  is meant to encapsulate logic and data for  database instance creation.
#  It is supposed to be configured and used in the following szenarios:
#</p>
#  <ol>
#  <li>
#  Be configured from user input entered into a GUI: The method <code>createInstance()</code> could
#  then be the action behind the <code>CREATE</code>-Button.
#  </li>
#  <li>
#  Be configured from an <code>install template</code>
#  </li>
#  <li>
#  Be configured and used by other PERL Modules.
#  </li>
#  </ol>
#  <h2>Configuration</h2>
#  <p>
#  Instances can be configured with the following parameters to
#  be used for the creation of a SAPDB-Instance:
#</p>
  #endif





  # Instance-Variabless
  #+field(String:Database Name)
  my $sdb_name= "sdb_name";

#+field(String:Name of DBM User)
my $sdb_dbmuser= "sdb_dbmuser";

#+field(String:Password of DBM User)
my $sdb_dbmpass= "sdb_dbmpass";

#+field(String:Name of DBA User)
my $sdb_dbauser= "sdb_dbauser";

#+field(String:Password of DBA User)
my $sdb_dbapass= "sdb_dbapass";

#+field(String:Name of  SQL User)
my $sdb_sqluser= "sdb_sqluser";

#+field(String:Password of  SQL User)
my $sdb_sqlpass= "sdb_sqlpass";

#+field(String:Password of  Domain User)
my $sdb_domainpass= "sdb_domainpass";

#+field(HashRef:Database Parameters)
my $sdb_params= "sdb_params";

#+field(ListRefOfListRefs:Database Volumes)
my $sdb_volumes= "sdb_volumes";

#+field(FuncRef:Pointer To Function for Logging)
my $outputter= "outputter";

#+field(FuncRef:Pointer to Function for Status Messages)
my $statusputter= "statusputter";



#+field(boolean:NEEDED "Depend"-Path)
my $dependend_path= "dependend_path";

#+field(String:Declaration for a Medium)
my $mediumDeclaration= "mediumDeclaration";

#+field(String:Named pipe for Recover)
my $recoverFromPipe= "recoverFromPipe";

#+field(Path:Path for installation packages)
my $archiveDir='archiveDir';

#+field(Type:Type of the Template)
my $typeKey='typeKey';


#+field(HashRef:Generic parameters)
my $generic='genericKey';



#+field(FuncRef:Reference to function for calling dbmcli-commands)
my $scriptHandler='scriptHandler';


#+sub():Hash:Get all generics
sub getAllGenerics($){
  my ($this) = $_[0];
  return $this->{$generic};
}

#ifdef sub:addGeneric
#<p>Type is one of</p>
#  <ul>
#  <li><code>"Path"</code></li>
#  <li><code>"UserPassword"</code></li>
#  <li><code>"Number"</code></li>
#  <li><code>"Storage"</code></li>
#  <li><code>"String"</code></li>
#  <li><code>"Boolean"</code></li>
#  </ul>
#  <p>Format: "<code>!&lt;TYPE&gt;: &lt;KEY&gt;  &lt;VALUE&gt;</code>", beispielsweise</p>
#  <pre>
#  !String: SLOPPY yes,please
#  </pre>
  #endif

  #+sub(String:Key|String:Value|Type:type of the value):void:Add a generic parameter
  sub addGeneric($$$$){
    my ($this,$key,$value,$type) = @_;
    my $genericHash = $this->getAllGenerics();
    $genericHash->{$key}=[$value,$type];
  }

#+sub(String:key):List(value,type):Get the generic specified by key
sub getGeneric($$){
  my ($this,$key) = @_;
  my $genericHash = $this->getAllGenerics();
  return  $genericHash->{$key};
}


#+sub(String:key):value:Get the generic value specified by key
sub getGenericValue($$){
  my ($this,$key) = @_;
  my $aListRef = $this->getGeneric($key);
  my $aValue=undef;
  if ( defined $aListRef) {
    $aValue = $aListRef->[0];
  }
  return  $aValue;
}

#+sub(String:key):value:Get the type of a generic  value specified by key
sub getGenericType($$){
  my ($this,$key) = @_;
  my $aListRef = $this->getGeneric($key);
  my $aValue=undef;
  if ( defined $aListRef) {
    $aValue = $aListRef->[1];
  }
  return  $aValue;
}







#GETTERS



#+field(String:globalPassword)
my $globalPassword="globalPasswordKey";
#+sub(String):void:Set globalPassword
sub setGlobalPassword{
  my ($this, $aValue) = @_;
  $this->{$globalPassword}=$aValue;
}
#+sub():String:Get globalPassword
sub getGlobalPassword{
  my ($this ) = @_;
  return $this->{$globalPassword};
}


sub setAdvicedGlobalPassword{
  my ($this, $aValue) = @_;
  willSet($this,$globalPassword);
  $this->setGlobalPassword($aValue);
  haveSet($this,$globalPassword);
}





#+field(String:confirmGlobalPassword)
my $confirmGlobalPassword="confirmGlobalPasswordKey";
#+sub(String):void:Set confirmGlobalPassword
sub setConfirmGlobalPassword{
  my ($this, $aValue) = @_;
  $this->{$confirmGlobalPassword}=$aValue;
}
#+sub():String:Get confirmGlobalPassword
sub getConfirmGlobalPassword{
  my ($this ) = @_;
  return $this->{$confirmGlobalPassword};
}

sub setAdvicedConfirmGlobalPassword{
  my ($this, $aValue) = @_;
  willSet($this,$confirmGlobalPassword);
  $this->setConfirmGlobalPassword($aValue);
  haveSet($this,$confirmGlobalPassword);
}

#+field(String:ConfirmDBMPass)
my $ConfirmDBMPass="ConfirmDBMPassKey";
#+sub(String):void:Set ConfirmDBMPass
sub setConfirmDBMPass{
  my ($this, $aValue) = @_;
  $this->{$ConfirmDBMPass}=$aValue;
}
#+sub():String:Get ConfirmDBMPass
sub getConfirmDBMPass{
  my ($this ) = @_;
  return $this->{$ConfirmDBMPass};
}

#+field(String:confirmDBAPass)
my $confirmDBAPass="confirmDBAPassKey";
#+sub(String):void:Set confirmDBAPass
sub setConfirmDBAPass{
  my ($this, $aValue) = @_;
  $this->{$confirmDBAPass}=$aValue;
}
#+sub():String:Get confirmDBAPass
sub getConfirmDBAPass{
  my ($this ) = @_;
  return $this->{$confirmDBAPass};
}






#+field(String:globalUser)
my $globalUser="globalUserKey";
#+sub(String):void:Set globalUser
sub setGlobalUser{
  my ($this, $aValue) = @_;
  $this->{$globalUser}=$aValue;
}
#+sub():String:Get globalUser
sub getGlobalUser{
  my ($this ) = @_;
  return $this->{$globalUser};
}

sub setAdvicedGlobalUser{
  my ($this, $aValue) = @_;
  willSet($this,$globalUser);
  $this->setGlobalUser($aValue);
  haveSet($this,$globalUser);
}





#+sub():String:Get the type of the template
sub getType($){ return $_[0]{$typeKey}; }

sub setType($$){ $_[0]{$typeKey}=$_[1]; }




#+field(Datasize:DBSize)
my $DBSize="DBSizeKey";
#+sub(Datasize):void:Set DBSize
sub setDBSize{
  my ($this, $aValue) = @_;
  $this->{$DBSize}=$aValue;
}
#+sub():Datasize:Get DBSize
sub getDBSize{
  my ($this ) = @_;
  return $this->{$DBSize};
}

sub setAdvicedDBSize{
  my ($this, $aValue) = @_;
  willSet($this,$DBSize);
  $this->setDBSize($aValue);
  haveSet($this,$DBSize);
}

sub validateDBSize{
  my ($this,$aSize) = @_;
  my @retVal = (1);
  my $volumes = $this->cloneDBVolumes("DATA");
  my $fsresult = $this->checkVolumeSizeOnVolumeArray($volumes);
  my $aSizekb = $aSize * 8;
  if ( $aSizekb < (64 * 1024)) {
    @retVal = (0,"Database size is too small","Database size of less than 64 MB is not possible");
    return @retVal;
  }
  foreach my $key (keys %$fsresult) {
    my $fsdata  = $fsresult->{$key}; 
    my $neededkb = $fsdata->{"needed"};
    my $availablekb = $fsdata->{"available"};
    my $totalkb = $fsdata->{"total"};
    if ($availablekb  < $aSizekb) {
      my ($needed,$neededUnit)=readableUnit($neededkb,"KB"); 
      my ($available,$availableUnit)=readableUnit($availablekb,"KB"); 
      my ($total,$totalUnit)=readableUnit($totalkb,"KB"); 
      my $advice =  "Not enough space on \"$key\" : (Available: $available $availableUnit, " .
        "Needed $needed $neededUnit)\nProvide enought disk space";
      my $message = "Insufficient space left on device";
      @retVal = (0,$message,$advice);
      last;
    }
  }
  return @retVal;
}



#+field(Datasize:DBMemory)
my $DBMemory="DBMemoryKey";
#+sub(Datasize):void:Set DBMemory
sub setDBMemory{
  my ($this, $aValue) = @_;
  $this->{$DBMemory}=$aValue;
}

sub setAdvicedDBMemory{
  my ($this, $aValue) = @_;
  willSet($this,$DBMemory);
  $this->setDBMemory($aValue);
  haveSet($this,$DBMemory);
}




#+sub():Datasize:Get DBMemory
sub getDBMemory{
  my ($this ) = @_;
  return $this->{$DBMemory};
}

#+field(int:CPUCount)
my $CPUCount="CPUCountKey";
#+sub(int):void:Set CPUCount
sub setCPUCount{
  my ($this, $aValue) = @_;
  $this->{$CPUCount}=$aValue;
}
#+sub():int:Get CPUCount
sub getCPUCount{
  my ($this ) = @_;
  return $this->{$CPUCount};
}


sub setAdvicedCPUCount{
  my ($this, $aValue) = @_;
  willSet($this,$CPUCount);
  $this->setCPUCount($aValue);
  haveSet($this,$CPUCount);
}


#ifdef sub:setRecoverFromPipe
#endif
#+sub(String:Medium Declaration):void:Set the pipe to recover from
sub setRecoverFromPipe(){
  my ($this,$pipeName)=@_;
  $this->{ $recoverFromPipe}=$pipeName;
  return $pipeName;
}
#setRecoverFromPipe()




#+field(seconds:AsyncWaitTime)
my $AsyncWaitTime="AsyncWaitTimeKey";
#+sub(seconds):void:Set AsyncWaitTime
sub setAsyncWaitTime{
  my ($this, $aValue) = @_;
  $this->{$AsyncWaitTime}=$aValue;
}
#+sub():seconds:Get AsyncWaitTime
sub getAsyncWaitTime{
  my ($this ) = @_;
  return $this->{$AsyncWaitTime};
}









#Legt ein Verzeichnis an, und wenn ntig die darber befindlichen Unterverzeichnisse.
sub mymkdir($){
  if ($^O =~ /mswin/i) {
    $_[0] =~ /^(\w):\\(.*)$/;
    my $drive = $1;
    my $path = $2;

    unless ( (defined $path) && ($path =~ m/\S+/) ){
      return "Invalid path specified.";
    }
    my @aSplit = reverse (split /\\/,$path);
    my $aSub = (defined $drive)? ($drive . ":\\" .  pop @aSplit) : (pop @aSplit) ;
    while ( 1) {
      unless( -d $aSub){
        mkdir($aSub,0777);
      }
      unless (-d $aSub){
        return "Could not create directory $aSub";
      }
      my $atom = pop @aSplit;
      unless ($atom){
        last ;
      }
      $aSub = $aSub . "\\" . $atom;
    }
  } else {
    #Momentan nur Windows implementiert.
  }
  return 0;
}
#mkdir()

#+sub():String:Get the named pipe to recover from
sub getRecoverFromPipe(){
  my ($this)=@_;
  return $this->{ $recoverFromPipe};
}

#+sub():String:Get the Medium Declaration.
sub getMediumDeclaration(){
  my ($this) = @_;
  my $aMediumDeclaration = $this->{$mediumDeclaration};
  my $aResult ; 
  my $aBackup = $this->getBackup();
  my $anArchive = $this->getArchive();
  my $pipeName = $this->getRecoverFromPipe();
  if (defined $aMediumDeclaration) {
    $aResult = $aMediumDeclaration;
  } elsif ((defined $aBackup) and (not defined $anArchive)) {
    $aResult = "SDBINSTBACKUP " . $aBackup . " FILE DATA 0 8 YES";
  } elsif ((defined $aBackup) and (defined $anArchive)) {

    #Datei fuer Pipe aussuchen
    #   my $temp = $ENV{"TMP"};
    #   unless (defined $temp){
    #     $temp = $ENV{"TEMP"};
    #   }
    #   unless (defined $temp){
    #     $this->writeLog ("Temporary directory not set\n");
    #     return undef;
    #   }
    #   my $separator =  ($^O =~ /mswin/i)?"\\":"/";
    #   my $counter =1;
    #   my $pipeFile= $temp . $separator . "sdbinstpipe" . $counter;
    #   while (-e $pipeFile) {
    #     $counter ++;
    #     $pipeFile =  $temp . $separator . "sdbinstpipe" . $counter;
    #   }

    #Windows-Fall
    $aResult =  "$pipeName \\\\.\\pipe\\$pipeName PIPE DATA 0 0 YES"; 
  }
  return $aResult;
}
#getMediumDeclaration()


my $archive="archive";
#ifdef sub:setArchive
#<p>Set the archive to be used to read the backup from for db_activate RECOVER</p>
  #endif
  #+sub(String:An Archive):void:Set the archive
  sub setArchive(){ 
    my ($this,$anArchive)=  @_;
    $this->{$archive}=$anArchive; 
    if (defined $anArchive) {
      $this->setRecoverFromPipe("SDBINSTPIPE");
    } else {
      $this->setRecoverFromPipe(undef);
    }
  }

#+sub():String:Get the Archive.
sub getArchive(){ $_[0]{$archive}; }

#+field(Untgz:ArchiveUntgz)
my $ArchiveUntgz="ArchiveUntgzKey";
#+sub(Untgz):void:Set ArchiveUntgz
sub setArchiveUntgz{
  my ($this, $aValue) = @_;
  $this->{$ArchiveUntgz}=$aValue;
  $this->setArchive($aValue);
}
#+sub():Untgz:Get ArchiveUntgz
sub getArchiveUntgz{
  my ($this ) = @_;
  return $this->{$ArchiveUntgz};
}

my $backup="backup";
#ifdef sub:setBackup
#<p>Set the backup to be used  for db_activate RECOVER ...</p>
  #endif
  #+sub(String:A Backup):void:Set the backup
  sub setBackup(){
    my ($this,$aBackup)=  @_;
    $this->{$backup}=$aBackup; 
  }
#setBackup()

#+sub():String:Get the backup
sub getBackup(){ 
  my ($this)=  @_;
  return $this->{$backup}; 
}
#getBackup()




#+field(boolean:On/Off Auto-Extend)
my $autoextend= "isAutoExtendKey";

#+sub(bool:autoext):bool:Set On/Off autoextend feature on/off
sub setAutoExtend(){ $_[0]{$autoextend}=$_[1]; }

#+sub():bool:Is autoextend on/off
sub getAutoExtend(){ $_[0]{$autoextend}; }

sub setAdvicedAutoExtend{
  my ($this, $aValue) = @_;
  willSet($this,$autoextend);
  $this->setAutoExtend($aValue);
  haveSet($this,$autoextend);
}




#+field(boolean:On/Off Log Auto Overwrite)
my $autoOverWriteLog= "autoOverWriteLogKey";


#+sub(bool:autoext):bool:Set auto log overwrite on/off
sub setAutoOverWriteLog(){ $_[0]{$autoOverWriteLog}=$_[1]; }

#+sub():bool:Is auto log overwrite on/off
sub getAutoOverWriteLog(){ $_[0]{$autoOverWriteLog}; }



sub setAdvicedAutoOverWriteLog{
  my ($this, $aValue) = @_;
  willSet($this,$autoOverWriteLog);
  $this->setAutoOverWriteLog($aValue);
  haveSet($this,$autoOverWriteLog);
}



#+sub():String:Get the name of the Database
sub getDBName(){ $_[0]{$sdb_name}; }
#+sub(String:name):void:Set the name of the Database
sub setDBName(){ $_[0]{$sdb_name}=$_[1]; }



sub setAdvicedDBName{
  my ($this, $aValue) = @_;
  willSet($this,$sdb_name);
  $this->setDBName($aValue);
  haveSet($this,$sdb_name);
}




#+sub():String:Get database manager
sub getDBM(){ $_[0]{$sdb_dbmuser}; }

#+sub(String:name):void:Set database manager 
sub setDBM(){ $_[0]{$sdb_dbmuser}=$_[1]; }

#+sub():String:Get database manager password
sub getDBMPass(){ $_[0]{$sdb_dbmpass}; }
#+sub(String:password):void:Set database manager password
sub setDBMPass(){ $_[0]{$sdb_dbmpass}=$_[1]; }
#+sub():String:Get name of Database Administrator
sub getDBA(){ $_[0]{$sdb_dbauser}; }
#+sub(String:name):void:Set name of Database Administrator
sub setDBA(){ $_[0]{$sdb_dbauser}=$_[1]; }
#+sub():String:Get password of Database Administrator
sub getDBAPass(){ $_[0]{$sdb_dbapass}; }
#+sub(String:password):void:Set name of Database Administrator
sub setDBAPass(){
  $_[0]{$sdb_dbapass}=$_[1];
  #  $_[0]{$sdb_domainpass}=$_[1];
}
#+sub():String:Get password of Domain user
sub getDomainPass(){ $_[0]{$sdb_domainpass}; }
#+sub(String:password):void:Set password of Domain user
sub setDomainPass(){
  $_[0]{$sdb_domainpass}=$_[1]; 
}
#+sub():String:Get name of SQL-User
sub getSQLU(){ $_[0]{$sdb_sqluser}; }
#+sub(String:name):void:Set name of SQL-User
sub setSQLU(){ $_[0]{$sdb_sqluser}=$_[1]; }
#+sub():String:Get password of SQL-User
sub getSQLPass(){ $_[0]{$sdb_sqlpass}; }
#+sub(String:password):void:Set password of SQL-User
sub setSQLPass(){ $_[0]{$sdb_sqlpass}=$_[1]; }


#+field(String:recoveryMode)
my $recoveryMode="recoveryModeKey";
#+sub(String):void:Set recoveryMode
sub setRecoveryMode{
  my ($this, $aValue) = @_;
  $this->{$recoveryMode}=$aValue;
  
  if ($aValue =~ /Simple/i) {
    $this->setAutoOverWriteLog("on");
  } else {
    $this->setAutoOverWriteLog("off");
  }
}
#+sub():String:Get recoveryMode
sub getRecoveryMode{
  my ($this ) = @_;
  
  return $this->{$recoveryMode};
}




#+field(String:TemplateCode)
my $TemplateCode="TemplateCodeKey";
#+sub(String):void:Set TemplateCode
sub setTemplateCode{
  my ($this, $aValue) = @_;
  $this->{$TemplateCode}=$aValue;
}
#+sub():String:Get TemplateCode
sub getTemplateCode{
  my ($this ) = @_;
  return $this->{$TemplateCode};
}


sub setAdvicedTemplateCode{
  my ($this, $aValue) = @_;
  willSet($this,$TemplateCode);
  $this->setTemplateCode($aValue);
  haveSet($this,$TemplateCode);
}



#+field(Drive:DataDrive)
my $DataDrive="DataDriveKey";
#+sub(Drive):void:Set DataDrive
sub setDataDrive{
  my ($this, $aValue) = @_;
  my $goodDrive = matchDrive($aValue);
  $this->{$DataDrive}=$goodDrive;
}
#+sub():Drive:Get DataDrive
sub getDataDrive{
  my ($this ) = @_;
  return $this->{$DataDrive};
}
sub setAdvicedDataDrive{
  my ($this, $aValue) = @_;
  willSet($this,$DataDrive);
  $this->setDataDrive($aValue);
  haveSet($this,$DataDrive);
}


#+field(Drive:LogDrive)
my $LogDrive="LogDriveKey";
#+sub(Drive):void:Set LogDrive
sub setLogDrive{
  my ($this, $aValue) = @_;
  my $goodDrive = matchDrive($aValue);
  $this->{$LogDrive}=$goodDrive;
}
#+sub():Drive:Get LogDrive
sub getLogDrive{
  my ($this ) = @_;
  return $this->{$LogDrive};
}

sub setAdvicedLogDrive{
  my ($this, $aValue) = @_;
  willSet($this,$LogDrive);
  $this->setLogDrive($aValue);
  haveSet($this,$LogDrive);
}

#ifdef sub:setCreationStrategy
#<p>Set strategy for instance creation. Strategies are:</p>
#  <ul>
#  <li><strong>"1": </strong>Do not check database existance, just create</li>
#  <li><strong>"2": </strong>Fail if database exists: 
#<ul class="c3">
#  <li>Fail if goodness &gt; <strong>1</strong> or <strong>0</strong></li>
#  </ul>
#  </li>
#  <li><strong>"3": </strong>Delete database, if it exists:
#<ul class="c3">
#  <li> Create the instance  if goodness is <strong>1</strong> </li>
#  <li> Delete and recreate it, if   goodness &gt;  <strong>3</strong>.</li>
#  <li> Fail if goodness is <strong>2,3</strong> or <strong>0</strong> </li>
#  </ul>
#  </li>
#  <li><strong>"4": </strong> Try to reuse the database:
#<ul class="c3">
#  <li> Create the instance  if goodness is <strong>1</strong> </li>
#  <li> Delete and recreate it, if     <strong>4</strong> or  <strong>5</strong>.</li>
#  <li> Try to reuse the instance, if its goodness is &gt;= <strong>6</strong>. </li>
#  <li> Fail if goodness is <strong>2,3</strong> or <strong>0</strong> </li>
#  </ul>
#  </li>
#  </ul>
  #endif
  #+field(int:creationStrategy)
  my $creationStrategy="creationStrategyKey";
#+sub(int):void:Set creationStrategy
sub setCreationStrategy{
  my ($this, $aValue) = @_;
  $this->{$creationStrategy}=$aValue;
}
#+sub():int:Get creationStrategy
sub getCreationStrategy{
  my ($this ) = @_;
  return $this->{$creationStrategy};
}

#ifdef sub:setDeletionStrategy
#<p>Set strategy for instance deletion. Strategies are:</p>
#  <ul>
#  <li><strong>"1": </strong>delete without files</li>
#  <li><strong>"2": </strong>delete with files</li>
#  </ul>
  #endif

  #+field(int:deletionStrategy)
  my $deletionStrategy="deletionStrategyKey";
#+sub(int):void:Set deletionStrategy
sub setDeletionStrategy{
  my ($this, $aValue) = @_;
  $this->{$deletionStrategy}=$aValue;
}
#+sub():int:Get deletionStrategy
sub getDeletionStrategy{
  my ($this ) = @_;
  return $this->{$deletionStrategy};
}



sub writeConfig($$){
  my ($this,$write) = @_;
  my $paramsRef = $this->getParams();
  my %allParams = %$paramsRef;

  &$write("\n");
  &$write("[DBInstance]\n");
  &$write("TYPE " . $this->getType() . "\n");

  &$write("#Users and passwords\n");
  if ($this->getDebug()) {
    &$write("DBM  " . $this->getDBM() . "," . $this->getDBMPass() .  "\n");
    &$write("DBA  " . $this->getDBA() . "," . $this->getDBAPass() .  "\n");
  } else {
    &$write("DBM  " . $this->getDBM() . ",***\n");
    &$write("DBA  " . $this->getDBA() . ",***\n");
  }

  my $sqlUser = $this->getSQLU();
  if (defined $sqlUser) {
    if ($this->getDebug()) {
      &$write("SQLUSER  " . $this->getSQLU() . "," . $this->getSQLPass() .  "\n");
    } else {
      &$write("SQLUSER  " . $this->getSQLU() . ",***\n");
    }
  }

  defaultWriteParams(\%allParams,"DBInstance",$write);





  &$write("\n");
  &$write("#Database parameters:\n");
  my %dbparams = $this->getDBParams();
  while ((my $key,my $value) = each %dbparams) {
    &$write("DBPARAM $key   $value\n");
  }
  &$write("\n");

  &$write("#Database volumes:\n");
  &$write("#Format: <NUM>;<ART>;<PATH>;<TYPE>;<SIZE>\n");
  my $vols = $this->getDBVolumes();
  foreach my $vol (@$vols) {
    &$write("DBVOLUME " .  join(";",@$vol) . "\n");
  }
  &$write("\n");


  &$write("[Generic]\n");
  my $generics = $this->getAllGenerics();
  while ((my $key,my $value) = each %{$generics}) {
    my $pair = $this->getGeneric($key);
    &$write("!$pair->[1]: $key $pair->[0]\n");
  }
  &$write("\n");
  &$write("################\n");
  &$write("#GENERATED DATA\n");
  &$write("################\n");
  &$write("\n");
  my $fullDataSize = $this->getFullDataVolumeSize();
  &$write("#Complete datasize: " . $fullDataSize . " pages, (" . $fullDataSize * 8 / 1024 . " MB)\n");
  my $fullLogSize = $this->getFullLogVolumeSize();
  &$write("#Complete log size: " . $fullLogSize . " pages, (" . $fullLogSize * 8 /1024 . " MB)\n");
  my $pipeName = $this->getRecoverFromPipe();
  if ( defined $pipeName) {
    &$write("#Recover from pipe: " . $pipeName . "\n");
  }
  my $mediumDeclaration = $this->getMediumDeclaration();
  if ( defined $mediumDeclaration) {
    &$write("#Medium declaration: \"" . $mediumDeclaration . "\"\n");
  }
  my $backup = $this->getBackup();
  if ( defined $backup) {
    &$write("#Backup: \"" . $backup . "\"\n");
  }
  my $archive = $this->getArchive();
  if ( defined $archive) {
    &$write("#Archive: \"" . $archive . "\"\n");
  }
  
  &$write("#END OF CONFIGURATION DATA FOR MAXDB INSTANCE CREATION ....\n");
  &$write("#############################################################\n");
}
#end writeConfig();











#ifdef sub:isRegistered
#<p>Uses <code>SAPDB::Install::Registry</code> to get info if database is registered
#  Return values are </p>
#  <ul>
#  <li><code>0</code>: Database not found</li>
#  <li><code>1</code>: Database found</li>
#  </ul>
  #endif

  #+sub():int:Is the instance registered (Windows: Windows Registry, Unix: DB Registry)
  sub isRegistered{
    my ($this ) = @_;
    my $aDBName =$this->getDBName();
    my $retVal = 0;
    $this->addGeneric("registeredString", "Registration of  database $aDBName not found","String");
    my %instance = getInstances();
    if ( defined $instance{$aDBName}) {
      $retVal=1;
      $this->addGeneric("registeredString", "Database $aDBName is registered","String");
    }
    return  $retVal;
  }

#ifdef sub:getGoodness
#<p>Uses <code>SAPDB::Install::InstInfo</code> to get info about the database.
#  Return values are </p>
#  <ul>
#  <li><code>0</code>: General Info not available </li>
#  <li><code>1</code>: Database info not available (Database seems not to exist)</li>
#  <li><code>2</code>: db software path does not match</li>
#  <li><code>3</code>: Could not  log on  as dbm</li>
#  <li><code>4</code>: Not enough data size, or info about data size not available </li>
#  <li><code>5</code>: Not enough log size, or info about log size not available</li>
#  <li><code>6</code>: db is pretty good</li>
#  </ul>
  #endif

  #+sub():int:Get the goodness of the database
  sub getGoodness{
    my ($this ) = @_;
    my $aDBName =$this->getDBName();
    my $retVal = 0;
    if (defined $this->getScriptHandler()) {
      $retVal = 1;
      $this->addGeneric("goodnessString", "Goodness of DB not tested. This is a test run only","String");
      return $retVal;
    }
    $this->addGeneric("goodnessString", "General db info  not available, numeric goodness is $retVal","String");
    my $aDependendPath = $this->getDependendPath();
    my $info = SAPDB::Install::InstInfo::new ($aDBName);
    if ( defined $info) {
      $retVal = 1;
      $this->addGeneric("goodnessString", "General db info available","String");
    } else {
      return $retVal;
    }

    my $infoListRef =  $info->{'database'};
    my $dbInfo = $infoListRef->{$aDBName};
    if ( defined $dbInfo) {
      $retVal = 2;
      $this->addGeneric("goodnessString", "Info  on $aDBName available","String");
    } else {
      $this->addGeneric("goodnessString", "Info  on $aDBName unavailable","String");
      return $retVal;
    }
    
    #      my $realDBName = $dbInfo->{'dbname'};
    #      my $dbowner = $dbInfo->{'dbowner'};
    #      my $paramfile = $dbInfo->{'paramfile'};
    my $dbswdir = $dbInfo->{'dbswdir'};
    my $path1= $dbswdir;
    my $path2= $aDependendPath;
    $path1 =~ s/\\/\//g;
    $path2 =~ s/\\/\//g;
    $path1 =~ s/\/$//g;
    $path2 =~ s/\/$//g;
    if ( $^O =~ /mswin/i ) {
      $path1 = uc $path1;
      $path2 = uc $path2;
    }
    if ( $path1 eq $path2) {
      $retVal=3;
      $this->addGeneric("goodnessString", "DB Software path matches for $aDBName, numeric goodness is $retVal)","String");
    } else {
      $this->addGeneric("goodnessString", 
                        "Database $aDBName found, but db software path is \"$dbswdir\" " . 
                        "and not \"$aDependendPath\" (Goodness $retVal)",
                        "String");
      return $retVal;
    }


    my $dbm = $this->createDBMCmd();
    if (defined $this->logonAsDBM($dbm)) {
      $retVal=4;
      $this->addGeneric("goodnessString", "Logged on as database manager to  $aDBName, numeric goodness is $retVal.","String");
    } else {
      return $retVal;
    }
    my $fullDataSize = $this->getFullDataVolumeSize();
    my $realFullDataSize = $this->getRealFullDataVolumeSize($dbm);
    if ( defined $realFullDataSize and $realFullDataSize >= $fullDataSize) {
      $retVal=5;
      $this->addGeneric("goodnessString", 
                        "$aDBName: Sufficient data size " . 
                        "(Have $realFullDataSize, need $fullDataSize, numeric goodness is $retVal]","String");
    } else {
      my $sizeInfo = $realFullDataSize;
      unless(defined $realFullDataSize){
        $sizeInfo = "not found info about data size";
      }
      $this->addGeneric("goodnessString", 
                        "$aDBName: Insufficient data size " . 
                        "[Have $sizeInfo, need $fullDataSize  pages, numeric goodness is $retVal]","String");
      return $retVal;
    }

    my $fullLogSize = $this->getFullLogVolumeSize();
    my $realFullLogSize = $this->getRealFullLogVolumeSize($dbm);
    if ( defined $realFullLogSize and $realFullLogSize >= $fullLogSize) {
      $retVal=6;
      $this->addGeneric("goodnessString", 
                        "$aDBName is pretty good" . 
                        ", [Numeric goodness is $retVal]","String");
    } else {
      my $sizeInfo = $realFullLogSize;
      unless(defined $realFullLogSize){
        $sizeInfo = "not found info about log size";
      }
      $this->addGeneric("goodnessString", 
                        "$aDBName: Insufficient log size " . 
                        "[Have $sizeInfo, need $fullLogSize  pages, numeric goodness is $retVal]","String");
      return $retVal;
    }

    return $retVal;
  }
#getGoodness

sub getGoodnessString{
  my ($this ) = @_;
  return $this->getGenericValue("goodnessString");
}
#getGoodnessString

sub getRegisteredString{
  my ($this ) = @_;
  return $this->getGenericValue("registeredString");
}
#getRegisteredString





#+sub():Hash:Get all DB-Parameters
sub getDBParams(){
  my ($this) = $_[0];
  %{$_[0]{$sdb_params}};
} 
#ifdef sub:setDBParams
#<pre>my $dbi = SAPDB::Install::DBIDesc::new();
#$dbi->setDBParams({
#                   "MAXUSERTASKS", 5 ,
#                   "NOCHNKEY", "NOCHWERT" 
#                  });</pre>
  #endif
  #+sub(HashRef:DB-Params):void:Set all Database parameters
  sub setDBParams(){ $_[0]{$sdb_params}=$_[1]; }
#ifdef sub:addDBParam
#<pre>my $dbi = SAPDB::Install::DBIDesc::new();
#$dbi->addDBParam("MAXUSERTASKS", 5);</pre>
  #endif
  #+sub(String:key|String:value):void:Add one Database Parameter
  sub addDBParam(){
    my ($this,$key,$value) = @_;
    $this->{$sdb_params}->{$key} = $value;
  }


my $dataAreaLabel =   "Data Area" ;
my $logAreaLabel = "Log Area";

#ifdef sub:getDBVolumes
#<pre><![CDATA[
#              my $volumes =  $instance->getDBVolumes();
#              foreach my $volume (@$volumes) {
#                my $number = @$volume[0]; #Zaehler  (fuer daten und log volumes getrennt)
#                my $art = @$volume[1]; # "DATA" oder "LOG"
#                my $path = @$volume[2]; # Pfad bzw Ort
#                my $type = @$volume[3]; # "F" steht fuer File, es gibt aber auch rawdevices
#                my $size = @$volume[4]; # Groesse in 8K-Pages
#              }
#             ]]>
#  </pre>
  #endif
  #+sub():ListOfListRefs:Get all DB-Volumes
  sub getDBVolumes(){
    my ($this) = $_[0];
    return $this->{$sdb_volumes}; 
  }


sub cloneDBVolumes{
  my ($this,$art) = @_;
  my $volumes = $this->getDBVolumes();
  my $clone = [];
  foreach my $volume (@$volumes) {
    my $anart = @$volume[1];
    if (defined $art) {
      if ( (uc $art) eq (uc $anart) ) {
        push @$clone, [@$volume[0],@$volume[1],@$volume[2],@$volume[3],@$volume[4]];
      }
    } else {
      push @$clone, [@$volume[0],@$volume[1],@$volume[2],@$volume[3],@$volume[4]];
    }
  }
  return $clone;
}



#ifdef sub:setDBVolumes
#<pre><![CDATA[
#              my $instance = SAPDB::Install::DBIDesc::new();
#              $instancedbi->setDBVolumes( [  
#                                           #    Anzahl Art   Pfad      Typ  Groesse
#                                           [1      "LOG"   "DISKL0001"   "F"    2000   ],
#                                           [1      "DATA"  "DISKD0001"   "F"    10000  ]
#                                          ]);
#            }]]>
#  </pre>
  #endif
  #+sub(ListRefOfListRefs:volumes):void:Set all database volumes
  sub setDBVolumes(){ $_[0]{$sdb_volumes}=$_[1]; }

#ifdef sub:addDBVolume
#<pre><![CDATA[
#              $instance->addDBVolume("1","LOG","c:/sdb/midb1/log/LOG_002","F","3000");
#             ]]>
#  </pre>
  #endif
  #+sub(number:anzahl|String:art|String:ort|String:typ|Number:groesse):void:Add one database volume
  sub addDBVolume(){
    #              Nummer Art   Name      Typ  Groesse
    my ($this,$anzahl,$art,$name,$typ,$groesse) = @_;
    push @{$_[0]{$sdb_volumes}},[$anzahl,$art,$name,$typ,$groesse];

  }


sub getVolStructCount{
  my ($this,$mart) = @_;
  my $volumes =  $this->getDBVolumes();
  my $counter =0;
  foreach my $volume (@$volumes) {
    my $art = @$volume[1];      # "DATA" oder "LOG"
    if ( (uc $mart) eq (uc $art)) {
      $counter++;
    }
  }
  return $counter;
}


sub getVolStructDataCount{
  my ($this) = @_;
  return $this->getVolStructCount("DATA");
}

sub getVolStructLogCount{
  my ($this) = @_;
  return $this->getVolStructCount("LOG");
}



sub getVolStructDataPath{
  my ($this,$volId) = @_;
  return $this->getVolStructField("DATA",$volId,2);
}

sub setVolStructDataPath{
  my ($this,$volId,$newVal) = @_;
  $this->setVolStructField("DATA",$volId,2,$newVal);
}

sub getVolStructLogPath{
  my ($this,$volId) = @_;
  return $this->getVolStructField("LOG",$volId,2);
}

sub setVolStructLogPath{
  my ($this,$volId,$newVal) = @_;
  $this->setVolStructField("LOG",$volId,2,$newVal);
}


######################################################

sub getVolStructDataSize{
  my ($this,$volId) = @_;
  return $this->getVolStructField("DATA",$volId,4);
}

sub setVolStructDataSize{
  my ($this,$volId,$newVal) = @_;
  $this->setVolStructField("DATA",$volId,4,$newVal);
}

sub getVolStructLogSize{
  my ($this,$volId) = @_;
  return $this->getVolStructField("LOG",$volId,4);
}

sub setVolStructLogSize{
  my ($this,$volId,$newVal) = @_;
  $this->setVolStructField("LOG",$volId,4,$newVal);
}

######################################################

sub getVolStructDataName{
  my ($this,$volId) = @_;
  my $aPath = $this->getVolStructDataPath($volId);
  return basename($aPath);
}

sub setVolStructDataName{
  my ($this,$volId,$newVal) = @_;
  my $aPath = $this->getVolStructDataPath($volId);
  my $dirName = dirname($aPath);
  my $newPath = catfile($dirName,$newVal);
  $this->setVolStructDataPath($volId,$newPath);
}

sub getVolStructLogName{
  my ($this,$volId) = @_;
  my $aPath = $this->getVolStructLogPath($volId);
  return basename($aPath);
}

sub setVolStructLogName{
  my ($this,$volId,$newVal) = @_;
  my $aPath = $this->getVolStructLogPath($volId);
  my $dirName = dirname($aPath);
  my $newPath = catfile($dirName,$newVal);
  $this->setVolStructLogPath($volId,$newPath);
}


sub identifyLogDataStructsByLabel{
  my ($this,$aStruct) = @_;
  my $dataStruct=undef;
  my $logStruct=undef;
  foreach my $aVol (@$aStruct) {
    my $label = $aVol->{"Label"};
    if (defined $label) {
      if ($label eq $dataAreaLabel) {
        $dataStruct=$aVol;
      } elsif ($label eq $logAreaLabel) {
        $logStruct=$aVol;
      }
    }
  }
  return ($dataStruct,$logStruct);
}

sub getLogDataStructValues{
  my ($this,$aStruct,$aKey) = @_;
  my @aShit = $this->identifyLogDataStructsByLabel($aStruct);
  return $this->getLogDataStructValuesFromShit(\@aShit,$aKey);
}


sub getLogDataStructValuesFromShit{
  my ($this,$aShit,$aKey) = @_;
  my ($dataStruct,$logStruct) = @$aShit;
  my $dataValue = defined $dataStruct ? $dataStruct->{$aKey} : undef;
  my $logValue = defined $logStruct ? $logStruct->{$aKey} : undef;
  return ($dataValue,$logValue);
}


sub setVolStructDrive{
  my ($this,$aStruct) = @_;
  my ($dataDrive,$logDrive) = $this->getLogDataStructValues($aStruct,"Drive");
  if (defined $dataDrive) {
    $this->setAdvicedDataDrive($dataDrive);
  }
  if (defined $logDrive) {
    $this->setAdvicedLogDrive($logDrive);
  }
}


sub setVolStructDir{
  my ($this,$aStruct) = @_;
  my ($dataPath,$logPath) = $this->getLogDataStructValues($aStruct,"Location");
  if (defined $dataPath) {
    $this->setDataPath($dataPath);
  }
  if (defined $logPath) {
    $this->setLogPath($logPath);
  }
}


sub volStructDirValidator{
  my @retVal = (1);
  my ($this,$aStruct) = @_;
  my ($dataPath,$logPath) = $this->getLogDataStructValues($aStruct,"Location");
  my $logSize = $this->getLogSize();
  my $dataSize = $this->getDataSize();
  my $logMount = GetFSName($dataPath);
  my $dataMount = GetFSName($logPath);
  my %mounts ;
  if (defined $logMount) {
    $mounts{$logMount} = [$logSize,$logPath,GetFSSize($logPath)];
  }
  if (defined $dataMount and not defined $mounts{$dataMount} ) {
    $mounts{$dataMount} = [$dataSize,$dataPath,GetFSSize($dataPath)];
  } elsif (defined $dataMount) {
    my $commonMount = $mounts{$dataMount};
    @$commonMount[0]+=$dataSize;
  }
  foreach my $key (keys %mounts) {
    my $mount = $mounts{$key};
    my $neededkb = $mount->[0] * 8;
    my $totalkb = $mount->[2];
    my $availkb = $mount->[3];
    if ($neededkb > $availkb) {
      my ($needed,$neededUnit)=readableUnit($neededkb,"KB"); 
      my ($available,$availableUnit)=readableUnit($availkb,"KB"); 
      my ($total,$totalUnit)=readableUnit($totalkb,"KB"); 
      my $message =  "Not enough space on \"$key\" : (Available: $available $availableUnit, " .
        "Needed: $needed $neededUnit)\nProvide enough disk space";
      @retVal = (0,"Insufficient space left on device",$message);
      last;
    }
  }

  return @retVal; 
}




sub setVolStructDriveSize{
  my ($this,$aStruct) = @_;

  my @aShit = $this->identifyLogDataStructsByLabel($aStruct);

  my ($dataDrive,$logDrive) =  $this->getLogDataStructValuesFromShit(\@aShit,"Drive");
  my ($dataSize,$logSize) =  $this->getLogDataStructValuesFromShit(\@aShit,"Size");

  if (defined $dataDrive) {
    $this->setAdvicedDataDrive($dataDrive);
  }
  if (defined $logDrive) {
    $this->setAdvicedLogDrive($logDrive);
  }
  if (defined $dataSize) {
    $this->setDataSize($dataSize);
  }
  if (defined $logSize) {
    $this->setLogSize($logSize);
  }
}                               #setVolStructDriveSize





sub setVolStructDirSize{
  my ($this,$aStruct) = @_;
  my @aShit = $this->identifyLogDataStructsByLabel($aStruct);
  my ($dataPath,$logPath) =  $this->getLogDataStructValuesFromShit(\@aShit,"Location");
  my ($dataSize,$logSize) =  $this->getLogDataStructValuesFromShit(\@aShit,"Size");

  if (defined $dataPath) {
    $this->setDataPath($dataPath);
  }
  if (defined $logPath) {
    $this->setLogPath($logPath);
  }
  if (defined $dataSize) {
    $this->setDataSize($dataSize);
  }
  if (defined $logSize) {
    $this->setLogSize($logSize);
  }
}                               #setVolStructDirSize


sub volumeFromStruct{
  my ($this,$aVol,$counter,$mart) = @_;
  my $location = $aVol->{"Location"};
  my $size = $aVol->{"Size"};
  my $name = $aVol->{"Name"};
  my $volumePath = catfile($location,$name);
  return ($counter,$mart,$volumePath,"F",$size);
}

sub setVolStructLog{
  my ($this,$aStruct) = @_;
  $this->clearVolStruct("log");
  my $counter = 1;
  my $logSizeVar=0;
  foreach my $aVol (@$aStruct) {
    my @aVolArray = $this->volumeFromStruct($aVol,$counter,"LOG");
    $this->addDBVolume(@aVolArray);
    $logSizeVar += $aVolArray[4];
    $counter++;
  }
  $this->setLogSize($logSizeVar);
}



sub volStructLogValidator{
  my ($this,$aStruct) = @_;
  my @retVal = (1);
  my $counter = 1;
  my $volClone = $this->cloneDBVolumes("DATA");
  my $logSizeVar=0;
  my %nameChecker;
  foreach my $aVol (@$aStruct) {
    my @aVolArray = $this->volumeFromStruct($aVol,$counter,"LOG");
    push @$volClone, \@aVolArray;
    $logSizeVar += $aVolArray[4];
    my $aPath = $this->volumePath(\@aVolArray);
    if ( defined $aPath) {
      my $aName = basename($aPath);
      if ( defined $aName) {
        if ( $nameChecker{$aName}) {
          return (0, "Duplicate volume name", "Volume name $aName is used more than once.\n".
                  "Use unique volume names");
        } else {
          $nameChecker{$aName}=1;
        }
      }
    }
    $counter++;
  }
  if ( $logSizeVar <= 0) {
    my ($logSize,$logSizeUnit)=readableUnit($logSizeVar,"PAGE"); 
    return (0,"Invalid value for log size", "Log size of $logSize $logSizeUnit does not make sense.\n".
            "Specify  useful sizes for your log volumes");
  }
  #test if log size is less than  64 MB
  if ( $logSizeVar <    (8  * 1024 ) ) { 
    my ($logSize,$logSizeUnit)=readableUnit($logSizeVar,"PAGE"); 
    return (0,"Value for log size too low","Log size of $logSize $logSizeUnit is not enough.\n".
            "Specify  enough  space for database log (64 MB is Minimum)");
  }
  @retVal = $this->validateVolumeSize($volClone,"Provide enough space for data and log volumes");
  return @retVal; 
}



sub setVolStructData{
  my ($this,$aStruct) = @_;
  $this->clearVolStruct("DATA");
  my $counter = 1;
  my $dataSizeVar =0;
  foreach my $aVol (@$aStruct) {
    my @aVolArray = $this->volumeFromStruct($aVol,$counter,"DATA");
    $this->addDBVolume(@aVolArray);
    $dataSizeVar += $aVolArray[4];
    $counter++;
  }
  $this->setDBSize($dataSizeVar);
}



sub volStructDataValidator{
  my @retVal = (1);
  my ($this,$aStruct) = @_;
  my $counter = 1;
  my $volClone = $this->cloneDBVolumes("LOG");
  my $dataSizeVar=0;
  my %nameChecker;
  foreach my $aVol (@$aStruct) {
    my @aVolArray = $this->volumeFromStruct($aVol,$counter,"DATA");
    push @$volClone, \@aVolArray;
    $dataSizeVar += $aVolArray[4];
    my $aPath = $this->volumePath(\@aVolArray);
    if ( defined $aPath) {
      my $aName = basename($aPath);
      if ( defined $aName) {
        if ( $nameChecker{$aName}) {
          return (0,"Duplicate volume name","Volume name $aName is used more than once.\n".
                  "Use unique volume names");
        } else {
          $nameChecker{$aName}=1;
        }
      }
    }
    $counter++;
  }
  if ( $dataSizeVar <= 0) {
    my ($dataSize,$dataSizeUnit)=readableUnit($dataSizeVar,"PAGE"); 
    return (0,"Invalid value for data volume size",
            "Data size of $dataSize $dataSizeUnit does not make sense\n" . 
            "Specify  useful sizes for your log volumes");
  }
  #test if data size is less than  64 MB
  if ( $dataSizeVar <    (8  * 1024 ) ) { 
    my ($dataSize,$dataSizeUnit)=readableUnit($dataSizeVar,"PAGE"); 
    return (0,"Value for data size too low","Data size of $dataSize $dataSizeUnit is not enough.\n".
            "Specify  enough  space for data volumes (64 MB is Minimum)");
  }
  @retVal = $this->validateVolumeSize($volClone,"Provide enough space for data and log volumes");
  return @retVal; 
}





sub setVolStruct{
  my ($this,$aStruct) = @_;
  $this->clearVolStruct();
  my $logCounter = 1;
  my $dataCounter = 1;
  my $dbsizeVar =0;
  my $dblogVar = 0;
  foreach my $aVol (@$aStruct) {
    my $location = $aVol->{"Location"};
    my $size = $aVol->{"Size"};
    my $fileName = $aVol->{"Name"};
    my $art = $aVol->{"Art"};
    my $volumePath = catfile($location,$fileName);
    if ((lc $art) eq "log") {
      $this->addDBVolume($logCounter,$art,$volumePath,"F",$size);
      $logCounter++;
      $dblogVar += $size;
    } else {
      $this->addDBVolume($dataCounter,$art,$volumePath,"F",$size);
      $dataCounter++;
      $dbsizeVar += $size;
    }
  }
  $this->setDBSize($dbsizeVar);
  $this->setLogSize($dblogVar);
}


sub clearVolStruct{
  my ($this,$mart) = @_;
  my $volumes =  $this->getDBVolumes();
  my $newVols = $this->clearVolStructHash($mart,$volumes);
  $this->setDBVolumes($newVols);
}
sub clearVolStructHash{
  my ($this,$mart,$volumes) = @_;
  my $newVols = [];
  foreach my $volume (@$volumes) {
    my $art = @$volume[1];
    if ( defined $mart and (uc $mart) eq (uc $art)) {
    } elsif ( not defined $mart ) {
    } else {
      push @$newVols, ($volume);
    }
  }
  return $newVols;
}



sub fillVolStruct{
  my ($this,$retVal,$mart) = @_;
  my $volumes =  $this->getDBVolumes();

  unless(defined $volumes ){
    my $dbsize = $this->getDBSize();
    my $logsize = $this->getLogSize();
    $dbsize = defined $dbsize ? $dbsize : 0; 
    $logsize = defined $logsize ? $logsize : 0; 
    $volumes = [
                [1,      "LOG",   catfile($this->getLogPath(),"DISKL0001"),   "F",    $logsize   ],
                [1,      "DATA",  catfile($this->getDataPath(),"DISKD0001"),  "F",    $dbsize  ]
               ];
  }


  foreach my $volume (@$volumes) {
    my $art = @$volume[1];
    if ( not defined $mart or (uc $mart) eq (uc $art)) {
      my $volHash = {};
      my $number  =@$volume[0];
      my $volumePath  =@$volume[2];
      my $type = @$volume[3];
      my $size = @$volume[4];
      my $drive = undef;
      my $dir = undef;
      my $location  =dirname($volumePath);
      my $name = basename($volumePath);
      my $label=undef;
      my $prefix = undef;
      my $postfix = undef;
      my $dbName = $this->getDBName();
      if ( $^O =~ /mswin/i) {
        $drive =  (lc $art) eq  "data" ?  $this->getDataDrive() : $this->getLogDrive();
        if ( defined $drive and $drive =~ /^[a-z,A-Z]$/) {
          $drive = $drive . ":";
        }
        $dir =  (lc $art) eq  "data" ?  $this->getDataPath() : $this->getLogPath();
      } else {
        $dir =  (lc $art) eq  "data" ?  $this->getDataPath() : $this->getLogPath();
      }
      $prefix =  (lc $art) eq  "data" ?  $this->getDataPathPrefix() : $this->getLogPathPrefix();

      if ( (lc $art) eq  "data") {
        $postfix = $this->getDataPathPostfix();
      } else {
        $postfix = $this->getLogPathPostfix();
      }
      $label =  (lc $art) eq  "data" ?  $dataAreaLabel : $logAreaLabel;

      $volHash->{"Postfix"}=$postfix;
      $volHash->{"Prefix"}=$prefix;

      $volHash->{"Dir"}=$dir;
      $volHash->{"Drive"}=$drive;
      $volHash->{"Name"}=$name;
      $volHash->{"Number"}=$number;
      $volHash->{"Art"}=$art;
      $volHash->{"Location"}=$location;
      $volHash->{"Type"}=$type;
      $volHash->{"Size"}=$size;
      $volHash->{"Label"}=$label;
      push @$retVal, ($volHash);
    }
  }
  return $retVal;
}                               #fillVolStruct



sub getVolStruct{
  my ($this) = @_;
  my $retVal=[];
  $this->fillVolStruct($retVal);
  return $retVal;
}


sub getVolStructData{
  my ($this) = @_;
  my $retVal = [];
  return $this->fillVolStruct($retVal,"data");
  return $retVal;
}

sub getVolStructLog{
  my ($this) = @_;
  my $retVal = [];
  return $this->fillVolStruct($retVal,"log");
  return $retVal;
}


sub getVolStructField{
  my ($this,$mart,$volId,$field) = @_;
  my $retVal=undef;
  my $volumes =  $this->getDBVolumes();
  foreach my $volume (@$volumes) {
    my $art = @$volume[1];
    my $aVolId= @$volume[0];
    if ( ((uc $mart) eq (uc $art)) and ($aVolId ==  $volId)) {
      $retVal=@$volume[$field];
      last;
    }
  }
  return $retVal;
}

sub setVolStructField{
  my ($this,$mart,$volId,$field,$newVal) = @_;
  my $retVal=undef;
  my $volumes =  $this->getDBVolumes();
  foreach my $volume (@$volumes) {
    my $art = @$volume[1];      
    my $aVolId= @$volume[0];      
    if ( ((uc $mart) eq (uc $art)) and ($aVolId ==  $volId)) {
      $volume->[$field]=$newVal;
      last;
    }
  }
}







#+sub(LOGGING.COMPOUND):void:Set loggingStruct
sub setLoggingStruct{
  my ($this, $loggingStruct) = @_;
  my $isAutoOverWriteLog = $loggingStruct->{"AutoOverWriteLog"};
  my $isAutoSaveLog = $loggingStruct->{"AutoSaveLog"};
  my $initialBackupDir = $loggingStruct->{"InitialBackupDir"};
  my $logSaveDir = $loggingStruct->{"LogSaveDir"};
  $this->setAutoOverWriteLog($isAutoOverWriteLog);
  $this->setAdvicedAutoSaveLog($isAutoSaveLog);
  $this->setInitialBackupDir($initialBackupDir);
  $this->setLogSaveDir($logSaveDir);
}
#+sub():LOGGING.COMPOUND:Get loggingStruct
sub getLoggingStruct{
  my ($this ) = @_;
  my $isAutoOverWriteLog = isTrue($this->getAutoOverWriteLog());
  my $isAutoSaveLog = isTrue($this->getAutoSaveLog());
  my $initialBackupDir = $this->getInitialBackupDir();
  my $dataPrefix = $this->getDataPathPrefix();
  
  if ( not defined $dataPrefix) {
    $this->writeLog("Internal error in subroutine getLoggingStruct: Missing parameter DataPathPrefix\n");
  }
  if ( not defined $initialBackupDir) {
    my $dbname = $this->getDBName();
    $initialBackupDir = catfile($dataPrefix,$dbname,"backup","data");
  }
  my $logSaveDir = $this->getLogSaveDir();
  if ( not defined $logSaveDir) {
    my $dbname = $this->getDBName();
    $logSaveDir = catfile($dataPrefix,$dbname,"backup","log");
  }
  my  $aLogStruct = {
                     "AutoOverWriteLog" => $isAutoOverWriteLog,
                     "AutoSaveLog" => $isAutoSaveLog,
                     "InitialBackupDir" => $initialBackupDir,
                     "LogSaveDir" => $logSaveDir
                    };
  return $aLogStruct;
}












#ifdef sub:proposeMemory
#<h3 id="memory_estimate">Schtzung fr Memory</h3>
#  <p>
#  Es wird folgender <code>"Algorithmus"</code> angewendet 
#  (Alle Angaben in Pages)
#  </p>
#  <ul>
#  <li label="REALMEM">Wert des <code>MachInfo</code>-Moduls: Vorhandener Hauptspeicher</li>
#  <li label="CACHE_SIZE">Vorschlagswert</li>
#  <li label="MIN">: 1000</li>
#  <li label="MAX">: Datenbankgre</li>
#  <li label="OSFIX">: Windows: 16384 , Unix: 8192</li>
#  <li label="PERCENTAGE">0.7 (70 %)</li>
#  </ul>
#  <p>Berechnung: </p>
#  <pre>
#  <![CDATA[
#           CACHE_SIZE = (REALMEM - OSFIX) * PERCENTAGE;
#           CACHE_SIZE = (CACHE_SIZE <= MIN) ? MIN : CACHE_SIZE;
#           CACHE_SIZE = (CACHE_SIZE >= MAX) ? MAX : CACHE_SIZE;
#          ]]>
#  </pre>
  #endif
  #+sub():void:Propose Memory consumption for installation
  sub proposeMemory{
    my ($this) = @_;
    my ($real,$unit) =getPhysicalMemoryTotal8KPages(); 
    my $perc =  0.7;
    my $fullData= $this->getFullDataVolumeSize();
    my $min = $this->getMinimalMemory();
    my $retVal =  $this->getMaximalMemory();

    $retVal *= $perc;

    if ($min< $real) {
      $retVal = $min> $retVal ? $min: $retVal;
    }
    return sprintf "%d", $retVal;
  }



sub getMinimalMemory{
  return 1000;
}
sub getMaximalMemory{
  my ($this) = @_;
  my ($real,$unit) =getPhysicalMemoryTotal8KPages(); 
  my $dbsize = $this->getDBSize();
  my $fullData= undef;
  if ( defined $dbsize) {
    $fullData= $dbsize;
  } else {
    $fullData= $this->getFullDataVolumeSize();
  }

  my $osFix = ($^O =~ /mswin/i) ? 128 * (1024/ 8) : 64  *  (1024/ 8)   ;
  $osFix = sprintf "%d" , $osFix;
  my $max = $real - $osFix;
  $max = $fullData <= $max ? $fullData: $max;

  #  print "Maximal memory: $max, datavols: $fullData\n";
  return $max;
}


#+sub():void:Propose number of cpus for installation
sub proposeCPU{
  my ($this) = @_;
  my $cpu = getNumOfCPU();
  return sprintf "%d", ($cpu + 1) /2 ; 
}



#+sub():Pages:Get the complete data size (sum of size of all data volumes)
sub getFullDataVolumeSize{
  my ($this) = @_;
  return $this->getFullVolumeSizeByType("DATA");
}
#getFullDataVolumeSize


#+sub():Pages:Get the complete data size (sum of size of all data volumes)
sub getFullLogVolumeSize{
  my ($this) = @_;
  return $this->getFullVolumeSizeByType("LOG");
}
#getFullDataVolumeSize


#+sub():Pages:Get the complete data size (sum of size of all data volumes)
sub getFullVolumeSizeByType{
  my ($this,$art) = @_;

  my $volumes = $this->getDBVolumes();
  my $fullSize=0;
  foreach my $volume (@$volumes) {
    my $aArt = @$volume[1];
    my $aSize = @$volume[4];
    if ($aArt =~ /^\s*$art\s*$/i && defined $aSize) {
      $fullSize += $aSize;
    }
  }
  #foreach
  return $fullSize;
}
#getFullDataVolumeSize


sub checkVolumeSize{
  my ($this) = @_;
  my $volumes = $this->getDBVolumes();
  my $retVal = $this->checkVolumeSizeOnVolumeArray($volumes);
  return $retVal;
}
sub checkVolumeSizeOnVolumeArray{
  my ($this,$volumes) = @_;
  my $retVal = {};
  foreach my $volume (@$volumes) {
    my $aPath = $this->volumePath($volume);
    my $aSize = $this->volumeSize($volume) *8;
    my $aFS = GetFSName($aPath);
    next unless($aFS);
    my $fsData=undef;
    if ( defined $retVal->{$aFS}) {
      $fsData = $retVal->{$aFS};
    } else {
      $retVal->{$aFS} = {};
      $fsData = $retVal->{$aFS};
      my ($totalkb, $availkb) = GetFSSize($aPath);
      $fsData->{"total"} = $totalkb;
      $fsData->{"available"} = $availkb;
    }
    $fsData->{"needed"} += $aSize;
  }
  return $retVal;
}


sub validateVolumeSize{
  my ($this,$volClone,$advice) = @_;
  my @retVal = (1);
  my $fsresult  = $this->checkVolumeSizeOnVolumeArray($volClone);
  foreach my $key (keys %$fsresult) {
    my $fsdata  = $fsresult->{$key}; 
    my $neededkb = $fsdata->{"needed"};
    my $availablekb = $fsdata->{"available"};
    my $totalkb = $fsdata->{"total"};
    if ($neededkb > $availablekb) {
      my ($needed,$neededUnit)=readableUnit($neededkb,"KB"); 
      my ($available,$availableUnit)=readableUnit($availablekb,"KB"); 
      my ($total,$totalUnit)=readableUnit($totalkb,"KB"); 
      my $message =  "Not enough space on \"$key\" : (Available: $available $availableUnit, " .
        "Needed $needed $neededUnit)\n" . $advice;
      @retVal = (0,"Insufficent space left on device",$message);
      last;
    }
  }
  return @retVal; 
}





#+sub(Volume:aVolume):void:Get the number of a volume
sub volumeNumber{
  my ($this,$volume) = @_;
  return $volume->[0];
}
#+sub(Volume:aVolume):void:Get the kind of a volume ("DATA" or "LOG")
sub volumeArt{
  my ($this,$volume) = @_;
  return $volume->[1];
}
#+sub(Volume:aVolume):void:Get the path of a volume
sub volumePath{
  my ($this,$volume) = @_;
  return $volume->[2];
}
#+sub(Volume:aVolume):void:Get the type of a volume (i.e. 'F' for file)
sub volumeType{
  my ($this,$volume) = @_;
  return $volume->[3];
}
#+sub(Volume:aVolume):void:Get the size of a volume (in 8K Pages)
sub volumeSize{
  my ($this,$volume) = @_;
  return $volume->[4];
}


sub dbmcmdAddAllVolumes{
  my ($this,$dbm) = @_;
  my $volumes =  $this->getDBVolumes();
  foreach my $volume (@$volumes) {
    unless (defined $this->dbmcmdAddOneVolume($dbm,@$volume)){
      return undef;
    }
  }
  return 0;
}
#dbmcliAddAllVolumes


sub dbmcmdAddOneVolume{
  my ($this,$dbm, @volume) = @_;
  my $statement = "param_addvolume " . $volume[0]
    . " " . $volume[1]
      . " \"" . $volume[2] . "\"" 
        . " " . $volume[3]
          . " " . $volume[4]
            . "";
  if ( not $this->createVolumeForPath(\@volume)) {
    return undef;
  }
  my $retVal = $this->dbm($dbm,$statement,"ERROR: Could not add volume\n","Volume $volume[2] added.\n" );
  return $retVal;
}
#dbmcliAddOneVolume 


sub createVolumeForPath{
  my ($this, $volume) = @_;
  my $retVal = 1;

  my $path = $volume->[2];
  my $dir = dirname($path);
  if ( defined $dir and not "." eq $dir and not -d $dir) {
    my $makeDir = yamakedir($dir,0750);
    if ( $makeDir > 1) {        # directory could not be created
      $this->writeLog("ERROR creating volume directory \"$dir\"\n" );
      $retVal = 0;
    } else {
      unless ($^O =~ /mswin/i){
        my $parentDir = dirname($dir);
        $this->writeLog("[chmod (0550,$parentDir)]\n");
        chmod (0550, $parentDir);
      }                         #unless
    }
  }                             #if
  return $retVal;
}






sub getAllRealVolumes(){
  my ($this,$dbm ) = @_;
  my $volRef = $dbm->param_getvolsall();
  unless(defined $volRef){
    $this-> dbmcmdError($dbm,"Cannot read existing volumes\n");
  }
  return $volRef;
}



#+sub():Pages:Get the real complete data size (sum of size of all data volumes)
sub getRealFullDataVolumeSize{
  my ($this,$dbm) = @_;
  return $this->getRealFullVolumeSizeByType($dbm,"DATA_VOLUME");
}

#+sub():Pages:Get the real complete log size (sum of size of all log volumes)
sub getRealFullLogVolumeSize{
  my ($this,$dbm) = @_;
  return $this->getRealFullVolumeSizeByType($dbm,"LOG_VOLUME");
}

sub getRealFullVolumeSizeByType{
  my ($this,$dbm,$type ) = @_;
  my $fullDataSize = undef;
  my $realVols = $this->getAllRealVolumes($dbm);
  unless(defined $realVols){
    return $fullDataSize;
  }
  $fullDataSize = 0;
  while (my ($key, $val) = each (%$realVols)) {
    if (ref ($val) eq "HASH") {
      my $aVol = $key;
      my $aSize = $val->{'size'};
      my $aType = $val->{'type'};
      my $aName = $val->{'name'};
      if ($aVol =~ /^$type/i and $aSize =~ /^\d+$/) {
        $fullDataSize+=$aSize;
      } elsif (not $aSize =~ /^\d+$/) {
        $fullDataSize = undef;
        last;
      }
    } else {
    }
  }
  return $fullDataSize;
}                               #getRealFullVolumeSizeByType


sub writeLog{
  my ($this,$message) = @_;
  &{$this->getLogCallback()}($message);
}

sub showStatusMessage{
  my ($this,$message) = @_;
  if ( $message =~ /^ERROR:/) {
    $this->setErrorMessage($message);
  }
  &{$this->getStatusCallback()}($message);
}


sub sendAsyncCallback{
  my ($this,$elapsed,$message) = @_;
  &{$this->getAsyncCallback()}($elapsed,$message);
}




sub writeLogAndStatusMessage{
  my ($this,$message) = @_;
  $this->writeLog($message);
  $this->showStatusMessage($message);
}





# Gibt sein Argument nach Stdout:
sub stdCallback{
  print( $_[0] );
}

#+sub():FuncRef:Get the function for output (logging) 
sub getLogCallback{ $_[0]{$outputter}; }
#+sub(FuncRef:Function):void:Set the function for output
sub setLogCallback{ $_[0]{$outputter}=$_[1]; }



#+sub():FuncRef:Get the function for status messages
sub getStatusCallback{ $_[0]{$statusputter}; }
#+sub(FuncRef:Function):FuncRef:Set the function for status messages
sub setStatusCallback{ $_[0]{$statusputter}=$_[1]; }



#+field(FuncRef:AsyncCallback)
my $AsyncCallback="AsyncCallbackKey";




#+sub(FuncRef):void:Set AsyncCallback
sub setAsyncCallback{
  my ($this, $aValue) = @_;
  $this->{$AsyncCallback}=$aValue;
}
#+sub():FuncRef:Get AsyncCallback
sub getAsyncCallback{
  my ($this ) = @_;
  return $this->{$AsyncCallback};
}






#+field(StatusListe:statusList)
my $statusList="statusListKey";
#+sub(StatusListe):void:Set statusList
sub setStatusList{
  my ($this, $aValue) = @_;
  $this->{$statusList}=$aValue;
}
#+sub():StatusListe:Get statusList
sub getStatusList{
  my ($this ) = @_;
  return $this->{$statusList};
}



#Adds an item to status list before another otem
sub addItemToStatusList{
  my ($this,$afterKey,$newStatus ) = @_;
  my $newId = $newStatus->{"Id"};
  $this->removeItemFromStatusList($newId);
  my $statusList = $this->getStatusList();
  my $index = $this->itemIndexInStatusList($afterKey);
  my $afterValue = $statusList->[$index];
  return splice (@$statusList,$index,1,($newStatus,$afterValue));
}

sub removeItemFromStatusList{
  my ($this,$itemKey) = @_;
  my $statusList = $this->getStatusList();
  my $index = $this->itemIndexInStatusList($itemKey);
  if ($index >= 0) {
    splice (@$statusList,$index,1);
  }
  return $index;
}



sub itemIndexInStatusList{
  my ($this,$itemKey) = @_;
  my $index=-1;
  my $statusList = $this->getStatusList();
  my $counter=0;
  foreach my $statusItem (@$statusList) {
    my $id = $statusItem->{"Id"};
    if ( $id eq $itemKey) {
      $index=$counter;
      next;
    }
    $counter++;
  }
  return $index;
}






sub setStatusInStatusList{
  my ($this,$id,$newStatus ) = @_;
  my $statusList = $this->getStatusList();
  foreach my $statusItem (@$statusList) {
    my $anId = $statusItem->{"Id"};
    if ( $anId eq $id) {
      $statusItem->{"Status"}=$newStatus;
      if ( $newStatus eq "RUNNING") {
        $statusItem->{"StartTime"} = time();
      } elsif ( $newStatus =~ "^FINISHED") {
        my $aTime = time();
        $statusItem->{"EndTime"} = $aTime;
        unless (defined $statusItem->{"StartTime"}) {
          $statusItem->{"StartTime"} = $aTime;
        }
      }
      my $statusListCallback = $this->getStatusListCallback();
      &$statusListCallback($statusList);
    }
  }
}

sub setStatusRunning{
  my ($this,$id ) = @_;
  $this->setStatusInStatusList($id,"RUNNING");
}

sub setStatusFinishedSuccess{
  my ($this,$id ) = @_;
  $this->setStatusInStatusList($id,"FINISHED_SUCCESS");
}

sub setStatusFinishedError{
  my ($this,$id ) = @_;
  $this->setStatusInStatusList($id,"FINISHED_ERROR");
}




#+field(Callback Function:statusListCallback)
my $statusListCallback="statusListCallbackKey";
#+sub(Callback Function):void:Set statusListCallback
sub setStatusListCallback{
  my ($this, $aValue) = @_;
  $this->{$statusListCallback}=$aValue;
}
#+sub():Callback Function:Get statusListCallback
sub getStatusListCallback{
  my ($this ) = @_;
  return $this->{$statusListCallback};
}





#+field(String:LastDBMError)
my $LastDBMError="LastDBMErrorKey";
#+sub(String):void:Set LastDBMError
sub setLastDBMError{
  my ($this, $aValue) = @_;
  $this->{$LastDBMError}=$aValue;
}
#+sub():String:Get LastDBMError
sub getLastDBMError{
  my ($this ) = @_;
  return $this->{$LastDBMError};
}
#+field(String:LastDBMMessage)
my $LastDBMMessage="LastDBMMessageKey";
#+sub(String):void:Set LastDBMMessage
sub setLastDBMMessage{
  my ($this, $aValue) = @_;
  $this->{$LastDBMMessage}=$aValue;
}
#+sub():String:Get LastDBMMessage
sub getLastDBMMessage{
  my ($this ) = @_;
  return $this->{$LastDBMMessage};
}





#+sub():FuncRef:Get the dbmcli-Handler
sub getScriptHandler{ $_[0]{$scriptHandler}; }
#+sub(FuncRef:Function):void:Set the dbmcli-Handler
sub setScriptHandler{ $_[0]{$scriptHandler}=$_[1]; }

#+sub(String:dependend Path):void:Set the dependend path
sub setDependendPath{
  my ($this,  $aPath) = @_;
  my $aDependendPath;
  if (defined $aPath) {
    if (isPathAbsolute($aPath)) {
      $aDependendPath = $aPath;
    } elsif ("" eq $aPath) {
      my $aSID = $this->getDBName();
      my $aRootPath = $this->getInstRootPath();
      if (defined $aSID and defined $aRootPath) {
        $aDependendPath = catfile($aRootPath ,$aSID ,'db');
      }
    } else {
      my $aSID = $this->getDBName();
      my $aRootPath = $this->getInstRootPath();
      if (defined $aSID and defined $aRootPath) {
        $aDependendPath = catfile($aRootPath ,$aSID ,$aPath);
      }
    }
  } 
  $this->{$dependend_path}=$aDependendPath;
  return  $aDependendPath;
}















#+sub():FuncRef:Get the dependend path
sub getDependendPath{
  return $_[0]{$dependend_path};
}





my $dataPathKey='dataPathKey';
#+sub():String:Get path to data volumes
sub getDataPath($){ return $_[0]{$dataPathKey}; }
#+sub(String:path):void:Set path for data volumes
sub setDataPath{
  my ($this,  $aPath) = @_;
  my $aDataPath;
  if (defined $aPath) {
    if (isPathAbsolute($aPath)) {
      $aDataPath = $aPath;
    } elsif ("" eq $aPath) {
      my $aSID = $this->getDBName();
      my $aRootPath = $this->getInstRootPath();
      $aDataPath = catfile($aRootPath ,$aSID ,'data');
    } else {
      my $aSID = $this->getDBName();
      my $aRootPath = $this->getInstRootPath();
      $aDataPath = catfile($aRootPath ,$aSID ,$aPath);
    }
  } 
  # print "Base::setDataPath: New data path: " . $aDataPath . "\n";
  $this->{$dataPathKey}=$aDataPath; 
  return  $aDataPath;
}
#+field(Path:DataPathPrefix)
my $DataPathPrefix="DataPathPrefixKey";
#+sub(Path):void:Set DataPathPrefix
sub setDataPathPrefix{
  my ($this, $aDataPathPrefix) = @_;
  $this->{$DataPathPrefix}=$aDataPathPrefix;
  if (defined $aDataPathPrefix) {
    my $newDataPath = catfile($aDataPathPrefix,$this->getDataPathPostfix());
    $this->setDataPath($newDataPath);
  }
}
sub setDataPathPrefixInit{
  my ($this, $aDataPathPrefix) = @_;
  $this->{$DataPathPrefix}=$aDataPathPrefix;
  if (defined $aDataPathPrefix) {
    my $aDataPath = $this->getDataPath();
    unless($aDataPath){
      my $newDataPath = catfile($aDataPathPrefix,$this->getDataPathPostfix());
      $this->setDataPath($newDataPath);
    }
  }
}
#+sub():Path:Get DataPathPrefix
sub getDataPathPrefix{
  my ($this ) = @_;
  return $this->{$DataPathPrefix};
}


sub getDataPathPostfix{
  my ($this ) = @_;
  return catfile($this->getDBName(),"data");
}

sub getLogPathPostfix{
  my ($this ) = @_;
  return catfile($this->getDBName(),"log");
}





my $dataVolCountKey='dataVolCountKey';
#+sub():int:Get number of data volumes
sub getDataVolCount{ return $_[0]{$dataVolCountKey}; }
#+sub(int:number):void:Set the number of data volumes
sub setDataVolCount{ $_[0]{$dataVolCountKey}=$_[1]; }

sub setAdvicedDataVolCount{
  my ($this, $aValue) = @_;
  willSet($this,$dataVolCountKey);
  $this->setDataVolCount($aValue);
  haveSet($this,$dataVolCountKey);
}




my $dataSizeKey='dataSizeKey';
#+sub():pages:Standard size of data volumes
sub getDataSize($){ return $_[0]{$dataSizeKey}; }
#+sub(pages:size):pages:Get the standard size of data volumes
sub setDataSize($$ ){ $_[0]{$dataSizeKey}=$_[1]; }

my $logPathKey='logPathKey';
#+sub():String:Get the path to log volumes
sub getLogPath($){ return $_[0]{$logPathKey}; }


#+sub(String:path):void:Set the path of log volumes
sub setLogPath{
  my ($this,  $aPath) = @_;
  my $aLogPath;
  if (defined $aPath) {
    if (isPathAbsolute($aPath)) {
      $aLogPath = $aPath;
    } elsif ("" eq $aPath) {
      my $aSID = $this->getDBName();
      my $aRootPath = $this->getInstRootPath();
      $aLogPath = catfile($aRootPath ,$aSID ,'log');
    } else {
      my $aSID = $this->getDBName();
      my $aRootPath = $this->getInstRootPath();
      $aLogPath = catfile($aRootPath ,$aSID ,$aPath);
    }
  } 
  $this->{$logPathKey}=$aLogPath; 
  return  $aLogPath;
}

#+field(Path:LogPathPrefix)
my $LogPathPrefix="LogPathPrefixKey";
#+sub(Path):void:Set LogPathPrefix
sub setLogPathPrefix{
  my ($this, $aLogPathPrefix) = @_;
  $this->{$LogPathPrefix}=$aLogPathPrefix;
  if (defined $aLogPathPrefix) {
    my $newLogPath = catfile($aLogPathPrefix,$this->getLogPathPostfix());
    $this->setLogPath($newLogPath);
  }
}
sub setLogPathPrefixInit{
  my ($this, $aLogPathPrefix) = @_;
  $this->{$LogPathPrefix}=$aLogPathPrefix;
  if (defined $aLogPathPrefix) {
    my $aLogPath  = $this->getLogPath();
    if (not defined $aLogPath) {
      my $newLogPath = catfile($aLogPathPrefix,$this->getLogPathPostfix());
      $this->setLogPath($newLogPath);
    }
  }
}
#+sub():Path:Get LogPathPrefix
sub getLogPathPrefix{
  my ($this ) = @_;
  return $this->{$LogPathPrefix};
}








my $logVolCountKey='logVolCountKey';
#+sub():int:Get number of log volumes
sub getLogVolCount{ return $_[0]{$logVolCountKey}; }
#+sub(int:number):void:Set the number of log volumes
sub setLogVolCount{ $_[0]{$logVolCountKey}=$_[1]; }



sub setAdvicedLogVolCount{
  my ($this, $aValue) = @_;
  willSet($this,$logVolCountKey);
  $this->setLogVolCount($aValue);
  haveSet($this,$logVolCountKey);
}






my $logSizeKey='logSizeKey';
#+sub():pages:Standard size of log volumes
sub getLogSize($){ return $_[0]{$logSizeKey}; }
#+sub(pages:size):pages:Get the standard size of log volumes
sub setLogSize($$){ $_[0]{$logSizeKey}=$_[1]; }


my $drive='drive';
#+sub():String:Get the drive for the complete installation
sub getDrive{ 
  my ($this) = @_;
  return $this->{$drive}; 
}
#+sub(String:drive):void:Set the drive for the complete installation
sub setDrive{
  my ($this, $aDrive) = @_;
  my $goodDrive = matchDrive($aDrive);
  $this->{$drive}=$goodDrive;
}

sub setAdvicedDrive{
  my ($this, $aValue) = @_;
  willSet($this,$drive);
  $this->setDrive($aValue);
  haveSet($this,$drive);
}


sub rootPrefix($$){
  my ($this, $aDrive) = @_;
  if ($^O =~ /mswin/i) {
    return catfile($aDrive . ":" , "sdb");
  } else {
    return catfile($aDrive,"sdb");
  }
}


# Known generics:
my $gnutar = "gnutar";




sub initializeVolumeDefinitions($){
  my ($this ) =  @_;
  my $aDataVolCount = $this->getDataVolCount();
  my $aLogVolCount = $this->getLogVolCount();
  my $aDataPath = $this->getDataPath();
  my $aLogPath = $this->getLogPath();
  my $aDataSize = $this->getDataSize();
  my $aLogSize = $this->getLogSize();

  $this->setDBVolumes([]);
  my $counter=0;
  while ( ++$counter <= $aDataVolCount) {
    my $aDataVolName = "DISKD000" . $counter ;
    my $aDataVolume =  (defined $aDataPath) ? catfile($aDataPath,$aDataVolName ) :$aDataVolName ;
    $this->addDBVolume ($counter, "DATA", $aDataVolume, "F", $aDataSize);
  }
  $counter=0;
  while ( ++$counter <= $aLogVolCount) {
    my $aLogVolName = "DISKL000" . $aLogVolCount ;
    my $aLogVolume =  (defined $aLogPath) ? catfile($aLogPath,$aLogVolName):$aLogVolName;
    $this->addDBVolume ($counter , "LOG", $aLogVolume, "F", $aLogSize);
  }
}
#initializeVolumeDefinitions





#+field(String:Installation Root)
my $instRootPath='instRootPath';
#+sub():String:Get installation root
sub getInstRootPath($){
  my ($this) = @_;
  return $this->{$instRootPath}; 
}


#ifdef sub:setInstRootPath
#<p>Implicitly sets dependend, data path and log path according to default</p>
  #endif
  #+sub(String:installPath):void:Set installation root path
  sub setInstRootPath($$){
    my ($this, $anInstRootPath) = @_;
    $this->{$instRootPath}=$anInstRootPath;
    return  $anInstRootPath;
  }
#setInstRootPath

sub setAdvicedInstRootPath{
  my ($this, $aValue) = @_;
  willSet($this,$instRootPath);
  $this->setInstRootPath($aValue);
  haveSet($this,$instRootPath);
}


#+sub():void:Create the package for db dump
sub createPackage($){
  my ($this) = @_;
  my $backup = $this->getBackup();
  my $backupFile = basename($backup);
  my $backupDir =  dirname($backup);
  my $scriptName = "instinstance";
  my $instanceListFile = "instance.lst";
  my $backupData = catfile("$backupDir" , "PACKAGEDATA");
  my $instanceData = catfile("$backupDir" , "INSTANCEDATA");
  my $instanceList = catfile("$backupDir" , $instanceListFile);
  my $script  =  catfile("$backupDir" , $scriptName);
  my $aDBName = $this->getDBName();
  my $packageName = "DB Instance";  

  my ($sec,$min,$our,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
  $year = 1900 + $year;

  open INSTANCEDATA, ">$instanceData";
  print INSTANCEDATA <<INSTANCEDATAEND;
Generated : $mday.$mon.$year $our:$min:$sec
Backup: $backup
INSTANCEDATAEND

  open PACKAGEDATA, ">$backupData";
  print PACKAGEDATA <<PACKAGEDATAEND;
INTERFACE_VERSION = "0.1"
PACKAGE_NAME = "$packageName"
PACKAGE_VERSION = "0"
MODE = "32"
SOFTWARE_VERSION = "7.5.00.04"
MIN_VERSION = "7.2.4.4"
TYPE = "FULL"
MAGIC_MODE = "NOBIT"
ALLOW_SKIP = "1"
DEFAULT_USER = "sdb"
DEFAULT_GROUP = "sdba"
PART_OF = "$packageName"
IS_TOP_OF = "$packageName"
IS_SUBPACKAGE = "1"
REQUIRE = "Database Kernel >= Release"
FILELIST = "${instanceListFile}"
SCRIPT = "${scriptName}"
PACKAGEDATAEND

  open INSTANCELIST, ">$instanceList";
  print INSTANCELIST <<INSTANCELISTEND;
INSTANCELISTEND

  open SCRIPT, ">$script";
  print SCRIPT <<SCRIPTEND;
#!/usr/bin/perl


sub BEGIN{
	my (\$repo) = \@SAPDB::Install::Repository::AllRepo;
	my \@neededPackages = (
		'Templates::Setup'
	);
	foreach my \$package (\@neededPackages) {
	  	unless (defined \$repo->Eval
		("SAPDB::Install::\$package", 1.01)) {
               	print join ("\n", \$repo->GetErr)."\n";
              	die;
        	}
	}
}





\$path{'name'} = 'Instance  path';
\$path{'mode'}=0775;
\$path{'opt'}='instance_path=s';
\$path{'default'}='sapdb/instance';

\@paths = (\\\%path);
\$main_path=\\\%path;
\%config = ('template', 'mi','config', 'mi.config');


sub framestart{
SAPDB::Install::Templates::Setup::framestart(\\\@paths,\\\%config);
return 1;
}

sub preinstall{
SAPDB::Install::Templates::Setup::preinstall(\\\@paths,\\\%config);
return 1;
}
sub postprepare{
SAPDB::Install::Templates::Setup::postprepare(\\\@paths,\\\%config);
return 1;
}

sub postinstall{
SAPDB::Install::Templates::Setup::postinstall(\\\@paths,\\\%config);
return 1;
}

#registrierung z.B. in Windows-Registry
sub register{
SAPDB::Install::Templates::Setup::register(\\\@paths,\\\%config);
return 1;
}

sub framestop{
SAPDB::Install::Templates::Setup::framestop(\\\@paths,\\\%config);
return 1;
}

sub unregister{
SAPDB::Install::Templates::Setup::unregister(\\\@paths,\\\%config);
return 1;
}
sub preuninstall{
SAPDB::Install::Templates::Setup::preuninstall(\\\@paths,\\\%config);
return 1;
}
sub postuninstall{
SAPDB::Install::Templates::Setup::postuninstall(\\\@paths,\\\%config);
return 1;
}
sub verify{
SAPDB::Install::Templates::Setup::verify(\\\@paths,\\\%config);
return 1;
}
1;
SCRIPTEND



  #  print "Creating package for instance backup \"" . $parts[1] .  "\"\n";
  #  print "Directory:  \"" . $parts[0] .  "\"\n";
}                               #createPackage(..)

#+sub():void:Archive the instance package using gnu tar
sub tgzPackage($){
  my ($this) = @_;
  my $backup = $this->getBackup();
  unless (-f $backup){
    $this->writeLog("ERROR: Backup \"$backup\" not found\n");
    return undef;
  }
  my $backupFile = basename($backup);
  my $backupDir =  dirname($backup);
  my $archive = $this->getArchive();
  unless(defined $archive){
    $this->writeLog("ERROR: Archive name not specified. " . 
                    "Cannot create archive.\nSpecify archive.\n");
    return undef;
  }

  my $gnuTarGeneric = $this->getGeneric($gnutar);
  my $gnuTarExec = "tar";
  if (defined $gnuTarGeneric) {
    $gnuTarExec = $gnuTarGeneric->[0];
  }                             #if

  $archive   =~ s/\\/\//g;
  $backupDir =~ s/\\/\//g;
  $archive   =~ s/^([a-zA-Z]):/\/cygdrive\/$1/;
  $backupDir =~ s/^([a-zA-Z]):/\/cygdrive\/$1/;
  my $tgzCommand = $gnuTarExec . " vcfz " . $archive .
    #      " -C " . $backupDir . " $backupFile PACKAGEDATA INSTANCEDATA  instance.lst instinstance";
    " -C " . $backupDir . " $backupFile PACKAGEDATA instinstance";
  $this->writeLog("Executing " . $tgzCommand . " ...");
  my $retVal = system( $tgzCommand);
  $this->writeLog("\n... finished with return code: $retVal");
  return $retVal;
}                               #tgzPackage()




#+field(Boolean:autoRestart)
my $autoRestart="autoRestartKey";
#+sub(Boolean):void:Set if autoRestart database after reboot
sub setAutoRestart{
  my ($this, $aValue) = @_;
  $this->{$autoRestart}=$aValue;
}
#+sub():Boolean:Get if autoRestart database after reboot
sub getAutoRestart{
  my ($this ) = @_;
  return $this->{$autoRestart};
}

#+field(Boolean:isCreateDBPaths)
my $isCreateDBPaths="isCreateDBPathsKey";
#+sub(Boolean):void:Set isCreateDBPaths
sub setIsCreateDBPaths{
  my ($this, $aValue) = @_;
  $this->{$isCreateDBPaths}=$aValue;
}
#+sub():Boolean:Get isCreateDBPaths
sub getIsCreateDBPaths{
  my ($this ) = @_;
  return $this->{$isCreateDBPaths};
}

#+field(String:ErrorMessage)
my $ErrorMessage="ErrorMessageKey";
#+sub(String):void:Set ErrorMessage
sub setErrorMessage{
  my ($this, $aValue) = @_;
  $this->{$ErrorMessage}=$aValue;
}
#+sub():String:Get ErrorMessage
sub getErrorMessage{
  my ($this ) = @_;
  return $this->{$ErrorMessage};
}


#+field(String:ErrorDescription)
my $ErrorDescription="ErrorDescriptionKey";
#+sub(String):void:Set ErrorDescription
sub setErrorDescription{
  my ($this, $aValue) = @_;
  $this->{$ErrorDescription}=$aValue;
}
#+sub():String:Get ErrorDescription
sub getErrorDescription{
  my ($this ) = @_;
  return $this->{$ErrorDescription};
}



# Create a new instance of this object.
#+sub():BaseRef:Default-Konstruktor
sub new(){
  my $this = createThis();
  bless $this ;
  return  $this;
}
#new(..)

sub setDefaults{
  my ($this) = @_;
  $this->setAsyncWaitTime(1);
  $this->setAsyncCallback
    ( sub{
        my $elapsed = $_[0];
        my $txt = $_[1];
        $this->showStatusMessage($elapsed . "s". $txt);
      }
    );
  $this->setGlobalPassword("DBM");
  $this->setGlobalUser("DBADMIN");

  $this->setStatusList(
                       [
                        {"Label"=>"Creating database","Id"=>"db_create",
                         "Status"=>"PENDING"},
                        {"Label"=>"Logging on","Id"=>"logon",
                         "Status"=>"PENDING"},
                        {"Label"=>"Setting parameters","Id"=>"param_put",
                         "Status"=>"PENDING"},
                        {"Label"=>"Adding volumes","Id"=>"param_addvolume",
                         "Status"=>"PENDING"},
                        {"Label"=>"Changing to admin mode","Id"=>"db_admin",
                         "Status"=>"PENDING"},
                        {"Label"=>"Activating database","Id"=>"db_activate",
                         "Status"=>"PENDING"},
                        {"Label"=>"Loading system tables","Id"=>"load_systab",
                         "Status"=>"PENDING"},
                        {"Label"=>"Final steps","Id"=>"finalizing",
                         "Status"=>"PENDING"}
                       ]
                      );
  $this->setStatusListCallback(sub{});


  $this->addAfterAdvice("autoUpdateStatisticsKey",sub{
                          my ($this) = @_;
                          my $isAutoUpdate = $this->getAutoUpdateStatistics();
                          if (isTrue($isAutoUpdate)) {
                            $this->setRunEventDispatcher(1);
                          }
                        }
                       );
  $this->addAfterAdvice("isAutoExtendKey",sub{
                          my ($this) = @_;
                          my $isAutoExtend = $this->getAutoExtend();
                          if (isTrue($isAutoExtend)) {
                            $this->setRunEventDispatcher(1);
                          } 
                        }
                       );


  $this->addAfterAdvice("globalPasswordKey",sub{
                          my ($this) = @_;
                          my $globalPass = $this->getGlobalPassword();
                          $this->setDBMPass($globalPass);
                          $this->setDBAPass($globalPass);
                          $this->setSQLPass($globalPass);
                        }
                       );

  $this->addAfterAdvice("globalUserKey",sub{
                          my ($this) = @_;
                          my $globalUser = $this->getGlobalUser();
                          $this->setDBA($globalUser);
                        }
                       );


  $this->addAfterAdvice("autoSaveLogKey",sub{
                          my ($this) = @_;
                          my $isAutoSaveLog = $this->getAutoSaveLog();
                          if (isTrue($isAutoSaveLog)) {
                            $this->addItemToStatusList
                              ("finalizing",
                               {"Label"=>"Switching on automatic log backup","Id"=>"init_auto_save_log",
                                "Status"=>"PENDING"});
                          } else {
                            $this->removeItemFromStatusList("init_auto_save_log");
                          } 
                        } 
                       );

  $this->addAfterAdvice("loadSampleSchemaKey",sub{
                          my ($this) = @_;
                          my $isLoadSampleSchema = $this->getLoadSampleSchema();
                          if (isTrue($isLoadSampleSchema)) {
                            my $newItem = 
                              {"Label"=>"Loading tutorial data","Id"=>"load_sample_schema",
                               "Status"=>"PENDING"};
                            my $isAutoSaveLog = $this->getAutoSaveLog();
                            if (isTrue($isAutoSaveLog)) {
                              $this->addItemToStatusList("init_auto_save_log",$newItem);
                            } else {
                              $this->addItemToStatusList("finalizing",$newItem);
                            }
                          } else {
                            $this->removeItemFromStatusList("load_sample_schema");
                          } 
                        } 
                       );

  $this->addAfterAdvice("parallelConnectionsKey",sub{
                          my $aParallelConnectionsNumber = $this->getParallelConnections();
                          $this->addDBParam("MAXUSERTASKS",$aParallelConnectionsNumber);
                        } 
                       );

}                               #setDefaults





sub createThis(){
  return {
          $sdb_name , "MAXDB1",
          $sdb_dbmuser , "dbm",
          $sdb_dbmpass , "dbm",
          $sdb_dbauser , "DBADMIN",
          $isCreateDBPaths , 1,
          #              $sdb_sqluser , "scott",
          #              $sdb_sqlpass , "tiger",
          $sdb_dbapass , "DBA",
          $sdb_domainpass , "DBA",
          $sdb_params ,{        #REFERENZ AUF HASH
                        #                            "MAXUSERTASKS", 5 
                       },
          $sdb_volumes , [      #REFERENZ AUF LISTE
                          #                              Nummer Art   Ort      Typ  Groesse
                          [qw (1      LOG   DISKL0001   F    2000)   ],
                          [qw (1      DATA  DISKD0001   F    10000)  ]
                         ],
          $outputter , \&stdCallback,
          $statusputter , \&stdCallback,
          $creationStrategy, 1,
          $deletionStrategy, 2,
          $generic , {}
         };
}
#createThis


sub configure{
  my ($this) = @_;
  my @configResult = (1);
  my $dataVolCount= $this->getDataVolCount();
  my $dataSize= $this->getDataSize();
  my $templateCode = $this->getTemplateCode();
  unless(defined $templateCode){
    $templateCode="UNDEFINED";
  }
  if (defined $dataSize and defined $dataVolCount and not "templ_server_advanced" eq $templateCode) {
    $this->setDBSize($dataSize * $dataVolCount);
  }
  return @configResult;
}


sub initialize{
  my ($this) = @_;
  my $dataVolCount= $this->getDataVolCount();
  my $dataSize= $this->getDataSize();
  my $templateCode = $this->getTemplateCode();
  unless(defined $templateCode){
    $templateCode="UNDEFINED";
  }
  if (defined $dataSize and defined $dataVolCount and not "templ_server_advanced" eq $templateCode) {
    $this->setDBSize($dataSize * $dataVolCount);
  }
}





#+sub(String:fileName):void:Parse String and configure
sub configureFromString($$){
  my ($this,$aString) = @_;
  my @lines = split /\n/ ,$aString;
  $this->configureFromLines(\@lines);
}

#+sub(String:fileName):void:Parse input file and configure
sub configureFromFile($$){
  my ($this,$file) = @_;
  open CONFIG, $file;
  my @lines = <CONFIG>;
  $this->configureFromLines(\@lines);
  close CONFIG;
}


#+sub(ArrayRef:Lines):void:Configure from an Array of lines
sub configureFromLines($$){
  my ($this,$linesRef) = @_;
  my @lines = @$linesRef;
  my ($section);
  foreach my $line (@lines) {
    my ($key,$value,$type);
    parseConfigLine
      ($line,
       sub{ $section =$_[0];},
       sub {$key=$_[0];
            $value=$_[1];
            if (defined $_[2]) {
              $type=$_[2];
            }
          },
       $this
      );
    if ( defined $key) {
      if ( $section eq 'DBInstance') {
        $this->configureInstanceBaseFromKeyValuePairs($key,$value);
      } elsif ( $section eq 'DBParam') {
        $this->configureDBParamsFromKeyValuePairs($key,$value);
      } elsif ( $section eq 'Generic') {
        $this->configureGenericFromKeyValuePairs($key,$value,$type);
      }
    }
    #if
  }
  #foreach
}
#configureFromLines


# Vorkonfigurieren der Datenbankinstanz mit Werten aus der Parameterdatei,
# vorgeparst als Schluessel/Wert-Paare
sub configureInstanceBaseFromKeyValuePairs($$$){
  my ($this,$key,$value) = @_;
  if ( $key =~ m/^\s*DBM\s*$/i) {
    my ($dbmname,$dbmpass) = matchUserPassword($value);
    #    print "value: $value\n";
    #    print "Setting $dbmname,$dbmpass\n";
    $this->setDBM($dbmname);
    if ( defined $dbmpass) {
      $this->setDBMPass($dbmpass);
    }
  } elsif ( $key =~ m/^\s*DBA\s*$/i) {
    my ($dbaname,$dbapass) = matchUserPassword($value);
    $this->setDBA($dbaname);
    if ( defined $dbapass) {
      $this->setDBAPass($dbapass);
    }
  } elsif ( $key =~ m/^\s*DOMAINPASS\s*$/i) {
    my ($domainpass) = matchUserPassword($value);
    $this->setDomainPass($domainpass);
  } elsif ( $key =~ m/^\s*SQLUSER\s*$/i) {
    if ( $value =~ /^\s*$/) {
      $this->setSQLU(undef);
      $this->setSQLPass(undef);
    } else {
      my ($dbuser,$dbpass) = matchUserPassword($value);
      $this->setSQLU($dbuser);
      if ( defined $dbpass) {
        $this->setSQLPass($dbpass);
      }
    }
  } elsif ( $key =~ m/^\s*ARCHIVE\s*$/i) {
    if ( $value =~ /^\s*$/) {
      $this->setArchive(undef);
    } else {
      $this->setArchive(matchPath($value));
    }
  } elsif ( $key =~ m/^\s*BACKUP\s*$/i) {
    if ( $value =~ /^\s*$/) {
      $this->setBackup(undef);
    } else {
      $this->setBackup(matchPath($value));
    }
  } else {
    my $paramsRef = $this->getParams();
    my $pDesc = $paramsRef->{uc $key};
    if (defined $pDesc) {
      my $setter = @$pDesc[0];
      &$setter($value);
    }
  }
}
#configureInstanceBaseFromKeyValuePairs









# Vorkonfigurieren der Datenbankparameter mit Werten aus der Parameterdatei,
# vorgeparst als Schluessel/Wert-Paare
sub configureDBParamsFromKeyValuePairs($$$){
  my ($this,$key,$value) = @_;
  $value =~ m/^\s*(\S.*\S?)\s*$/;
  my $parValue = $1;
  $this->addDBParam($key,$parValue);
}
#configureDBParamsFromKeyValuePairs


sub createInstancePaths{
  my ($this) = @_;
  my $aSID = $this->getDBName();
  my $retVal=1;

  if (defined $this->getScriptHandler()) {
    return $retVal;
  }


  my $aLogPath = $this->getLogPath();

  my $uid = "";
  my $gid = "";
  unless ($^O =~ /mswin/i){
    $uid = getpwnam($SAPDB::Install::Values::owner);
    $gid = getgrnam($SAPDB::Install::Values::group);
  }

  if ( (defined $aLogPath) and (not -d $aLogPath)) {
    $this->writeLog("[makedir ($aLogPath,0750,$uid,$gid)]\n");
    makedir($aLogPath,0750,$uid,$gid);
    unless ($^O =~ /mswin/i){
      my $logBase = dirname($aLogPath);
      $this->writeLog("[chmod (0550,$logBase)]\n");
      chmod (0550, $logBase);
    }
    if ( -d $aLogPath) {
      $retVal=2;
    } else {
      $this->writeLog("ERROR creating directory \"$aLogPath\"\n");
      $retVal=0;
    }
  }
  #if
  if ( $retVal) {
    my $aDataPath = $this->getDataPath();
    if ( (defined $aDataPath) and (not -d $aDataPath)) {
      $this->writeLog("[makedir ($aDataPath,0750,$uid,$gid)]\n");
      makedir($aDataPath,0750,$uid,$gid);
      unless ($^O =~ /mswin/i){
        my $dataBase = dirname($aDataPath);
        $this->writeLog("[chmod (0550,$dataBase)]\n");
        chmod (0550, $dataBase);
      }
      if (-d $aDataPath) {
        $retVal=3;
      } else {
        $this->writeLog("ERROR creating directory \"$aDataPath\"\n" );
        $retVal=0;
      }
    }
  }
  return $retVal;
}
#createInstancePath





my ($doReuseInstance,$doNotReuseInstance) =(1,0);

sub createInstance{
  my ($this) = @_;
  my $aIsCreateDbPaths = $this->getIsCreateDBPaths();
  my $aDBName = $this->getDBName();
  if ( isTrue($aIsCreateDbPaths)) {
    my $instancePathSuccess = $this->createInstancePaths();
    unless($instancePathSuccess){
      $this->showStatusMessage("ERROR: Failed to create paths for database $aDBName.\n" );
      return undef;
    }
  }
  my $creationStrategy = $this->getCreationStrategy();
  if ( $creationStrategy == 1) {
    return $this->createInstanceInternal($doNotReuseInstance);
  } elsif ($creationStrategy == 2) {
    my  $goodness = $this->getGoodness();
    if ( $goodness == 0) {
      $this->writeLog("Cannot create $aDBName, because its status is  unknown. Goodess was " . 
                      $this->getGoodnessString() . "\n(Creation strategy was $creationStrategy.)" 
                     );
      $this->showStatusMessage("ERROR: Cannot create $aDBName, because its status is unknown.");
      return undef;
    } elsif ( $goodness > 1 ) {
      $this->showStatusMessage
        ("ERROR: Cannot create $aDBName, because it already exists. (Configuration does not allow deletion)" );

      $this->writeLog("Cannot create $aDBName, because it already exists. (Goodness: " . 
                      $this->getGoodnessString() . ")\n(Creation strategy was $creationStrategy, if " .
                      "you want to delete the database choose another creation strategy.)\n"
                     );
      return undef;
    } else {
      return $this->createInstanceInternal($doNotReuseInstance);
    }
  } elsif ($creationStrategy == 3) {
    my  $goodness = $this->getGoodness();
    if ( $goodness == 0, or $goodness == 2 or $goodness == 3) {
      $this->showStatusMessage("WARNING: Cannot create $aDBName, because existing instance cannot be deleted." );

      $this->writeLog("Cannot create $aDBName, because  I cannot delete already existing ". 
                      " instance.\nGoodness: " . 
                      $this->getGoodnessString() . "\n" 
                     );


    } elsif ($goodness == 1) {
      return $this->createInstanceInternal($doNotReuseInstance);
    } elsif ($goodness >= 3) {
      my $deleteSuccess = $this->deleteInstance();
      unless(defined $deleteSuccess){
        $this->showStatusMessage("ERROR: Cannot create $aDBName instance, because existing instance". 
                                 "cannot be deleted.\n"  
                                );
        $this->writeLog("Cannot create $aDBName instance, because  I cannot delete ". 
                        "already existing instance.\n"  
                       );
        return undef;
      }
      return $this->createInstanceInternal($doNotReuseInstance);
    }
  } elsif ($creationStrategy == 4) {
    my  $goodness = $this->getGoodness();
    if ( $goodness == 1) {
      return $this->createInstanceInternal($doNotReuseInstance);
    } elsif ($goodness == 4 or $goodness == 5) {
      my $deleteSuccess = $this->deleteInstance();
      unless(defined $deleteSuccess){
        $this->showStatusMessage("ERROR: Cannot create $aDBName instance, because existing instance". 
                                 "cannot be deleted.\n"  
                                );
        $this->writeLog("Cannot create $aDBName, because  I cannot delete ". 
                        "already existing instance.\n"  
                       );
        return undef;
      }
      return $this->createInstanceInternal($doNotReuseInstance);
    } elsif ($goodness >= 6) {
      return $this->createInstanceInternal($doReuseInstance);
    } elsif ($goodness == 0 or $goodness == 2 or $goodness == 3) {
      $this->showStatusMessage("ERROR: Cannot create $aDBName, status is invalid." );
      $this->writeLog("ERROR: Cannot create $aDBName:\n". 
                      "Status of  $aDBName is invalid, goodness " .  $this->getGoodnessString() );
      return undef;
    }
  } else {
    $this->writeLog("ERROR: Cannot create database: " . 
                    "Invalid value \"$creationStrategy\" for creation strategy.\n"
                   );
    $this->showStatusMessage("ERROR: Cannot create database because of configuration error." );
    return undef;
  }
}
#createInstance




#ifdef sub:createInstanceInternal
#<p>
#  At the moment, depending on the create strategy, the following steps a performed, when a database
#  instance is created.
#  (For most of the  steps  the <code>SAPDB::Install::DBMCmd</code>-Modul is used).
#  </p>
#  <ol>
#  <li>
#  Create instance using <code>dbmcli db_create</code>
#  </li>
#  <li>
#  Connect to the instance as database manager
#  </li>
#  <li>
#  Set all configured database parameters.
#  </li>
#  <li>
#  Add all configured database volumes.
#  </li>
#  <li>
#  Change the instance to admin state.
#  </li>
#  <li>
#  Do <code>db_activate</code>
#  </li>
#  <li>
#  Do load_systabs
#  </li>
#  <li>
#  Do db_online
#  </li>
#  <li>
#  Open an SQL-Connection as database manager.
#  </li>
#  <li>
#  Create one user.
#  </li>
#  <li>
#  Close the SQL-Connection.
#  </li>
#  </ol>
  #endif
  #+sub():void:Create the database instance
  sub createInstanceInternal{
    # Initialize configuration
    my ($this,$isReuse) = @_;
    my $dbname = $this->getDBName();
    my $mydependend = $this->getDependendPath();
    my $scriptHandler = $this->getScriptHandler();
    my $autoRestart = $this->getAutoRestart();
    #checks and directory creation
    $this->doSomeChecks or return undef;
    # do 'db_create'
    $this->doDBCreate($isReuse) or return undef;
    my $dbm = $this->createDBMCmd();
    my $dbmuser = $this->getDBM();
    my $dbmpass = $this->getDBMPass();
    $this->setStatusRunning("logon");
    defined $this->logonAsDBM($dbm) or 
      return $this->msgError("ERROR: Cannot create $dbname, logon as $dbmuser failed.\n","logon");
    $this->setStatusFinishedSuccess("logon");

    if ( $isReuse) {
      $this->setStatusFinishedSuccess("param_put");
      $this->setStatusFinishedSuccess("param_addvolume");
    } else {
      $this->showStatusMessage("Setting database parameters.\n");
      $this->dbmcmdSetDBParams($dbm) or return undef;
      $this->showStatusMessage("Adding volumes.\n");
      $this->setStatusRunning("param_addvolume");
      unless ( defined  $this->dbmcmdAddAllVolumes($dbm)) {
        $this->setStatusFinishedError("param_addvolume");
        return undef;
      }
      $this->setStatusFinishedSuccess("param_addvolume");
    }
    
    if ($this->getAutoUpdateStatistics or $this->getAutoExtend){
      $this->doRunEventDispatcher($dbm);
    }
    $this->showStatusMessage("Change to admin state.\n");
    $this->setStatusRunning("db_admin");
    unless (defined ($this->changeState($dbm,"db_admin"))){
      $this->setStatusFinishedError("db_admin");
      return undef;
    }
    $this->setStatusFinishedSuccess("db_admin");

    $this->setStatusRunning("db_activate");
    unless (defined ($this->db_activate($dbm))){
      $this->setStatusFinishedError("db_activate");
      return undef;
    }
    $this->setStatusFinishedSuccess("db_activate");
    $this->doEnableAutoFeatures($dbm) and return undef;
    $this->showStatusMessage("Loading system tables.\n");
    $this->setStatusRunning("load_systab");
    unless (defined ($this->load_systab($dbm))){
      $this->setStatusFinishedError("load_systab");
      return undef;
    }
    $this->setStatusFinishedSuccess("load_systab");
    if (isTrue($this->getLoadSampleSchema())) {
      $this->setStatusRunning("load_sample_schema");
      if ($this->doLoadSampleSchema($dbm)) {
        $this->setStatusFinishedSuccess("load_sample_schema");
      } else {
        $this->setStatusFinishedError("load_sample_schema");
      }
    }

    if (isTrue($this->getAutoSaveLog())) {
      $this->setStatusRunning("init_auto_save_log");
      if ($this->doInitAutoSaveLog($dbm) == 0) {
        $this->setStatusFinishedSuccess("init_auto_save_log");
      } else {
        $this->setStatusFinishedError("init_auto_save_log");
      }
    }
    $this->setStatusRunning("finalizing");
    my $isLogOverWrite = $this->getAutoOverWriteLog();
    if ( isTrue($isLogOverWrite)) {
      $this->switchOnAutoOverWriteLog($dbm);
    }
    my $isAutoExtend = $this->getAutoExtend();
    if ( isTrue($isAutoExtend)) {
      #    $this->showStatusMessage("Setting auto extend feature on.\n");
      $this->switchOnAutoExtension($dbm);
    }
    unless($isReuse){
      if (defined($this->getSQLU())) {
        $this->createSQLUser($dbm);
      }
    }
    if ($isReuse) {
      $this->showStatusMessage("Finished refreshing  $dbname.\n");
    } else {
      $this->showStatusMessage("Finished creating   $dbname.\n");
    }
    $this->setStatusFinishedSuccess("finalizing");
    return 0;
  }
#createInstanceInternal




  #optionally enable auto update statistics and auto extension
  sub doEnableAutoFeatures{
    my ($this,$dbm) = @_;
    my $version = $this->getDBMVersion($dbm);
    if($version lt 7.6){
      return $this->doEnableAutoFeatures75($dbm);
    }
    else{
#       my $msg = "WARNING: " .
#                       "The dbm commands 'auto_update_statistics' and 'auto_extend' are buggy in release $version.\n" .
#                      "         They are not supported at them moment by the installation tool.\n" .
#                      "         This means that automatic update of statistics and auto extension of volumes will NOT\n" .
#                      "         be  enabled after the creation of the database instance " . $this->getDBName() . ".\n";
#       $this->writeLog($msg);
#      return 0;
      return $this->doEnableAutoFeatures76($dbm);
    }
    return 0;
  }
  sub doEnableAutoFeatures76{
    my ($this,$dbm) = @_;
    $this->writeLog("INFO: Using 7.6 way to initialize auto features!!!\n");
    my @res;
    if ($this->getAutoUpdateStatistics()) {
      $this->callDBM($dbm,"auto_update_statistics ON",\@res)and $this->writeLog(join "\n", @res);
    }
    if ($this->getAutoExtend()) {
      $this->callDBM($dbm,"auto_extend ON",\@res) and $this->writeLog(join "\n", @res);
    }
    return 0;
  }

  sub doEnableAutoFeatures75{
    my ($this,$dbm) = @_;
    if ($this->getAutoUpdateStatistics()) {
      if ($this->doEnableAutoStatEvent()) {
        my $message = "Enabled auto update statistics in event dispatcher file\n";
        $this->writeLog($message);
      } else {
        my $message = "WARNING: Could not create/update event dispatcher configuration " .
          "file for auto update statistics\n";
        $this->writeLog ($message);
      }
    }

    if ($this->getAutoExtend()) {
      if ($this->doEnableAutoExtendEvent()) {
        my $message = "Enabled automatic extension in event dispatcher file\n";
        $this->writeLog ($message);
      } else {
        my $message = "WARNING: Could not create/update event dispatcher configuration " .
          "file for automatic extension \n";
        $this->writeLog ($message);
      }
    }
    return 0;
  }


  














  sub doDBCreate{
    my ($this,$isReuse) = @_;
    my $retVal =1;
    my $dbname = $this->getDBName;
    my $mydependend = $this->getDependendPath;
    if ($isReuse) {
      $this->showStatusMessage("Trying to refresh existing database $dbname\n");
      $this->setStatusFinishedSuccess("db_create");
    } else {
      $this->setStatusRunning("db_create");
      my $aFlag = (isTrue($autoRestart) and ($^O =~ /mswin/i)) ? "-a" : "";
      if ( $^O =~ /mswin/i) {
        $mydependend =~ s/\//\\/g;
      }
      my $create_command =
        "-R \"" . $mydependend . 
          "\" db_create " . $aFlag . " " .  $this->getDBName() . " " . $this->getDBM() . ",". $this->getDBMPass();
      my $displayCommand = $create_command;
      $displayCommand =~ s/(\w+),\w+/$1,\*\*\*/;
      unless( defined $this->dbmcliCall( $create_command, 
                                         "Start  creating db instance $dbname.\n",
                                         "ERROR: Instance creation failed.\n",$displayCommand)){
        $this->setStatusFinishedError("db_create");
        return undef;
      }
      $this->setStatusFinishedSuccess("db_create");
    }
    return $retVal;
  }

  sub doSomeChecks{
    my $this = shift;
    my $mydependend = $this->getDependendPath();
    my $scriptHandler = $this->getScriptHandler();
    my $dbname = $this->getDBName;
    my $retVal = 1;
    my $msg;
    if ( not (defined $mydependend)) {
      return $this->msgError( "ERROR: Cannot create $dbname: \"Dependend path\" is not set.\n","db_create");
      return undef;
    }
    unless(defined $scriptHandler){
      if ( not -d $mydependend) {
        return $this->msgError("ERROR: Cannot create $dbname: " . 
                               "Dependend path \"$mydependend\" does not exist or is not a directory.\n" ,"db_create");
      }
      my $volumes = $this->getDBVolumes();
      #Check for duplicate volume names
      my %nameChecker;
      foreach my $volume (@$volumes) {
        my $path  = $this->volumePath($volume);
        my $volName = basename($path);
        if ($nameChecker{$volName}) {
          return $this->msgError("ERROR: duplicate volume name  $volName:  Will not create database $dbname\n","db_create") ;
        } else {
          $nameChecker{$volName}=1;
        }
      }
      #create directories for volumes
      foreach my $volume (@$volumes) {
        unless( $this->createVolumeForPath($volume)){
          my $path  = $this->volumePath($volume);
          return $this->msgError( "ERROR: Cannot create $dbname: Unable to create directory for $path\n", "db_create");
        }
      }
      #perform space check
      my $fsresult = $this->checkVolumeSize();
      foreach my $key (keys %$fsresult) {
        my $fsdata  = $fsresult->{$key}; 
        my $neededkb = $fsdata->{"needed"};
        my $availablekb = $fsdata->{"available"};
        my $totalkb = $fsdata->{"total"};
        if ($neededkb > $availablekb) {
          my ($needed,$neededUnit)=readableUnit($neededkb,"KB"); 
          my ($available,$availableUnit)=readableUnit($availablekb,"KB"); 
          my ($total,$totalUnit)=readableUnit($totalkb,"KB"); 
          return $this->msgError(
                                 "Not enough space on \"$key\"  to create log and data volumes " .
                                 "(Available: $available $availableUnit, " .
                                 "Needed $needed $neededUnit). Cannot create database instance $dbname.\n",
                                 "db_create"
                                );
        }
      }                         #foreach($fsresults)
    }                           #unless defined ($scriptHandler)
    return $retVal;
  }                             #doSomeChecks

  sub msgError{
    my ($this,$msg,$status)  = @_;
    $msg and $this->showStatusMessage ($msg);
    $msg and $this->writeLog ($msg);
    $status and $this->setStatusFinishedError($status);
    return undef;
  }

sub doEnableAutoStatEvent(){
  my($this) =@_;
  return $this->doEnableEvent('AUTO_UPDATE_STATISTICS');
}

sub doEnableAutoExtendEvent(){
  my($this) =@_;
  return $this->doEnableEvent('AUTO_EXTEND');
}



sub doEnableEvent(){
  my($this,$feature) =@_;
  my $dbname = $this->getDBName();
  my $retVal=1;
  my $success = AddDBEvtHandler($dbname,$feature);
  if ($success ) {
    #  $this->writeLog("AddDBEvtHandler returned $success for feature $feature\n");
  } else {
    $retVal=0;
    $this->writeLog("AddDBEvtHandler returned false for feature $feature\n");
  }
  return $retVal;
}

sub doRunEventDispatcher(){
  my($this,$dbm) =@_;
  my $dba = $this->getDBA();
  my $dbapass = $this->getDBAPass();
  my $version = $this->getDBMVersion($dbm);
   if($version gt '7.5'){
#     $this->writeLog("INFO: No explicit start of event dispatcher in release $version necessary\n");
     return 1;
   }
  my $cmd = "dbm_configset RunEventDispatcher yes"; 
  unless ( defined $this->dbm
           ($dbm,$cmd,
            "ERROR: Failed to configure event dispatcher. \n")){
    return 0;
  }
  return 1;
}


sub doInitAutoSaveLog(){
  my($this,$dbm) =@_;
  my $retVal=1;
  $retVal = $this->performInitialBackup($dbm);
  if ($retVal == 0) {
    $retVal = $this->switchOnAutoSaveLog($dbm);
  } else {
    #    print "Initial backup returned $retVal\n";
  }
  return $retVal;
}

#for display in the summary:
sub createInitialBackupPath{
  my($this) =@_;
  my $dbname = $this->getDBName();
  my $backupDir = $this->getInitialBackupDir();
  my $backupName = $dbname . ".dat";
  my $backupPath=undef;
  if (defined $backupDir) {
    $backupPath = catfile($backupDir,$backupName);
  } else {
    $backupPath = $backupName;
  }
  return $backupPath;
}
sub performInitialBackup(){
  my($this,$dbm) =@_;
  my $retVal = 0;
  my $dbname = $this->getDBName();
  my $backupDir = $this->getInitialBackupDir();
  my $backupName = $dbname . ".dat";
  my $backupPath=undef;
  my $mediumName = "INITIALBACKUP";
  if (defined $backupDir) {
    my $makeDir = yamakedir($backupDir);
    if ($makeDir > 1) {
      $retVal=1;
    }
    $backupPath = catfile($backupDir,$backupName);
  } else {
    $backupPath = $backupName;
  }

  if ($retVal == 0) {
    if ( $backupPath =~ / /) {
      $backupPath = "\"$backupPath\""; 
    }
    my $mediumDeclaration =  "$mediumName $backupPath  FILE DATA 0 8 YES";
    my $mediumPutCommand = "medium_put " . $mediumDeclaration; 
    unless(defined $this->dbm($dbm,$mediumPutCommand,"Could not put medium $mediumName\n")){
      $retVal=2;
    }
  }

  if ( $retVal == 0) {
    unless(defined $this->dbm($dbm,"util_connect", 
                              "Could not start util session\n")){
      $retVal=3;
    }
  }
  if ( $retVal == 0) {
    unless(defined $this->async_dbm($dbm,"backup_start $mediumName DATA", 
                                    "Backup to $backupPath failed\n")){
      $retVal=4;
    }
  }
  unless($retVal == 3){
    unless(defined $this->dbm($dbm,"util_release" , "Could not release util session.\n")){
      $retVal=5;
    }
  }


  return $retVal;
}                               #performInitialBackup


sub createLogSavePath{
  my($this) =@_;
  my $dbname = $this->getDBName();
  my $logSaveDir = $this->getLogSaveDir();
  my $logBackupName = $dbname . "_log";
  my $logBackupPath=undef;
  if (defined $logSaveDir) {
    $logBackupPath = catfile($logSaveDir,$logBackupName);
  } else {
    $logBackupPath = $logBackupName;
  }
  return $logBackupPath;
}

sub switchOnAutoSaveLog(){
  my($this,$dbm) =@_;
  my $retVal = 0;
  my $dbname = $this->getDBName();
  my $logSaveDir = $this->getLogSaveDir();
  my $logBackupName = $dbname . "_autosavelog";
  my $logBackupPath=undef;
  my $mediumName = "LOGBACKUP";
  if (defined $logSaveDir) {
    my $makeDir = yamakedir($logSaveDir);
    if ($makeDir > 1) {
      $retVal=1;
    }
    $logBackupPath = catfile($logSaveDir,$logBackupName);
  } else {
    $logBackupPath = $logBackupName;
  }

  if ($retVal == 0) {
    if ( $logBackupPath =~ / /) {
      $logBackupPath = "\"$logBackupPath\""; 
    }
    my $mediumDeclaration =  "$mediumName $logBackupPath  FILE LOG 0 8 YES";
    my $mediumPutCommand = "medium_put " . $mediumDeclaration; 
    unless(defined $this->dbm($dbm,$mediumPutCommand,"Could not put medium $mediumName\n")){
      $retVal=2;
    }
  }

  if ( $retVal == 0) {
    unless(defined $this->dbm($dbm,"util_connect", "Could not start util session\n")){
      $retVal=3;
    }
  }
  if ( $retVal == 0) {
    unless(defined $this->dbm($dbm,"autolog_on $mediumName", "Failed to set autolog on\n")){
      $retVal=4;
    }
  }
  unless($retVal == 3){
    unless(defined $this->dbm($dbm,"util_release" , "Could not release util session.\n")){
      $retVal=5;
    }
  }
  return $retVal;
}                               #switchOnAutoSaveLog



sub doLoadSampleSchema(){
  my($this,$dbm) =@_;
  my $dba = $this->getDBA();
  my $dbapass = $this->getDBAPass();

  #,""C:\program files\sdb\MAXDB1\db\bin\x_python" "C:\program files\sdb\MAXDB1\db
  #\env\ltutorial.py" -R "C:\program files\sdb\MAXDB1\db" -d MAXDB1 -u dba,*"
  #HOTELDB_COMPLETE

  my $cmd = "load_tutorial" ;
  my $displayCmd = "load_tutorial";

  #   my $cmd = "exec_load " . $dba ."," . $dbapass . " ../demo/ltutorial.py";
  #   my $displayCmd = "exec_load " . $dba .",*** ../demo/ltutorial.py";

  unless ( defined $this->async_dbm
           ($dbm,$cmd,
            "ERROR: Failed to load tutorial db schema \n",
            "Tutorial db schema loaded successfully.\n",
            undef,undef,$displayCmd)){
    return 0;
  }
  return 1;
}

sub createSQLUser{
  my ($this,$dbm) = @_;
  my $dbname = $this->getDBName();
  my $sqluser = $this->getSQLU();
  my $sqlpassword = $this->getSQLPass();
  $this->showStatusMessage("Creating default user.\n");
  unless (defined ($this->sql_connectAsDBA($dbm))){
    return undef;
  }
  my $user_statement = "CREATE USER " .  $sqluser . " PASSWORD " . $sqlpassword
    . " DBA NOT EXCLUSIVE";
  $this->dbm($dbm,"sql_execute " . $user_statement ,
             "ERROR: Failed to create default user \"$sqluser\".\n",
             "User \"$sqluser\" with initial password \"$sqlpassword\" created.\n");

  
  my $dbaname = $this->getDBA();
  unless (defined ($this->dbm
                   ($dbm,
                    "sql_release",
                    "ERROR: Failed to close sql connection to $dbname.\n",
                    "SQL-Connection  to $dbname as $dbaname is now closed.\n"
                   ))){
    return undef;
  }
  return 1;
}
#createSQLUser

 

sub changeState($$$){
  my ($this,$dbm,$state)=@_;
  my $retVal;
  unless ( defined $this->async_dbm
           ($dbm, "$state",  "ERROR: Error in  $state .\n")) {
    return undef;
  }
  my   @stateResult = $this->dbm($dbm, "db_state",  "ERROR: db_state failed after $state.\n" );
  my $aResultState = $stateResult[2];
  if (defined $aResultState) {
    $this->writeLog("Database state is now $aResultState\n");
    return $aResultState;
  } else {
    return undef;
  }
}
#changeState


#ifdef sub:deleteInstance
#<p>
#  Shut down the instance and do db_drop
#  </p>
  #endif
  #+sub():void:Delete the database instance
  sub deleteInstance(){
    my ($this) = @_;


    my $dbname = $this->getDBName();
    my $goodness = $this->getGoodness();
    unless($goodness >= 4){
      my $goodnessString = $this->getGoodnessString();
      $this->showStatusMessage("Cannot delete $dbname ([$goodness]: $goodnessString)\n");
      return undef;
    }



    my $dbm = $this->createDBMCmd();
    unless ( defined $this->logonAsDBM($dbm)) {
      $this->showStatusMessage("Cannot delete $dbname (Connection failed)\n");
      return undef;
    }
    $this->showStatusMessage("Start deleting instance $dbname\n");

    $this->changeState($dbm,"db_offline");
    unless (defined ($this->async_dbm
                     ($dbm,
                      "db_drop",
                      "ERROR: Failed to drop instance $dbname.\n",
                      "Instance $dbname is now dropped.\n"))){ 
      return undef;
    } 
    $this->showStatusMessage("Instance $dbname deleted.\n");
    return 0;
  }
#deleteInstance(..)



sub logonAsDBM{
  my ($this,$dbm) = @_;
  my $dbname = $this->getDBName();
  my $dbmuser = $this->getDBM();
  my $dbmpass = $this->getDBMPass();
  my $aUserLogon = "user_logon " . $dbmuser . "," . $dbmpass;

  my $displayCommand = $aUserLogon;
  $displayCommand =~ s/(\w+),\w+/$1,\*\*\*/;

  unless ( defined $this->dbm
           ($dbm, $aUserLogon,  "ERROR: Logon as $dbmuser failed.\n",
            undef, undef,undef,$displayCommand)) {
    return undef;
  }
}

#+sub():void:Create a Backup
sub backup(){
  my ($this) = @_;
  my $dbm = $this->createDBMCmd();
  my $aBackup = $this->getBackupFile();
  my $dbname = $this->getDBName();
  my $mediumDeclaration =  "SDBINSTBACKUP " . $aBackup . " FILE DATA 0 8 YES";
  my @tags = split (/\s+/,$mediumDeclaration);
  my $mediumName = $tags[0];
  my $dbmuser = $this->getDBM();
  my $dbmpass = $this->getDBMPass();
  unless ( defined $this->logonAsDBM($dbm)) {
    return undef;
  }
  my $mediumPutCommand = "medium_put " . $mediumDeclaration; 
  unless(defined $this->dbm($dbm,$mediumPutCommand)){
    return undef;
  }
  #do util_connect
  unless( defined $this->dbm ($dbm,
                              "util_connect " .  $this->getDBM() . "," . $this->getDBMPass() ,
                              "ERROR:  \"util_connect  \" failed.\n"
                             )){
    $this->dbm($dbm,"util_release","\"util_release\" failed.\n");
    return undef;
  }
  unless( defined $this->dbm ($dbm,
                              "backup_save " . $mediumName,
                              "ERROR:  \"backup failed\" failed.\n",
                              "Backup successfull.\n",
                              "",
                              "Starting backup ...\n"
                             )){
    $this->dbm($dbm,"util_release","\"util_release\" failed.\n");
    return undef;
  }
  #end util session 
  $this->dbm($dbm,"util_release","\"util_release\" failed.\n");
  #delete medium
  $this->dbm($dbm,"medium_delete " . $mediumName,
             "Could not delete medium $mediumName\n");
  return 0;
}
#backup()

#Instance-Method: Needs DBMCmd instance as arg
sub db_drop ($$){
  my ($this, $dbm) = @_;
  return $this->dbm($dbm,"db_drop") ;
}
#db_drop()


sub load_systab($$){
  my ($this, $dbm) = @_;
  my $adba = $this->getDBA();
  my $adbaPass = $this->getDBAPass();
  my $aDomainPass = $this->getDomainPass();
  my $loadCommand = "load_systab -u $adba,$adbaPass -ud $aDomainPass";
  my $displayCommand = $loadCommand;
  $displayCommand =~ s/(\w+),\w+/$1,\*\*\*/;

  return $this->async_dbm
    ($dbm,
     $loadCommand,
     "ERROR: Failed loading system tables.\n",
     "System tables are loaded.\n",
     undef,undef,$displayCommand) ;
}
#load_systab()


sub medium_put($$){
  my ($this, $dbm) = @_;
  my $mediumDeclaration = $this->getMediumDeclaration();
  my $mediumPutCommand = "medium_put " . $mediumDeclaration; 
  return $this->dbm($dbm,$mediumPutCommand)
}
#medium_put()



sub dbm{
  my ($this ) = @_;
  my $scriptHandler = $this->getScriptHandler();
  if ( defined $scriptHandler) {
    return &$scriptHandler(@_);
  } else {
    return defaultDbmcliHandler(@_);
  }
}
#dbm()

sub async_dbm{
  my ($this ) = @_;
  my $scriptHandler = $this->getScriptHandler();
  if ( defined $scriptHandler) {
    return &$scriptHandler(@_);
  } else {
    return asyncDbmcliHandler(@_);
  }
}
#dbm()


sub asyncDbmcliHandler{
  my ($this, $dbm,$command, $errorMessage,$successMessage,$showResult,$beforeMessage,$displayCommand) = @_;
  if ( defined $beforeMessage) {
    $this->writeLog($beforeMessage);
  }
  if (defined $displayCommand) {
    $this->writeLog("[$displayCommand]\n");
  } else {
    $this->writeLog("[$command]\n");
  }
  my $waitTime = $this->getAsyncWaitTime();
  my $fatalError;
  my $go=1;
  my $output  = undef;
  my $id = $dbm->send ($command);
  my $startTime = time();
  while ($go) {
    my $retVal = $dbm->select($waitTime);
    my $newTime = time()-$startTime;
    unless(defined $retVal){
      $go=0;
      $fatalError = "Fatal error in dbm: $command\n";
      next;
    }
    if ($retVal == $id ) {
      $go=0;
      $output  = $dbm->recv ($id);
      $this->sendAsyncCallback($newTime,".[FINISHED]\n");
    } elsif ($retVal == 0) {
      $this->sendAsyncCallback($newTime,".");
    }
  }
  return $this->handleDbmOutput($dbm,$command, $errorMessage,$successMessage,
                                $showResult,$beforeMessage,$output,$fatalError,$displayCommand);
}                               #asyncDbmcliHandler



sub defaultDbmcliHandler{
  my ($this, $dbm,$command, $errorMessage,$successMessage,$showResult,$beforeMessage,
      $displayCommand) = @_;
  if ( defined $beforeMessage) {
    $this->writeLog($beforeMessage);
  }
  if (defined $displayCommand) {
    $this->writeLog("[$displayCommand]\n");
  } else {
    $this->writeLog("[$command]\n");
  }
  my $output = $dbm->exec($command);
  return $this->handleDbmOutput($dbm,$command, $errorMessage,
                                $successMessage,$showResult,$beforeMessage,$output,undef,$displayCommand);
}
#defaultDbmcliHandler(..)




sub callDBM{
  my ($this, $dbm,$command,$outputLines ) = @_;
  $this->writeLog("[$command]\n");
  my $output = $dbm->exec($command);
  my $retVal = 0;
  @$outputLines = split ('\n', $output);
  my $result = $outputLines->[0];
  $result =~ /^OK/ or $retVal = 1;
  return $retVal;
}
#defaultDbmcliHandler(..)




sub handleDbmOutput{
  my ($this, $dbm,$command, $errorMessage,
      $successMessage,$showResult,$beforeMessage,
      $output,$fatalError,$displayCommand) = @_;
  if (defined $fatalError) {
    $this->writeLog($fatalError);
    return undef;
  }
  unless(defined $output){
    return undef;
  }
  my @outputLines = split ('\n', $output);
  my $result = $outputLines[0];
  my $err = $outputLines[1];
  my @retVal =() ;
  my $success=undef;
  if ($result =~ /^OK/) {
    $success=1;
    if ( defined $successMessage) {
      $this-> writeLog($successMessage);
    }
    if ( defined $showResult) {
      $this-> writeLog($output);
    }
    @retVal = @outputLines;
    $this->setLastDBMMessage(undef);
    $this->setLastDBMError(undef);
  } else {
    if ( defined $errorMessage) {
      $this-> dbmcmdError($dbm,$errorMessage,$displayCommand);
    } else {
      $this->dbmcmdError($dbm,"",$displayCommand);
    }
    if ( defined $showResult) {
      $this-> writeLog($output);
    }
  }
  if ( defined $success) {
    return @retVal
  }
  return undef;
}                               #handleDbmOutput






#Instance-Method: Needs DBMCmd instance as arg
sub db_activate($$){
  my ($this, $dbm) = @_;
  my $pipeName = $this->getRecoverFromPipe();
  my $aBackup = $this->getBackup();
  my $aArchive = $this->getArchive();
  my $aRetVal;
  if ( defined $pipeName) {
    if ( defined $aBackup and defined $aArchive) {
      $this->writeLog("Reading backup  from archive \"$aArchive\".\n");
      $aRetVal = $this->db_activateWithBackupArchive($dbm);
    }
  } elsif ( defined $aBackup) {
    $this->showStatusMessage("Reading backup from file.\n");
    $aRetVal = $this->db_activateFromFile($dbm);
    if (defined  $aRetVal) {
      $aRetVal = $this->changeState($dbm,"db_online");
    }
  } else {
    $this->showStatusMessage("No backup specified: No backup will be loaded.\n");
    $aRetVal =  $this->db_activateNormal($dbm);
  }
  return $aRetVal;
}
#db_activate

#Normal activation
sub db_activateNormal ($$){
  my ($this, $dbm) = @_;

  my $activateCommand  = "db_activate " . $this->getDBA() . "," . $this->getDBAPass();

  $this->showStatusMessage("Now start db activation (This can take a while, be patient.)\n");

  my $displayCommand = $activateCommand;
  $displayCommand =~ s/(\w+),\w+/$1,\*\*\*/;



  return $this->async_dbm($dbm,$activateCommand, "ERROR: DB-Activation failed: \"$activateCommand\"\n",
                          undef,undef, "Start db activation (This can take a while, be patient.)\n",$displayCommand);
}
#db_activate

#db_activate recover with reading of bacup
sub db_activateWithBackupArchive{
  #Build Strings:
  my ($this, $dbm) = @_;
  my $pipeIdentifier = $this->getRecoverFromPipe();
  my $mediumDeclaration = $this->getMediumDeclaration();
  my @tags = split (/\s+/,$mediumDeclaration);
  my $pipeFile = $tags[1];
  #  my $activateCommand = "db_activate RECOVER " . $pipeIdentifier . " DATA";
  my $recoverCommand = "recover_start " . $pipeIdentifier ;
  my $backup = $this->getBackup();
  my $archive = $this->getArchive();
  my $scriptHandler = $this->getScriptHandler();

  if ( defined $scriptHandler) {
    $this->medium_put($dbm);
    $this->dbm ($dbm, "util_connect");
    &$scriptHandler(undef,undef,$recoverCommand);
    $this->dbm($dbm,"util_release");
    $this->dbm($dbm,"medium_delete " . $pipeIdentifier );
    return 1;
  }

  unless(defined $this->db_activateNormal($dbm)){
    return undef;
  }
  unless(defined $this->changeState($dbm,"db_offline")){
    return undef;
  }
  unless(defined $this->changeState($dbm,"db_admin")){
    return undef;
  }


  #Check some parameters
  unless (defined $backup){
    $this-> showStatusMessage("ERROR: Backup is not specified.\n");
    return undef;
  }
  unless (defined $archive){
    $this-> showStatusMessage("ERROR: Archive is not specified.\n");
    return undef;
  }
  #Open archive
  my $untgz = $this->getArchiveUntgz();
  my $archiveSuccess = 0;
  unless(defined $untgz){
    $untgz = SAPDB::Install::Untgz::new ();
    $archiveSuccess = $untgz->Open($archive);
  }
  unless (defined $archiveSuccess 
          && $archiveSuccess == 0){
    $this-> writeLog("Cannot open archive.\n");
    return undef;
  }
  #postion tgz cursor on backup
  my $aBackup;
  $untgz->Rewind();
  my @hrd = $untgz->Next();
  while (1) {
    unless(defined $hrd[0]){
      last;
    }
    my $maybeBackup = $hrd[3]; 
    if ( (uc $backup) eq (uc $maybeBackup)) {
      $aBackup = $maybeBackup;
      last;
    }
    @hrd = $untgz->Next();
  }
  #Check if backup was found in archive
  unless (defined $aBackup){
    $this->showStatusMessage ("Backup $backup  not found.\n(Archive was  \"$archive\".)\n");
    return undef;
  }
  #Open pipe
  my $bup = SAPDB::Install::BackupPipe->new ($pipeFile, 'w');
  unless (defined $bup){
    $this->showStatusMessage ("Could not create pipe $pipeFile\n");
    return undef;
  }
  #do medium_put 
  unless(defined $this->medium_put($dbm)){
    return undef;
  }
  #do util_connect
  unless( defined $this->dbm ($dbm,
                              "util_connect",
                              "ERROR:  \"util_connect\" failed.\n"
                             )){
    return undef;
  }

  unless( defined $this->dbm ($dbm,
                              "util_execute CLEAR LOG",
                              "ERROR:  \"util_execute CLEAR LOG\" failed.\n"
                             )){
    return undef;
  }


  # Start db_activate RECOVER ...
  my $activateId = $dbm->send ($recoverCommand);
    
  $this->writeLog ("[$recoverCommand]\n");


  # Open pipe
  unless(defined $bup->Open()){
    $this->showStatusMessage ("ERROR: Could not open pipe $bup\n");
    return undef;
  }

  # Write from TGZ-File into pipe
  $this->showStatusMessage ("Reading backup from archive (This can take a while, be patient.).\n");
  for (;;) {
    my ($got, $want, $buff);

    $want = 32 * 1024;
    $got = $untgz->ExtractRead ($buff, $want);
    unless (defined $got) {
      $this->showStatusMessage ("ERROR: Could not read from $archive\n");
      last;
    }

    if ($got <= 0) {
      last;
    }

    $want = $got;
    $got = $bup->Write ($buff, $want);
    if ($got != $want) {
      $this->showStatusMessage ("ERROR: Write failed to $pipeFile(Wanted : $want, got $got).\n");
      last;
    } else {
      #      $this->writeLog ("Wrote $want bytes\n");
    }

    if ($activateId == $dbm->select (0)) {
      $this->showStatusMessage ("ERROR, died before end of write\n: \"$recoverCommand \"\n");
      last;
    }
  }
  #close pipe
  $bup->Close();

  #Get Result of db_activate
  my $activateResult  = $dbm->recv ($activateId);
  my @activateLines =  split ('\n',$activateResult );
  my $activateSuccess = $activateLines[0];
  my $aReturnValue;
  if ($activateSuccess =~ /^OK/) {
    $aReturnValue = $activateSuccess;
    $this->writeLog("Result of \"$recoverCommand\":\n $activateResult\n");
  } else {
    $this->writeLog("Result of \"$recoverCommand\":\n $activateResult\n");
  }


  #end util session 
  $this->dbm($dbm,"util_release","\"util_release\" failed\n");
  #delete medium
  $this->dbm($dbm,"medium_delete " . $pipeIdentifier,
             "Could not delete medium $pipeIdentifier\n");



  unless(defined $this->changeState($dbm,"db_offline")){
    return undef;
  }
  unless(defined $this->changeState($dbm,"db_online")){
    return undef;
  }
  return $aReturnValue;
}
#db_activateWithBackupArchive

#db_activate recover from file
sub db_activateFromFile ($$){
  #Build Strings:
  my ($this, $dbm) = @_;
  my $mediumDeclaration = $this->getMediumDeclaration();
  my @tags = split (/\s+/,$mediumDeclaration);
  my $mediumName = $tags[0];
  my $activateCommand = "db_activate RECOVER " . $mediumName . "";
  my $backup = $this->getBackup();
  my $scriptHandler = $this->getScriptHandler();

  #Check some parameters
  unless (defined $backup){
    unless(defined $scriptHandler){
      $this-> writeLog("ERROR: Backup is not specified.\n");
      return undef;
    }
  }
  unless (-f $backup){
    unless(defined $scriptHandler){
      $this-> writeLog("ERROR: Backup \"$backup\" not found.\n");
      return undef;
    }
  }

  #do medium_put 
  unless(defined $this->medium_put($dbm)){
    return undef;
  }


  #do util_connect
  unless( defined $this->dbm ($dbm,
                              "util_connect",
                              "ERROR:  \"util_connect\" failed.\n"
                             )){
    return undef;
  }



  #Start db_activate RECOVER
  my $aReturnValue = $this->dbm($dbm,
                                $activateCommand,
                                "Error: DB-Activation failed.");

  #end util session 
  $this->dbm($dbm,"util_release","\"util_release\" failed.\n");
  #delete medium
  $this->dbm($dbm,"medium_delete " . $mediumName,
             "Could not delete medium $mediumName\n");
  return $aReturnValue;
}
#db_activateFromFile()



#Instance-Method: Needs DBMCmd instance as arg
sub sql_connectAsDBA ($$){
  my ($this, $dbm) = @_;
  my $aName = $this->getDBA();
  my $command = "sql_connect " . $this->getDBA() . ",". $this->getDBAPass();

  return $this->dbm
    ($dbm,
     $command,
     "Failed to open sql connection as $aName.\n",
     "SQL-Connection as $aName established.\n");
}


#Instance-Method: Needs DBMCmd instance as arg
#Sets log auto overwrite to on
sub switchOnAutoOverWriteLog($$){
  my ($this, $dbm) = @_;
  my $aName = $this->getDBName();
  return $this->dbm
    ($dbm,
     "db_execute SET LOG AUTO OVERWRITE ON",
     "Failed to switch on automatic log overwrite for database $aName.\n",
     "Automatic log overwrite is now ON.\n");
}
#switchOnAutoOverWriteLog


#Instance-Method: Needs DBMCmd instance as arg
#Sets auto extension to on
sub switchOnAutoExtension($$){
  my ($this, $dbm) = @_;
  my $aName = $this->getDBName();
  my $version = $this->getDBMVersion($dbm);
   if($version gt '7.5'){
#     $this->writeLog("INFO: No explicit setting of AutoExtFlag in  $version necessary\n");
     return 1;
   }
  return  $this->dbm($dbm,
                     "dbm_configset AutoExtFlag yes",
                     "Failed to switch on the automatic data area extension feature for database $aName.\n",
                     "Automatic data area extension  is now ON.\n");
}
#switchOnAutoExtension






# Fuehrt einen einzelnen DBMCLI-Befehl auf der Kommandozeile aus
sub dbmcliCall{
  my ($this,$command,$title,$errorMessage) = @_;
  my $scriptHandler = $this->getScriptHandler();
  if ( defined $scriptHandler) {
    return &$scriptHandler(undef,undef,$this->getDbmcliExe(). " " . $command);
  } else {
    return defaultDbmcliCall(@_);
  }
}
#dbmcliCall(..)



sub getDbmcliExe{
  my ($this) = @_;
  my $dbmcliExe = "dbmcli";
  my ($indep_data,$indep_prog) =  readIndepPath();
  if (defined $indep_prog) {
    my $aDbmcli = undef;
    if ($^O =~ /mswin/i) {
      $aDbmcli = catfile($indep_prog,"pgm","dbmcli.exe");
    } else {
      $aDbmcli = catfile($indep_prog,"bin","dbmcli");
    }
    if ( -f $aDbmcli) {
      $dbmcliExe = $aDbmcli;
    }
  }
  return $dbmcliExe;
}

# Fuehrt einen einzelnen DBMCLI-Befehl auf der Kommandozeile aus
sub defaultDbmcliCall{
  my ($this,$command,$title,$errorMessage,$displayCommand) = @_;
  my $success=1;
  my $dbmcliExe = $this->getDbmcliExe();


  my ($indep_data,$indep_prog) =  readIndepPath();
  if (defined $indep_prog) {
    my $aDbmcli = undef;
    if ($^O =~ /mswin/i) {
      $aDbmcli = catfile($indep_prog,"pgm","dbmcli.exe");
    } else {
      $aDbmcli = catfile($indep_prog,"bin","dbmcli");
    }
    if ( -f $aDbmcli) {
      $dbmcliExe = $aDbmcli;
    }
  }


  my $dbmcliCall=$dbmcliExe . " " . $command;
  $this->showStatusMessage( $title  );
  if (defined $displayCommand) {
    $this->writeLog( "[$dbmcliExe $displayCommand]\n" );
  } else {
    $this->writeLog( "[$dbmcliCall]\n" );
  }
  callSystem($dbmcliCall,
             sub{
               my $line = $_[0];
               if ($line =~ m/^Err/i) {
                 $success=undef;
               }
               $this->writeLog($line);
               return 1;
             },
             1);
  if (not defined $success) {
    if (defined $errorMessage) {
      $this->showStatusMessage($errorMessage);
    }
    $this->showStatusMessage("(Error  executing $dbmcliCall)\n");
  }
  return  $success;
}
#defaultDbmcliCall

#+field(Path:BackupFile)
my $BackupFile="BackupFileKey";
#+sub(Path):void:Set BackupFile
sub setBackupFile{
  my ($this, $aValue) = @_;
  $this->{$BackupFile}=$aValue;
}
#+sub():Path:Get BackupFile
sub getBackupFile{
  my ($this ) = @_;
  return $this->{$BackupFile};
}


#+field(int:parallelConnections)
my $parallelConnections="parallelConnectionsKey";
#+sub(int):void:Set parallelConnections
sub setParallelConnections{
  my ($this, $aValue) = @_;
  $this->{$parallelConnections}=$aValue;
}
#+sub():int:Get parallelConnections
sub getParallelConnections{
  my ($this ) = @_;
  return $this->{$parallelConnections};
}

sub setAdvicedParallelConnections{
  my ($this, $aValue) = @_;
  willSet($this,$parallelConnections);
  $this->setParallelConnections($aValue);
  haveSet($this,$parallelConnections);
}

#+field(String:Debug)
my $Debug="DebugKey";
#+sub(String):void:Set Debug
sub setDebug{
  my ($this, $aValue) = @_;
  $this->{$Debug}=$aValue;
}
#+sub():String:Get Debug
sub getDebug{
  my ($this ) = @_;
  return $this->{$Debug};
}


#+field(boolean:autoSaveLog)
my $autoSaveLog="autoSaveLogKey";
#+sub(boolean):void:Set autoSaveLog
sub setAutoSaveLog{
  my ($this, $aValue) = @_;
  $this->{$autoSaveLog}=$aValue;
}
#+sub():boolean:Get recoveryMode
sub getAutoSaveLog{
  my ($this ) = @_;
  return $this->{$autoSaveLog};
}

sub setAdvicedAutoSaveLog{
  my ($this, $aValue) = @_;
  willSet($this,$autoSaveLog);
  $this->setAutoSaveLog($aValue);
  haveSet($this,$autoSaveLog);
}




#+field(bool:RunEventDispatcher)
my $RunEventDispatcher="RunEventDispatcherKey";
#+sub(bool):void:Set RunEventDispatcher
sub setRunEventDispatcher{
  my ($this, $aValue) = @_;
  $this->{$RunEventDispatcher}=$aValue;
}
#+sub():bool:Get RunEventDispatcher
sub getRunEventDispatcher{
  my ($this ) = @_;
  return $this->{$RunEventDispatcher};
}







#+field(Path:logSaveDir)
my $logSaveDir="logSaveDirKey";
#+sub(Path):void:Set logSaveDir
sub setLogSaveDir{
  my ($this, $aValue) = @_;
  $this->{$logSaveDir}=$aValue;
}
#+sub():Path:Get logSaveDir
sub getLogSaveDir{
  my ($this ) = @_;
  return $this->{$logSaveDir};
}


#+field(Path:initialBackupDir)
my $initialBackupDir="initialBackupDirKey";
#+sub(Path):void:Set initialBackupDir
sub setInitialBackupDir{
  my ($this, $aValue) = @_;
  $this->{$initialBackupDir}=$aValue;
}
#+sub():Path:Get initialBackupDir
sub getInitialBackupDir{
  my ($this ) = @_;
  return $this->{$initialBackupDir};
}

#+field(bool:loadSampleSchema)
my $loadSampleSchema="loadSampleSchemaKey";
#+sub(bool):void:Set loadSampleSchema
sub setLoadSampleSchema{
  my ($this, $aValue) = @_;
  $this->{$loadSampleSchema}=$aValue;
}
#+sub():bool:Get loadSampleSchema
sub getLoadSampleSchema{
  my ($this ) = @_;
  return $this->{$loadSampleSchema};
}

sub setAdvicedLoadSampleSchema{
  my ($this, $aValue) = @_;
  willSet($this,$loadSampleSchema);
  $this->setLoadSampleSchema($aValue);
  haveSet($this,$loadSampleSchema);
}







#+field(boolean:autoUpdateStatistics)
my $autoUpdateStatistics="autoUpdateStatisticsKey";
#+sub(boolean):void:Set autoUpdateStatistics
sub setAutoUpdateStatistics{
  my ($this, $aValue) = @_;
  $this->{$autoUpdateStatistics}=$aValue;
}
#+sub():boolean:Get autoUpdateStatistics
sub getAutoUpdateStatistics{
  my ($this ) = @_;
  return $this->{$autoUpdateStatistics};
}
sub setAdvicedAutoUpdateStatistics{
  my ($this, $aValue) = @_;
  willSet($this,$autoUpdateStatistics);
  $this->setAutoUpdateStatistics($aValue);
  haveSet($this,$autoUpdateStatistics);
}








#PARAMS
sub getParams{
  my ($this) = @_;
  my $aParams = 
    {
     "DBM"   => [
                 sub{} ,
                 sub{} ,
                 \&noFormatter,
                 "STRING",
                 "DBInstance", 
                 \&voidValidate,
                 {"priority" => 50
                 }
                ] ,
     "DBMNAME"   => [
                     sub{$this->setDBM($_[0]);} ,
                     sub{ return $this->getDBM();} ,
                     \&noFormatter,
                     "STRING",
                     "DBInstance", 
                     \&voidValidate,
                     {"priority" => 50, "transient" => 1
                     }
                    ] ,
     "DBA"   => [sub{},
                 sub{} ,
                 \&noFormatter,
                 "STRING",
                 "DBInstance", 
                 \&voidValidate ,
                 {"priority" => 50 
                 }
                ] ,
     "DBANAME"   => [sub{$this->setDBA($_[0]);},
                     sub{return $this->getDBA();} ,
                     \&noFormatter,
                     "STRING",
                     "DBInstance", 
                     \&voidValidate ,
                     {"priority" => 50, "transient" => 1
                     }
                    ] ,

     "GLOBALUSER"   => [sub{$this->setAdvicedGlobalUser($_[0]);},
                        sub{return $this->getGlobalUser();} ,
                        \&defaultFormatter,
                        "STRING",
                        "DBInstance", 
                        \&voidValidate ,
                        {"priority" =>  0
                        }
                       ] ,
     "GLOBALPASS"   => [
                        sub {$this->setAdvicedGlobalPassword($_[0]);},
                        sub {return $this->getGlobalPassword();},
                        \&defaultFormatter,
                        "STRING",
                        "DBInstance" ,
                        sub {
                          return $this->passwordValidator($_[0]);
                        } ,
                        {
                         "priority" => 10
                        }
                       ] ,

     "CONFIRMGLOBALPASS"   => [
                               sub {$this->setAdvicedConfirmGlobalPassword($_[0]);},
                               sub {return $this->getConfirmGlobalPassword();},
                               \&defaultFormatter,
                               "STRING",
                               "DBInstance" ,
                               sub {
                                 my $globalPassword = $this->getGlobalPassword();
                                 my $confirmGlobalPassword = $_[0];
                                 return $this->confirmPasswordValidator($globalPassword,$confirmGlobalPassword);
                               }
                               ,
                               {
                                "priority" => 0, "transient" => 1
                               }
                              ] ,

     "CONFIRMDBMPASS"   => [
                            sub {$this->setConfirmDBMPass($_[0]);},
                            sub {return $this->getConfirmDBMPass();},
                            \&defaultFormatter,
                            "STRING",
                            "DBInstance" ,
                            sub {
                              my $dbmPass = $this->getDBMPass();
                              my $confirmDBMPass = $_[0];
                              return $this->confirmPasswordValidator($dbmPass,$confirmDBMPass);
                            }
                            ,
                            {
                             "priority" => 0, "transient" => 1
                            }
                           ] ,

     "CONFIRMDBAPASS"   => [
                            sub {$this->setConfirmDBAPass($_[0]);},
                            sub {return $this->getConfirmDBAPass();},
                            \&defaultFormatter,
                            "STRING",
                            "DBInstance" ,
                            sub {
                              my $dbaPass = $this->getDBAPass();
                              my $confirmDBAPass = $_[0];
                              return $this->confirmPasswordValidator($dbaPass,$confirmDBAPass);
                            }
                            ,
                            {
                             "priority" => 0, "transient" => 1
                            }
                           ] ,

     "VOLSTRUCT"   => [
                       sub {$this->setVolStruct($_[0]);},
                       sub {return $this->getVolStruct();},
                       \&defaultFormatter,
                       "VOLUMELIST.COMPOUND",
                       "DBInstance" ,
                       \&voidValidate
                       ,
                       {
                        "priority" => 0, "transient" => 1
                       }
                      ] ,

     "VOLSTRUCT.SIMPLE"   => [
                              sub {
                                $this->setVolStructDir($_[0]);
                              },
                              sub {return $this->getVolStruct();},
                              \&defaultFormatter,
                              "VOLUMELIST.COMPOUND",
                              "DBInstance" ,
                              sub {return $this->volStructDirValidator($_[0]);}
                              ,
                              {
                               "priority" => 0, "transient" => 1
                              }
                             ] ,

     "VOLSTRUCT.MEDIUM"   => [
                              sub {
                                $this->setVolStructDirSize($_[0]);
                              },
                              sub {return $this->getVolStruct();},
                              \&defaultFormatter,
                              "VOLUMELIST.COMPOUND",
                              "DBInstance" ,
                              \&voidValidate
                              ,
                              {
                               "priority" => 0, "transient" => 1
                              }
                             ] ,






     "VOLSTRUCT.DATA"   => [
                            sub {$this->setVolStructData($_[0]);},
                            sub {return $this->getVolStructData();},
                            \&defaultFormatter,
                            "VOLUMELIST.COMPOUND",
                            "DBInstance" ,
                            sub {return $this->volStructDataValidator($_[0]);}
                            ,
                            {
                             "priority" => 0, "transient" => 1
                            }
                           ] ,

     "VOLSTRUCT.LOG"   => [
                           sub {$this->setVolStructLog($_[0]);},
                           sub {return $this->getVolStructLog();},
                           \&volStructLogValidator,
                           "VOLUMELIST.COMPOUND",
                           "DBInstance" ,
                           sub {return $this->volStructLogValidator($_[0]);}
                           ,
                           {
                            "priority" => 0, "transient" => 1
                           }
                          ] ,

     "ADMINUSERSTRUCT"   => [
                             sub {$this->setAdminUserStruct($_[0]);},
                             sub {return $this->getAdminUserStruct();},
                             \&defaultFormatter,
                             "USERLIST.COMPOUND",
                             "DBInstance" ,
                             #                                   \&voidValidate

                             sub {return $this->adminUserStructValidator($_[0]);}
                             ,
                             {
                              "priority" => 0, "transient" => 1
                             }
                            ] ,

     "GLOBAL_USER_STRUCT"   => [
                                sub {$this->setGlobalUserStruct($_[0]);},
                                sub {return $this->getGlobalUserStruct();},
                                \&defaultFormatter,
                                "USER.COMPOUND",
                                "DBInstance" ,
                                #                                   \&voidValidate
                                sub {return $this->globalUserStructValidator($_[0]);}
                                ,
                                {
                                 "priority" => 0, "transient" => 1
                                }
                               ] ,


     "LOGGINGSTRUCT"   => [
                           sub {$this->setLoggingStruct($_[0]);},
                           sub {return $this->getLoggingStruct();},
                           \&defaultFormatter,
                           "LOGGING.COMPOUND",
                           "DBInstance" ,
                           \&voidValidate
                           ,
                           {
                            "priority" => 0, "transient" => 1
                           }
                          ] ,
     "LOGGING_MODE"   => [
                          sub {},
                          sub {
                            my $retVal="<UNDEF>";
                            my $isAutoSaveLog = isTrue($this->getAutoSaveLog());
                            my $isAutoOverWriteLog = isTrue($this->getAutoOverWriteLog());
                            if ($isAutoSaveLog) {
                              $retVal = "Automatic Log Backup";
                            } elsif (not $isAutoOverWriteLog) {
                              $retVal = "Manual Log Backup";
                            } else {
                              $retVal="No Log Backup";
                            }
                          },
                          \&defaultFormatter,
                          "NUMBER",
                          "DBInstance" ,
                          \&voidValidate,
                          {
                           "priority" => 0,"transient" => 1
                          }
                         ] ,

     "PLANNED_INITIAL_BACKUP_PATH"   => [
                                         sub {},
                                         sub {
                                           my $retVal=$this->createInitialBackupPath();
                                           return $retVal;
                                         },
                                         \&defaultFormatter,
                                         "PATH",
                                         "DBInstance" ,
                                         \&voidValidate,
                                         {
                                          "priority" => 0,"transient" => 1
                                         }
                                        ] ,

     "PLANNED_LOG_SAVE_PATH"   => [
                                   sub {},
                                   sub {
                                     my $retVal=$this->createLogSavePath();
                                     return $retVal;
                                   },
                                   \&defaultFormatter,
                                   "PATH",
                                   "DBInstance" ,
                                   \&voidValidate,
                                   {
                                    "priority" => 0,"transient" => 1
                                   }
                                  ] ,

     "DOMAINPASS" => [
                      sub {$this->setDomainPass($_[0]);},
                      sub {return $this->getDomainPass();},
                      , \&defaultFormatter,
                      "STRING",
                      "DBInstance",
                      \&voidValidate ,
                      {"priority" => 50
                      }
                     ]   ,
     "SQLUSER"   => [sub{}
                     , sub{}
                     , \&noFormatter,
                     "STRING",
                     "DBInstance", 
                     \&voidValidate ,
                     {"priority" => 50
                     }
                    ] ,
     "SQLUSERNAME"   => [sub{$this->setSQLU($_[0]);}
                         , sub{$this->getSQLU();}
                         , \&noFormatter,
                         "STRING",
                         "DBInstance", 
                         \&voidValidate ,
                         {"priority" => 50, "transient" => 1
                         }
                        ] ,
     "DBNAME"   => [
                    sub {$this->setAdvicedDBName($_[0]);},
                    sub {return $this->getDBName();},
                    \&defaultFormatter,
                    "STRING",
                    "DBInstance" , 
                    sub{return $this->dbnameValidator($_[0]);},
                    {"priority" => 50
                    }
                   ] ,
     "ARCHIVE"  => [
                    sub {$this->setArchive($_[0]);},
                    sub {return $this->getArchive();},
                    \&stringFormatter, 
                    "PATH",
                    "DBInstance" ,
                    \&voidValidate,
                    {"priority" => 0
                    }
                   ]  ,
     "BACKUP"   => [
                    sub {$this->setBackup($_[0]);},
                    sub {return $this->getBackup();},
                    \&stringFormatter, 
                    "PATH",
                    "DBInstance" ,
                    \&voidValidate,
                    {"priority" => 0
                    }
                   ] ,
     "BACKUPFILE"   => [
                        sub {$this->setBackupFile($_[0]);},
                        sub {return $this->getBackupFile();},
                        \&stringFormatter, 
                        "PATH",
                        "DBInstance" ,
                        \&voidValidate,
                        {"priority" => 0
                        }
                       ] ,
     "AUTORESTART"  => [  # automatic db start on windows after reboot
                        sub {$this->setAutoRestart($_[0]);},
                        sub {return $this->getAutoRestart();},
                        \&booleanFormatter,
                        "BOOLEAN",
                        "DBInstance" ,
                        \&voidValidate,
                        {
                         "priority" => 0
                        }
                       ]  ,

     "AUTO_SAVE_LOG"  => [      # Automatic save log
                          sub {$this->setAdvicedAutoSaveLog($_[0]);},
                          sub {return $this->getAutoSaveLog();},
                          \&booleanFormatter,
                          "BOOLEAN",
                          "DBInstance" ,
                          \&voidValidate,
                          {
                           "priority" => 0
                          }
                         ]  ,

     "LOG_SAVE_DIR"  => [       # Directory for log backups
                         sub {$this->setLogSaveDir($_[0]);},
                         sub {return $this->getLogSaveDir();},
                         \&booleanFormatter,
                         "PATH",
                         "DBInstance" ,
                         \&voidValidate,
                         {
                          "priority" => 0
                         }
                        ]  ,

     "INITIAL_BACKUP_DIR"  => [ # Directory for the initial backup
                               sub {$this->setInitialBackupDir($_[0]);},
                               sub {return $this->getInitialBackupDir();},
                               \&booleanFormatter,
                               "PATH",
                               "DBInstance" ,
                               \&voidValidate,
                               {
                                "priority" => 0
                               }
                              ]  ,

     "LOAD_SAMPLE_SCHEMA"  => [ # Directory for the initial backup
                               sub {$this->setAdvicedLoadSampleSchema($_[0]);},
                               sub {return $this->getLoadSampleSchema();},
                               \&booleanFormatter,
                               "BOOLEAN",
                               "DBInstance" ,
                               \&voidValidate,
                               {
                                "priority" => 0
                               }
                              ]  ,

     "RECOVERYMODE"   => [ # Abstract parameter for recovery -> auto save log etc
                          sub {$this->setRecoveryMode($_[0]);},
                          sub {return $this->getRecoveryMode();},
                          \&defaultFormatter,
                          "STRING",
                          "DBInstance" ,
                          \&voidValidate,
                          {
                           "priority" => 0,"transient" => 1
                          }
                         ] ,
     "AUTO_UPDATE_STATISTICS"  => [ # Auto update statistics
                                   sub {$this->setAdvicedAutoUpdateStatistics($_[0]);},
                                   sub {return $this->getAutoUpdateStatistics();},
                                   \&booleanFormatter,
                                   "BOOLEAN",
                                   "DBInstance" ,
                                   \&voidValidate,
                                   {
                                    "priority" => 100
                                   }
                                  ]  ,
     "RUN_EVENT_DISPATCHER"  => [ # Auto update statistics
                                 sub {$this->setRunEventDispatcher($_[0]);},
                                 sub {return $this->getRunEventDispatcher();},
                                 \&booleanFormatter,
                                 "BOOLEAN",
                                 "DBInstance" ,
                                 \&voidValidate,
                                 {
                                  "priority" => 99
                                 }
                                ]  ,
     "LOG_AUTO_OVERWRITE"  => [ # Automatic log overwrite 
                               sub {$this->setAutoOverWriteLog($_[0]);},
                               sub {return $this->getAutoOverWriteLog();},
                               \&booleanFormatter,
                               "BOOLEAN",
                               "DBInstance" ,
                               \&voidValidate,
                               {
                                "priority" => 0
                               }
                              ]  ,
     "AUTO_EXTEND"  => [        # Automatic extension of  data area
                        sub {$this->setAdvicedAutoExtend($_[0]);},
                        sub {return $this->getAutoExtend();},
                        \&booleanFormatter,
                        "BOOLEAN",
                        "DBInstance" ,
                        \&voidValidate,
                        {
                         "priority" => 100
                        }
                       ]  ,
     "DEPENDPATH"   => [
                        sub {$this->setDependendPath($_[0]);},
                        sub {return $this->getDependendPath();},
                        \&stringFormatter,
                        "PATH",
                        "DBInstance" ,
                        \&voidValidate,
                        {"priority" => 1
                        }
                       ] ,
     "CREATE_STRATEGY" => [
                           sub {$this->setCreationStrategy($_[0]);},
                           sub {return $this->getCreationStrategy();},
                           \&defaultFormatter,
                           "INTEGER",
                           "DBInstance" ,
                           \&voidValidate,
                           {"priority" => 0
                           }
                          ]   ,

     "PARALLELCONNECTIONS" => [
                               sub {$this->setAdvicedParallelConnections($_[0]);},
                               sub {return $this->getParallelConnections();},
                               \&defaultFormatter,
                               "INTEGER",
                               "DBInstance" ,
                               \&voidValidate,
                               {"priority" => 0
                               }
                              ]   ,

     "DELETE_STRATEGY" => [
                           sub {$this->setDeletionStrategy($_[0]);},
                           sub {return $this->getDeletionStrategy();},
                           \&defaultFormatter,
                           "INTEGER",
                           "DBInstance" ,
                           \&voidValidate,
                           {"priority" => 0
                           }
                          ]   ,
     "CREATE_DBPATHS"  => [
                           sub {$this->setIsCreateDBPaths($_[0]);},
                           sub {return $this->getIsCreateDBPaths();},
                           \&booleanFormatter,
                           "BOOLEAN",
                           "DBInstance" ,
                           \&voidValidate,
                           {"priority" => 0
                           }
                          ]  ,
     "DATAVOLS"  => [
                     sub {$this->setAdvicedDataVolCount($_[0]);},
                     sub {return $this->getDataVolCount();},
                     \&defaultFormatter,
                     "INTEGER",
                     "DBInstance" ,
                     \&voidValidate,
                     {"priority" => 0
                     }
                    ] ,
     "DATAPATH"  => [
                     sub {$this->setDataPath($_[0]);},
                     sub {return $this->getDataPath();},
                     \&defaultFormatter,
                     "PATH",
                     "DBInstance" ,
                     \&voidValidate,
                     {"priority" => 99
                     }
                    ]  ,
     "DATAPATHPREFIX"  => [
                           sub {$this->setDataPathPrefix($_[0]);},
                           sub {return $this->getDataPathPrefix();},
                           \&defaultFormatter,
                           "PATH",
                           "DBInstance" ,
                           \&voidValidate,
                           {"priority" => 100 
                           }
                          ]  ,
     "DATASIZE"  => [
                     sub {$this->setDataSize($_[0]);},
                     sub {return $this->getDataSize();},
                     \&defaultFormatter,
                     "DATASIZE",
                     "DBInstance" ,
                     \&voidValidate,
                     {"priority" => 99
                     }
                    ]  ,
     "LOGVOLS"   => [
                     sub {$this->setAdvicedLogVolCount($_[0]);},
                     sub {return $this->getLogVolCount();},
                     \&defaultFormatter,
                     "INTEGER",
                     "DBInstance" ,
                     \&voidValidate,
                     {"priority" => 0
                     }
                    ] ,
     "DRIVE" => [
                 sub {$this->setAdvicedDrive($_[0]);},
                 sub {return $this->getDrive();},
                 \&defaultFormatter,
                 "DRIVE",
                 "DBInstance",
                 \&driveValidate,
                 {
                  "priority" => 101
                 }
                ],
     "DATADRIVE" => [
                     sub {$this->setAdvicedDataDrive($_[0]);},
                     sub {return $this->getDataDrive();},
                     \&defaultFormatter,
                     "DRIVE",
                     "DBInstance",
                     \&driveValidate,
                     {
                      "priority" => 100
                     }
                    ],
     "LOGDRIVE" => [
                    sub {$this->setAdvicedLogDrive($_[0]);},
                    sub {return $this->getLogDrive();},
                    \&defaultFormatter,
                    "DRIVE",
                    "DBInstance",
                    \&driveValidate,
                    {
                     "priority" => 100
                    }
                   ],
     "INSTROOTPATH" => [
                        sub {$this->setAdvicedInstRootPath($_[0]);},
                        sub {return $this->getInstRootPath();},
                        \&defaultFormatter,
                        "PATH",
                        "DBInstance",
                        \&voidValidate,
                        {"priority" => 0
                        }
                       ],
     "LOGPATH"   => [
                     sub {$this->setLogPath($_[0]);},
                     sub {return $this->getLogPath();},
                     \&defaultFormatter,
                     "PATH",
                     "DBInstance" ,
                     \&voidValidate,
                     {"priority" => 99
                     }
                    ] ,
     "LOGPATHPREFIX"   => [
                           sub {$this->setLogPathPrefix($_[0]);},
                           sub {return $this->getLogPathPrefix();},
                           \&defaultFormatter,
                           "PATH",
                           "DBInstance" ,
                           \&voidValidate,
                           {"priority" => 99 
                           }
                          ] ,
     "LOGSIZE"   => [
                     sub {$this->setLogSize($_[0]);},
                     sub {return $this->getLogSize();},
                     \&defaultFormatter,
                     "DATASIZE",
                     "DBInstance" ,
                     \&voidValidate,
                     {"priority" => 0
                     }
                    ] ,
     "CPUCOUNT"   => [
                      sub {$this->setAdvicedCPUCount($_[0]);},
                      sub {return $this->getCPUCount();},
                      \&defaultFormatter,
                      "INTEGER",
                      "DBInstance" ,
                      \&voidValidate,
                      {"priority" => 0
                      }
                     ] ,
     "DBSIZE"   => [
                    sub {$this->setAdvicedDBSize($_[0]);},
                    sub {return $this->getDBSize();},
                    \&defaultFormatter,
                    "DATASIZE",
                    "DBInstance" ,
                    #                        sub {return $this->validateDBSize($_[0]);},
                    \&voidValidate,
                    {"priority" => 100
                    }
                   ] ,
     "DBPARAM"   => [
                     sub {
                       my $list = $_[0];
                       my ($key,$value) = @$list;
                       if (defined $key and defined $value) {
                         $this->addDBParam($key,$value);
                       }
                     },
                     sub {return $this->getDBParam($_[0]);},
                     \&defaultFormatter,
                     "DBPARAMETER",
                     "DBInstance" ,
                     \&voidValidate,
                     {"priority" => 0
                     }
                    ] ,

     "DBVOLUME"   => [
                      sub {
                        my $volume = $_[0];
                        my $number = @$volume[0]; 
                        my $art = @$volume[1]; 
                        my $path = @$volume[2];
                        my $type = @$volume[3]; 
                        my $size = @$volume[4];
                        #                          print "number = $number,art=$art,path=$path,type=$type,size=$size\n";
                        $this->addDBVolume(@$volume);
                      },
                      sub {return undef;},
                      \&volumeFormatter,
                      "DBVOLUME",
                      "DBInstance" ,
                      \&voidValidate,
                      {"priority" => 0
                      }
                     ] ,


     "DBMEMORY"   => [
                      sub {$this->setAdvicedDBMemory($_[0]);},
                      sub {return $this->getDBMemory();},
                      \&defaultFormatter,
                      "DATASIZE",
                      "DBInstance" ,
                      \&voidValidate,
                      {"priority" => 0
                      }
                     ] ,

     "DBMUSER"   => [
                     sub {$this->setDBM($_[0]);},
                     sub {return $this->getDBM();},
                     \&defaultFormatter,
                     "STRING",
                     "DBInstance" ,
                     \&voidValidate,
                     {
                      "transient" => 1,
                      "priority" => 0
                     }
                    ],

     "DBMPASS"   => [
                     sub {$this->setDBMPass($_[0]);},
                     sub {return $this->getDBMPass();},
                     \&defaultFormatter,
                     "STRING",
                     "DBInstance" ,
                     sub {
                       return $this->passwordValidator($_[0]);
                     } ,
                     {
                      "transient" => 1,
                      "priority" => 0
                     }
                    ], 

     "DBAUSER"   => [
                     sub {$this->setDBA($_[0]);},
                     sub {return $this->getDBA();},
                     \&defaultFormatter,
                     "STRING",
                     "DBInstance" ,
                     \&voidValidate,
                     {
                      "transient" => 1,
                      "priority" => 0
                     }
                    ],

     "DBAPASS"   => [
                     sub {$this->setDBAPass($_[0]);},
                     sub {return $this->getDBAPass();},
                     \&defaultFormatter,
                     "STRING",
                     "DBInstance" ,
                     sub {
                       return $this->passwordValidator($_[0]);
                     } ,
                     {
                      "transient" => 1,
                      "priority" => 0
                     }
                    ] ,


     "ASYNCWAIT"   => [
                       sub {$this->setAsyncWaitTime($_[0]);},
                       sub {return $this->getAsyncWaitTime();},
                       \&defaultFormatter,
                       "INTEGER",
                       "DBInstance" ,
                       \&voidValidate,
                       {
                        "priority" => 0
                       }
                      ] ,


     "DEBUGLEVEL"   => [
                        sub {$this->setDebug($_[0]);},
                        sub {return $this->getDebug();},
                        \&defaultFormatter,
                        "STRING",
                        "DBInstance" ,
                        \&voidValidate,
                        {
                         "priority" => 0,"transient" => 1
                        }
                       ] 
    };   

  my $volumes =  $this->getDBVolumes();
  foreach my $volume (@$volumes) {
    $aParams->{"VOLUMEPATH." . @$volume[1] .".". @$volume[0] } = $this->createVolumePathParam($volume);
    $aParams->{"VOLUMESIZE." . @$volume[1] .".". @$volume[0] } = $this->createVolumeSizeParam($volume);
    $aParams->{"VOLUMENAME." . @$volume[1] .".". @$volume[0] } = $this->createVolumeNameParam($volume);
  }

  my %dbparams = $this->getDBParams();

  while ((my $key,my $value) = each %dbparams) {
    $aParams->{"DBPARAM." . $key } = $this->createDBParamParam($key);
  }

  return $aParams;
}                               #getParams


sub createVolumeNameParam{
  my ($this,$volume) = @_;
  my $path = @$volume[2];
  return 
    [
     sub {
       my $dirName = dirname($path);
       @$volume[2]=catfile($dirName,$_[0]);
     },
     sub {return basename($path);},
     \&defaultFormatter,
     "STRING",
     "DBInstance" ,
     \&voidValidate,
     {
      "transient" => 1
      ,
      "priority" => 0
     }
    ]; 
}

sub createVolumePathParam{
  my ($this,$volume) = @_;
  return 
    [
     sub {@$volume[2]=$_[0];},
     sub {return @$volume[2];},
     \&defaultFormatter,
     "PATH",
     "DBInstance" ,
     \&voidValidate,
     {
      "transient" => 1
      ,
      "priority" => 0
     }
    ]; 
}


sub createVolumeSizeParam{
  my ($this,$volume) = @_;
  return 
    [
     sub {@$volume[4]=$_[0];},
     sub {return @$volume[4];},
     \&defaultFormatter,
     "DATASIZE",
     "DBInstance" ,
     \&voidValidate,
     {
      "transient" => 1
      ,
      "priority" => 0
     }
    ]; 
}







sub passwordValidator{
  my ($this,$aPassword) = @_;
  my @errorRet = (0,"Password missing", 
                  "Enter a password");
  my @successRet = (1);
  my @retVal = @successRet;
  if ((not (defined $aPassword)) or  ("" eq $aPassword)) {
    @retVal = @errorRet;
  }
  return @retVal;
} 

sub confirmPasswordValidator{
  my ($this,$aPassword,$aConfirmPassword) = @_;
  return $this->confirmPasswordValidatorLabeled($aPassword,$aConfirmPassword);
}

sub confirmPasswordValidatorLabeled{
  my ($this,$aPassword,$aConfirmPassword,$label) = @_;
  my $shit = $label ? " for " . $label  : "";
  my @errorRet = (0,"Confirm password different", 
                  "Enter the same password twice$shit");

  my @errorRet2 = (0,"Missing password", 
                   "Enter a password$shit");
  my @errorRet3 = (0,"Missing password confirmation", 
                   "Confirm password$shit");

  my @successRet = (1);
  my @retVal = @successRet;
  if (not defined $aPassword or "" eq $aPassword ) {
    @retVal = @errorRet2;
  } elsif (not defined $aConfirmPassword or "" eq $aConfirmPassword) {
    @retVal = @errorRet3;
  } elsif ($aPassword ne $aConfirmPassword) {
    @retVal = @errorRet;
  }
  return @retVal;
}







my $dbmDesc = "Database Manager Operator";
my $dbaDesc = "Database System Administrator";

sub getAdminUserStruct{
  my ($this) = @_;
  my $retVal = [];
  my $dbm  = {
              "Description" => $dbmDesc,
              "Name" => $this->getDBM(),
              #              "Password" => $this->getDBMPass(),
              "Password" => "",
              "ConfirmPassword" => $this->getConfirmDBMPass(),
              "Role" => "dbm"
             };
  my $dba  = {
              "Description" => $dbaDesc,
              "Name" => $this->getDBA(),
              #              "Password" => $this->getDBAPass(),
              "Password" => "",
              "ConfirmPassword" => $this->getConfirmDBAPass(),
              "Role" => "dba"
             };
  push @$retVal,($dbm,$dba);
  return $retVal;
}


sub setAdminUserStruct{
  my ($this,$aVal) = @_;
  my $dbm   =undef;
  my $dba  = undef;
  foreach my $user (@$aVal) {
    my $role = $user->{"Role"};
    my $desc = $user->{"Description"};
    if ($dbaDesc eq $desc) {
      $dba = $user;
    } elsif ($dbmDesc eq $desc) {
      $dbm  = $user;
    }
  }
  $this->setDBM($dbm->{"Name"});
  $this->setDBMPass($dbm->{"Password"});
  $this->setConfirmDBMPass($dbm->{"ConfirmPassword"});
  $this->setDBA($dba->{"Name"});
  $this->setDBAPass($dba->{"Password"});
  $this->setConfirmDBAPass($dba->{"ConfirmPassword"});
}

sub adminUserStructValidator{
  my ($this,$aVal) = @_;
  my $dbm   =undef;
  my $dba  = undef;
  my @retVal = (1);
  my @errorRets = (0,"DBM-User missing", 
                   "Enter data for dbm");
  my @errorRets2 = (0,"DBA-User missing", 
                    "Enter data for dba");
  my @errorRets3 = (0,"DBA-User name missing", 
                    "Enter name of dba user");
  my @errorRets4 = (0,"DBA-User name missing", 
                    "Enter name of dba user");
  my @errorRets5 = (0,"User names must not be equal", 
                    "Enter different  user names for DBM and DBA");
  foreach my $user (@$aVal) {
    my $desc = $user->{"Description"};
    if ($dbaDesc eq $desc) {
      $dba = $user;
    } elsif ($dbmDesc eq $desc) {
      $dbm  = $user;
    }
  }
  if (not defined $dbm) {
    @retVal = @errorRets;
  } elsif (not defined $dba) {
    @retVal = @errorRets2;
  } else {
    my $dbmName = $dbm->{"Name"};
    my $dbmPass = $dbm->{"Password"};
    my $dbmConfirm = $dbm->{"ConfirmPassword"};
    unless(defined $dbmName and not "" eq $dbmName){
      @retVal = @errorRets4;
    }
    if ($retVal[0]) {
      @retVal = $this->confirmPasswordValidatorLabeled($dbmPass,$dbmConfirm,$dbmName);
    }
    if ($retVal[0]) {
      my $dbaName = $dba->{"Name"};
      my $dbaPass = $dba->{"Password"};
      my $dbaConfirm = $dba->{"ConfirmPassword"};
      unless(defined $dbaName and not "" eq $dbaName){
        @retVal = @errorRets3;
      }
      if ($retVal[0]) {
        @retVal = $this->confirmPasswordValidatorLabeled($dbaPass,$dbaConfirm,$dbaName);
      }
    }
    if ($retVal[0]) {
      my $dbmName = $dbm->{"Name"};
      my $dbaName = $dba->{"Name"};
      if ((uc $dbmName) eq (uc $dbaName)) {
        @retVal = @errorRets5;
      }
    }
  }
  return @retVal;
}







sub getGlobalUserStruct{
  my ($this) = @_;
  my $user  = {
               "Description" => "Database System Administrator",
               "Name" => $this->getGlobalUser(),
               "Password" => "",
               #              "Password" => $this->getGlobalPassword(),
               "ConfirmPassword" => $this->getConfirmGlobalPassword(),
               "Role" => "globaluser"
              };
  return $user;
}

sub setGlobalUserStruct{
  my ($this,$val) = @_;
  my $name = $val->{"Name"};
  my $psw = $val->{"Password"};
  my $cpsw = $val->{"ConfirmPassword"};
  $this->setAdvicedGlobalUser($name);
  $this->setAdvicedGlobalPassword($psw);
  $this->setAdvicedConfirmGlobalPassword($cpsw);
}

sub globalUserStructValidator{
  my ($this,$val) = @_;

  my @retVal = (1);
  my @errorRets = (0,"Database user missing", 
                   "Enter data for user ");
  my @errorRets2 = (0,"Database user name missing", 
                    "Specify name of database user");
  
  if (not defined $val) {
    @retVal = @errorRets;
  } else {
    my $name = $val->{"Name"};
    my $psw = $val->{"Password"};
    my $cpsw = $val->{"ConfirmPassword"};
    my $dbm = $this->getDBM();
    unless(defined $name and not "" eq $name){
      @retVal = @errorRets2;
    }
    if ($retVal[0]) {
      @retVal = $this->confirmPasswordValidatorLabeled($psw,$cpsw,$name);
    }
    if ($retVal[0] ) {
      if ( (uc $dbm) eq  (uc $name)) {
        @retVal = (0,"Invalid user name", 
                   "User name $name not allowed, choose another name");
      }
    }
  }
  return @retVal;
}



sub dbnameValidator{
  my ($this,$val) = @_;
  my @retVal = $this->dbnameValidateCharacters($val);
  if ( $retVal[0]>= 1 ) {
    @retVal = $this->dbnameValidatorViaInstInfo($val);
    if ($retVal[0] > 1) {
      @retVal = $this->dbnameValidatorViaGetInstances($val);
    }
  }
  return @retVal;
}


sub dbnameValidateCharacters{
  my ($this,$dbname) = @_;
  my @retVal = (1);
  my $length = $dbname ? length($dbname): 0;
  if (not defined $dbname) {
    my $message="Database name undefined";
    my $desc = "Specifiy database name";
    @retVal =(0,$message,$desc);
  } elsif ( $length > 8) {
    my $message="Database name too long";
    my $desc = "Only 8 characters are allowed in database name, you have $length";
    @retVal =(0,$message,$desc);
  } elsif ($length <= 0) {
    my $message="No database name";
    my $desc = "No database name entered, specify a database name";
    @retVal =(0,$message,$desc);
  } else {
    foreach ( split(//,$dbname)) {
      if ( ord($_)>127) {
        my $message="Invalid database name";
        my $desc = "Use ascii7 characters only in database name (\"$_\" not allowed) ";
        @retVal = (0,$message,$desc);
        last ;
      }
    }
  }
  if ( $retVal[0]) {
    @retVal = (1,"Good database name \"$dbname\"","Go on and feel happy")
  }
  return @retVal;
}




sub dbnameValidatorViaGetInstances{
  my ($this,$val) = @_;
  my %instance = getInstances();
  my @retVal = (1);
  foreach my $aKey (keys %instance) {
    if ((uc $aKey) eq (uc $val)) {
      @retVal = (0,"DB $val already exists.","Choose another database name");
      last;
    }
  }
  return @retVal;
}


sub dbnameValidatorViaInstInfo{
  my ($this,$val) = @_;

  my @retVal = (1);
  my $createStrategy = $this->getCreationStrategy();
  if ($createStrategy >= 3) {
    return @retVal;
  }

  my $info = SAPDB::Install::InstInfo::new ($val);
  unless( defined $info) {
    @retVal = (2,"DB Info not available","SAPDB::Install::InstInfo returned undef");
  }
  my $infoListRef =  undef;
  if ($retVal[0]==1) {
    $infoListRef =  $info->{'database'};
    unless(defined $infoListRef){
      @retVal = (2,"DB Info not available","SAPDB::Install::InstInfo->{'database'} return undef");
    }
  }

  if ($retVal[0]==1) {
    my $dbInfo = $infoListRef->{$val};
    if ( defined $dbInfo) {
      @retVal = (0,"DB $val already exists.","Choose another database name");
    }
  }


  return @retVal;
}




sub createDBParamParam{
  my ($this,$paramName) = @_;

  return 
    [
     sub {$this->addDBParam($paramName,$_[0]);},
     sub {return $this->getDBParam($paramName);},
     \&defaultFormatter,
     "STRING",
     "DBInstance" ,
     \&voidValidate,
     {
      "transient" => 1
      ,
      "priority" => 0
     }
    ]; 
}


#matchNumber


sub configureGenericFromKeyValuePairs{
  my ($this,$key,$value,$type)=@_;
  my $matchers = getMatchers();
  my $matcher = $matchers->{$type};
  if (defined $matcher) {
    my $mVal = &$matcher($value);
    $this->addGeneric($key,$mVal,$type);
  } else {
    $this->writeLog("WARNING: Invalid type for generic value: \"$type\" (Ignored)");
  }
}




my %versionInfos;
sub getVersionInfo{
  my ($this,$dbm) = @_;
  unless (exists $versionInfos{$dbm}){
    my %dbmVersion =  $dbm->dbm_version;
   $versionInfos{$dbm} = \%dbmVersion;
  }
  return $versionInfos{$dbm};
}
sub getDBMVersion{
  my ($this,$dbm) = @_;
  my $versionInfo = $this->getVersionInfo($dbm);
  my $version = $versionInfo->{'VERSION'};
  return $version;
}


# Set DB parameters via DBMCmd
#
sub dbmcmdSetDBParams{
  my ($this,$dbm) = @_;
  $this->setStatusRunning("param_put");
  #  my %versionInfo = $dbm->dbm_version;
  #  my $version = $versionInfo{'VERSION'};
  my $versionSpecific = {'XP_MAXPAGER' => '7.6.00'};
  defined $this->dbm($dbm, "param_startsession", 
                     "param_startsession failed.\n",
                     "Parameter session started\n" ) or return $this->msgError(undef,"param_put");
  defined $this->dbm($dbm, "param_init", 
                     "param_init failed.\n"
                    ) or return $this->msgError(undef,'param_put');
  defined $this->dbm($dbm,"param_checkall", "DB-Parameter check failed.\n", "Checked DB Parameters successfully.\n")
    or return $this->msgError(undef,'param_put');
  defined $this->dbm($dbm,"param_commitsession", "Failed to commit parameter session.\n","Parameter session committed.\n")
    or return $this->msgError(undef,'param_put');
  my %dbparams = $this->getDBParams();
  while ((my $key,my $value) = each %dbparams) {
    my $parameterIsGood = 1;
    if ( $versionSpecific->{$key} and $this->callDBM($dbm,"param_getdefault $key")) {
      $parameterIsGood = 0;
      $this->writeLog("WARNING: Parameter '$key' is not supported on this database.\n");
    }
    $parameterIsGood and 
      $this->dbm($dbm, 
                 "param_directput " . $key . " " . $value, 
                 "Failed setting DB-Parameter \"" . $key . "\"\n",
                 "Set " .$key . " to " . $value . "\n");
  }
  defined $this->dbm($dbm,"param_checkall", "DB-Parameter check failed.\n", "Checked DB Parameters successfully.\n")
    or return $this->msgError(undef,'param_put');
  $this->setStatusFinishedSuccess("param_put");
  return 1;
}
#dbmcmdSetDBParams()



sub getDBParam{
  my ($this,$key) = @_;
  my $retVal=undef;
  my %dbparams = $this->getDBParams();
  if (defined $key) {
    while ((my $aKey,my $aValue) = each %dbparams) {
      if ( $aKey eq $key) {
        return $aValue;
      }
    }
  }
  return $retVal;
}



sub createDBMCmd($){
  my ($this) = @_;
  my $xserverInfo = xserverInfo();
  my $isRunning = $xserverInfo->{'is_running'};
  if ($isRunning) {
  } else {
    $this->showStatusMessage("INFO: Trying to start x_server.\n");
    $this->startXServer();
  }
  my $dbm = SAPDB::Install::DBMCmd->new ('', $this->getDBName());
  return $dbm;
}
#createDBMCmd()


=over 1 
=item startXServer

Start the X-Server if necessary

 my $listref = startXServer();
 print "Message-ID: ", $listref->[0],"\n";
 print "Message-Type: ", $listref->[1],"\n";
 print "Message: ", $listref->[2],"\n";

 --------------------------

 Message-ID: 18641
 Message-Type: ERROR
 Message: XSERVER is already running

 --------------------------

 Message-ID: 19720
 Message-Type: INFO
 Message: XSERVER started

=back

=cut

sub startXServer(){
  my ($this, $id, $type, $text) = @_;
  my ($indep_data,$indep_prog) =  readIndepPath();
  my $command = undef;
  if (defined $indep_prog) {
    my $xserver = undef;
    if ($^O =~ /mswin/i) {
      $xserver = catfile($indep_prog,"bin","x_server.exe");
    } else {
      $xserver = catfile($indep_prog,"bin","x_server");
    }
    if ( -f $xserver) {
      $command = $xserver . " start";
    }
  }
  unless (defined $command){
    $command = "x_server start";
  }

  $this->writeLog("[" . $command . "]\n");
  callSystem($command, 
             sub{
               my $line = $_[0];
               unless ($line =~ /^\s*$/){
                 $line =~ m/(\d+)\s+(\w+):\s+(.+)/; 
                 $id=$1;
                 $type=$2;
                 $text=$3;
               }
               return 1;
             },
             1);

  if (defined $id and $id == 18641  ) {
    $this->showStatusMessage("WARNING: x_server already started.\n");
  } elsif (defined $id and $id == 19720 ) {
    $this->showStatusMessage("INFO: x_server started.\n");
  } else {
    $this->writeLog("WARNING: " .
                    "I could not figure out, if your x_server is running.\n" .
                    "If errors occur, check if x_server is running.\n");
  }
  [$id, $type, $text];
}                               #startXServer



# Gibt einen Fehler aus, der durch
# von dbmserver kommt.
sub dbmcmdError{
  my ($dbc,$dbm,$message,$displayCommand) = @_;
  my $lastcmd = defined $displayCommand? $displayCommand : $dbm->lastcmd();
  my $lastmsg = $dbm->lastmsg();
  my $lasterr = $dbm->lasterr();
  $dbc->showStatusMessage("ERROR: " . $message);
  $dbc->writeLog($message);
  $dbc->writeLog("DBMCLI-Command: \"" . ((defined $lastcmd) ? $lastcmd : "" ) .  "\"\n");
  $dbc->writeLog("DBMCLI-Message: \"" . ((defined $lastmsg ? $lastmsg : "") . "\"\n"));
  $dbc->writeLog("DBMCLI-Error: \"" .   ((defined $lasterr ? $lasterr : "") . "\"\n"));
  $dbc->setLastDBMError($lasterr);
  $dbc->setLastDBMMessage($lastmsg);
  return;
}
#dbmcmdError


#Gibt die Konfigurationsdaten nach stdout
sub testDump(){
  my $this = $_[0];
  $this->writeLog( "DBNAME: \"" . $this->getDBName() .  "\"\n");
  $this->writeLog( "DEPENDEND-PATH: \"" . $this->getDependendPath() .  "\"\n");
  $this->writeLog( "DBM: \"" . $this->getDBM() .  "\"\n");
  $this->writeLog( "DBM PASSWORT: \"" .       $this->getDBMPass() . "\"\n");
  $this->writeLog( "DBA: \"" .     $this->getDBA() .  "\"\n");
  $this->writeLog( "DBA-PASSWORT:\"" .       $this->getDBAPass() . "\"\n");
  $this->writeLog( "DOMAIN-PASSWORT: \"" .      $this->getDomainPass() . "\"\n");
  $this->writeLog( "SQL-User: \"" .      $this->getSQLU() . "\"\n");
  $this->writeLog( "SQL-Passwort: \"" .      $this->getSQLPass() . "\"\n");
  $this->writeLog( "Domain-Passwort: \"" . $this->getDomainPass() . "\"\n");
  $this->writeLog(  "PARAMETERS:   \n");
  my %dbparams = $this->getDBParams();
  while ((my $key,my $value) = each %dbparams) {
    $this->writeLog("$key = $value\n");
  }
  $this->writeLog("VOLUMES:   \n");
  my $volumes = $this->getDBVolumes();
  foreach my $volume (@$volumes) {
    $this->writeLog( "[Nummer=" . @$volume[0]
                     . ",Art=" . @$volume[1]
                     . ",Ort=" . @$volume[2]
                     . ",Typ=" . @$volume[3]
                     . ",Groesse=" . @$volume[4]
                     . "]\n");
  }
  my $pipeName = $this->getRecoverFromPipe();
  if ( defined $pipeName) {
    $this->writeLog("Recover from pipe: " . $pipeName . "\n");
  }
  my $mediumDeclaration = $this->getMediumDeclaration();
  if ( defined $mediumDeclaration) {
    $this->writeLog("Medium declaration: \"" . $mediumDeclaration . "\"\n");
  }

  my $backup = $this->getBackup();
  if ( defined $backup) {
    $this->writeLog("Backup: \"" . $backup . "\"\n");
  }
  my $archive = $this->getArchive();
  if ( defined $archive) {
    $this->writeLog("Archive: \"" . $archive . "\"\n");
  }
  $this->writeLog("GENERICS:\n");
  my $generics = $this->getAllGenerics();
  while ((my $key,my $value) = each %{$generics}) {
    my $pair = $this->getGeneric($key);
    $this->writeLog("$key=\"$pair->[0]\" [$pair->[1]]\n");
  }
}
#end testDump();

1;



# Local Variables: ***
# perl-dbg-flags:"-w      -I../../../../../../wrk/script -I../../../sdbrun  " ***
# sdbinst-defines: "-define SCRIPTING=YES -define PERL5.6.1=YES" ***
# sdbinst-testfile: "BaseTest.pl" ***
# sdbinst-script-directory:  "sys/wrk/script/SAPDB/Install/Templates" ***
# End: ***

