#!/usr/bin/perl
package SAPDB::Install::Templates::WindowsInstaller;
#ifdef SCRIPTING
#
#
#
##c:/Windows/System32/Richttx32.ocx macht Probleme
##Weil er abfragt, ob die Datei schon existiert
#use strict;
#use SAPDB::Install::Templates::Util;
#use SAPDB::Install::Registry ;
#use SAPDB::Install::Untgz;
#use SAPDB::Install::System;
#require Exporter;
#our @ISA = qw (  Exporter );
#our @EXPORT=qw(
#               DBM 
#               SQLSTUDIO
#               wiframestart
#               wipreinstall
#               wipostprepare
#               wipostinstall
#               wiregister
#               wiframestop
#               wiframestop
#               wiunregister
#               wipreuninstall
#               wipostuninstall
#               wiverify
#               staticWillSkip
#              );
#endif
#because SDBINST

require SDB::Common::Exporter;

use SAPDB::Install::Registry;
use SAPDB::Install::Untgz;
use SAPDB::Install::System;
use SAPDB::Install::StdIO;
use SAPDB::Install::Templates::Util;

our @ISA = qw (SDB::Common::Exporter);
our @EXPORT = qw ( 
                  DBM 
                  SQLSTUDIO
                  wiframestart
                  wipreinstall
                  wipostprepare
                  wipostinstall
                  wiregister
                  wiframestop
                  wiframestop
                  wiunregister
                  wipreuninstall
                  wipostuninstall
                  wiverify
                  StdIO
                  staticWillSkip
);
#endbecause

#ifdef doc
#<h1>SAPDB::Install::Templates::WindowsInstaller </h1>
#  <p>
#  Install Database Manager and SQL-Studio.
#  </p>
#  <h2>Usage </h2>
#  <pre>
#  my $dbm = DBM();
#$dbm->setExecutable("c:/temp/dbm75.exe");
#$dbm->set...
#  $dbm->install();
#</pre>
#  <h2>What it does</h2>
#  <ul>
#  <li>
#  Generates  <code>response file</code> to run install shield
#  in silent mode:
#<ol>
#  <li>A response file to uninstall</li>
#  <li>A response file to install</li>
#  </ol>
#  </li>
#  <li>
#  Executes self extracting installshield executables in silent mode, i.e. 
#  the "Database Manager" and "SQL Studio" executables.
#  </li>
#  <li>
#  Checks windows registry  to see if software is installed.
#  <ul>
#  <li>Installation: If software is already installed (any version) no installation is done, a warning is issued</li>
#  <li>Uninstallation: If software is not installed 
#  or a different version is installed, no uninstallation is done, a warning is issued.</li>
#  </ul>
#  </li>
#  <li>Can be configured from file for testing purposes</li>
#  <li>Can be configured via get/set-Methods for integration purposes</li>
#  </ul>
#
#  <h2>Example for a configuration file </h2>
#
#  <pre>
#  [Database Manager]
#  RESPONSEFILEINSTALL    c:\aw\dbm\install_dbm.iss
#  RESPONSEFILEUNINSTALL    c:\aw\dbm\uninstall_dbm.iss
#  LOGFILEINSTALL  c:\aw\dbm\install_dbm.log
#  LOGFILEUNINSTALL  c:\aw\dbm\uninstall_dbm.log
#  INSTALLABLE c:\aw\dbm\dbm75.exe
#  VERSION 7.5.00.09      
#  RELEASE 7.5       
#  INSTALLDIR c:\sdb\programs\dbm
#
#  [SQL Studio]
#  RESPONSEFILEINSTALL    c:\aw\sqlstudio\install_studio.iss
#  RESPONSEFILEUNINSTALL    c:\aw\sqlstudio\uninstall_studio.iss
#  LOGFILEINSTALL  c:\aw\sqlstudio\install_studio.log
#  LOGFILEUNINSTALL  c:\aw\sqlstudio\uninstall_studio.log
#  INSTALLABLE c:\aw\sqlstudio\sqlstd75.exe
#  VERSION 7.5.00.08      
#  RELEASE 7.5       
#  INSTALLDIR c:\sdb\programs\SQLStudio
#  </pre>
  #endif
  
  #+field(String:The release of the software)
  my $release="releaseKey";

#+sub(String):void:Set the release
sub setRelease{
  my ($this, $aRelease) = @_;
  $this->{$release}=$aRelease;
}

#+sub():String:Get the release
sub getRelease{
  my ($this ) = @_;
  return $this->{$release};
}

sub setAdvicedRelease{
  my ($this, $aValue) = @_;
  willSet($this,$release);
  $this->setRelease($aValue);
  haveSet($this,$release);
}






#+field(String:The version of the software)
my $version="versionKey";
#+sub(String):void:Set the Version
sub setVersion{
  my ($this, $aVersion) = @_;
  $this->{$version}=$aVersion;
}

#+sub():String:Get the release
sub getVersion{
  my ($this ) = @_;
  return $this->{$version};
}

#+field(String:The name of the software)
my $name="nameKey";
#+sub(String):void:Set the Name
sub setName{
  my ($this, $aName) = @_;
  $this->{$name}=$aName;
}

#+sub():String:Get the Name
sub getName{
  my ($this ) = @_;
  return $this->{$name};
}

#+field(SubRef:Function to write install response file)
my $installResponseFileWriter="installResponseFileKey";

#+sub(SubRef:Function reference):void:Set the function to  write the install response file
sub setInstallResponseFileWriter{
  my ($this, $aSub) = @_;
  $this->{$installResponseFileWriter}=$aSub;
}

#+sub():String:Get the function to write the install response file
sub getInstallResponseFileWriter{
  my ($this ) = @_;
  return $this->{$installResponseFileWriter};
}

#+field(SubRef:updateResponseFileWriter)
my $updateResponseFileWriter="updateResponseFileWriterKey";
#+sub(SubRef):void:Set updateResponseFileWriter
sub setUpdateResponseFileWriter{
  my ($this, $aValue) = @_;
  $this->{$updateResponseFileWriter}=$aValue;
}
#+sub():SubRef:Get updateResponseFileWriter
sub getUpdateResponseFileWriter{
  my ($this ) = @_;
  return $this->{$updateResponseFileWriter};
}






#+field(String:The registry key for the version  of the software)
my $versionKey="versionKeyKey";
#+sub(String):void:Set the registry key for the version of the software
sub setVersionKey{
  my ($this, $aValue) = @_;
  $this->{$versionKey}=$aValue;
}
#+sub():String:Get the registry key for the version of the software
sub getVersionKey{
  my ($this ) = @_;
  return $this->{$versionKey};
}


#+field(Path:The Executable)
my $executable="executableKey";
#+sub(Path):void:Set the Executable
sub setExecutable{
  my ($this, $aValue) = @_;
  $this->{$executable}=$aValue;
}
#+sub():Path:Get the Executable
sub getExecutable{
  my ($this ) = @_;
  return $this->{$executable};
}


#+field(Path:response file for install)
my $responseFileInstall="responseFileInstallKey";
#+sub(Path):void:Set response file for install
sub setResponseFileInstall{
  my ($this, $aValue) = @_;
  $this->{$responseFileInstall}=$aValue;
}
#+sub():Path:Get response file for install
sub getResponseFileInstall{
  my ($this ) = @_;
  return $this->{$responseFileInstall};
}

#+field(Path:response file for uninstall)
my $responseFileUninstall="responseFileUninstallKey";
#+sub(Path):void:Set response file for uninstall
sub setResponseFileUninstall{
  my ($this, $aValue) = @_;
  $this->{$responseFileUninstall}=$aValue;
}
#+sub():Path:Get response file for uninstall
sub getResponseFileUninstall{
  my ($this ) = @_;
  return $this->{$responseFileUninstall};
}









#+field(Subref:logger)
my $logger="loggerKey";
#+sub(Subref):void:Set logger
sub setLogCallback{
  my ($this, $aValue) = @_;
  $this->{$logger}=$aValue;
}
#+sub():Subref:Get logger
sub getLogCallback{
  my ($this ) = @_;
  return $this->{$logger};
}

#+field(SubRef:Function to output status)
my $statusPutter="statusPutterKey";
#+sub(SubRef):void:Set Function to output status
sub setStatusCallback{
  my ($this, $aValue) = @_;
  $this->{$statusPutter}=$aValue;
}
#+sub():SubRef:Get Function to output status
sub getStatusCallback{
  my ($this ) = @_;
  return $this->{$statusPutter};
}



#+field(Path:logFileInstall)
my $logFileInstall="logFileInstallKey";
#+sub(Path):void:Set logFileInstall
sub setLogFileInstall{
  my ($this, $aValue) = @_;
  $this->{$logFileInstall}=$aValue;
}
#+sub():Path:Get logFileInstall
sub getLogFileInstall{
  my ($this ) = @_;
  return $this->{$logFileInstall};
}

#+field(Path:logFileUninstall)
my $logFileUninstall="logFileUninstallKey";
#+sub(Path):void:Set logFileUninstall
sub setLogFileUninstall{
  my ($this, $aValue) = @_;
  $this->{$logFileUninstall}=$aValue;
}
#+sub():Path:Get logFileUninstall
sub getLogFileUninstall{
  my ($this ) = @_;
  return $this->{$logFileUninstall};
}





#+field(Path:installDir)
my $installDir="installDirKey";
#+sub(Path):void:Set installDir
sub setInstallDir{
  my ($this, $aValue) = @_;
  $this->{$installDir}=$aValue;
}
#+sub():Path:Get installDir
sub getInstallDir{
  my ($this ) = @_;
  return $this->{$installDir};
}





#+field(SubRef:uninstallResponseFileWriter)
my $uninstallResponseFileWriter="uninstallResponseFileWriterKey";
#+sub(SubRef):void:Set uninstallResponseFileWriter
sub setUninstallResponseFileWriter{
  my ($this, $aValue) = @_;
  $this->{$uninstallResponseFileWriter}=$aValue;
}
#+sub():SubRef:Get uninstallResponseFileWriter
sub getUninstallResponseFileWriter{
  my ($this ) = @_;
  return $this->{$uninstallResponseFileWriter};
}

#+field(String:dirName)
my $dirName="dirNameKey";
#+sub(String):void:Set dirName
sub setDirName{
  my ($this, $aValue) = @_;
  $this->{$dirName}=$aValue;
}
#+sub():String:Get dirName
sub getDirName{
  my ($this ) = @_;
  return $this->{$dirName};
}


#+field(Path:TempDir)
my $TempDir="TempDirKey";
#+sub(Path):void:Set TempDir
sub setTempDir{
  my ($this, $aValue) = @_;
  $this->{$TempDir}=$aValue;
}
#+sub():Path:Get TempDir
sub getTempDir{
  my ($this ) = @_;
  unless (defined $this->{$TempDir}) {
    $this->mkTempDir();
  }
  my $aDir = $this->{$TempDir};
  return  $this->{$TempDir};
}

sub mkTempDir{
  my ($this) = @_;
  my $retVal=0;
  my $tempDir = $ENV{"TEMP"};
  unless (-d $tempDir){
    $tempDir = $ENV{"TMP"};
  }
  unless (-d $tempDir){
    my ($indep_data,$indep_prog)=readIndepPath(); # try to get independent paths from earlier installation
    my $tempDir = $indep_prog;
    unless(defined $indep_prog && -d $indep_prog){
      $retVal=1;
      $this->setErrorMessage("Cannot create temporary directory");
      $this->setErrorDescription("Cannot create temporary directory");
    }
  }
  unless($retVal){
    $tempDir = catfile($tempDir, $this->getDirName());
    unless(-d $tempDir){
      unless (mkdir  $tempDir){
        $retVal=2;
        $this->setErrorMessage("Cannot create temporary directory $tempDir: $!\n");
      }
    }
  }
  unless($retVal){
    $this->setTempDir($tempDir);
  }
  return $retVal;
}


sub configureFromIndepPath{
  my ($this) = @_;
  my ($indep_data,$indep_prog)=readIndepPath(); # try to get independent paths from earlier installation
  if (defined $indep_prog) {
    $this->configureFromProgDir($indep_prog);
  }
}

sub configureFromProgDir{
  my ($this,$indepProg ) = @_;
  my $dirName = $this->getDirName();
  my $aProgDir = catfile ($indepProg,$dirName);
  $this->setInstallDir($aProgDir);
}

sub configureFromLines{
  my ($this,$lines) = @_;
  my ($key,$value,$type,$section);
  foreach ( @$lines) {
    parseConfigLine($_,
                    sub{ 
                      $section =$_[0];
                      my $myName = $this->getName();
                    },
                    sub {$key=$_[0];
                         $value=$_[1];
                         if (defined $_[2]) {
                           $type=$_[2];
                         }
                       }
                   );
    if ( defined $key and defined $section and $section eq $this->getName()) {
      $this->configureInstallerFromKeyValuePairs($key,$value);
    }
    #if
  }
  #while
}


sub configureFromStream{
  my ($this,$stream) = @_;
  my @lines = <$stream>;
  $this->configureFromLines(\@lines);
}

sub configureInstallerFromKeyValuePairs{
  my ($this,$key,$value) = @_;
  my $paramsRef = $this->getParams();

  #  print "Configure: $key = $value\n";
  my $pDesc = $paramsRef->{uc $key};
  if (defined $pDesc) {
    my $setter = @$pDesc[0];
    &$setter($value);
  }
}
#configureInstallerFromKeyValuePairs




#+field(PATH:Archive)
my $Archive="ArchiveKey";
#+sub(PATH):void:Set Archive
sub setArchive{
  my ($this, $aValue) = @_;
  $this->{$Archive}=$aValue;
}
#+sub():PATH:Get Archive
sub getArchive{
  my ($this ) = @_;
  return $this->{$Archive};
}

sub setAdvicedArchive{
  my ($this, $aValue) = @_;
  willSet($this,$Archive);
  $this->setArchive($aValue);
  haveSet($this,$Archive);
}
#+field(PATH:ArchivedInstallable)
my $ArchivedInstallable="ArchivedInstallableKey";
#+sub(PATH):void:Set ArchivedInstallable
sub setArchivedInstallable{
  my ($this, $aValue) = @_;
  $this->{$ArchivedInstallable}=$aValue;
}
#+sub():PATH:Get ArchivedInstallable
sub getArchivedInstallable{
  my ($this ) = @_;
  return $this->{$ArchivedInstallable};
}


#+field(Untgz:ArchiveUntgz)
my $ArchiveUntgz="ArchiveUntgzKey";
#+sub(Untgz):void:Set ArchiveUntgz
sub setArchiveUntgz{
  my ($this, $aValue) = @_;
  $this->{$ArchiveUntgz}=$aValue;
}
#+sub():Untgz:Get ArchiveUntgz
sub getArchiveUntgz{
  my ($this ) = @_;
  return $this->{$ArchiveUntgz};
}




#+field(String:ErrorMessage)
my $ErrorMessage="ErrorMessageKey";
#+sub(String):void:Set ErrorMessage
sub setErrorMessage{
  my ($this, $aValue) = @_;
  $this->{$ErrorMessage}=$aValue;
}
#+sub():String:Get ErrorMessage
sub getErrorMessage{
  my ($this ) = @_;
  return $this->{$ErrorMessage};
}







#+field(String:ErrorDescription)
my $ErrorDescription="ErrorDescriptionKey";
#+sub(String):void:Set ErrorDescription
sub setErrorDescription{
  my ($this, $aValue) = @_;
  $this->{$ErrorDescription}=$aValue;
}
#+sub():String:Get ErrorDescription
sub getErrorDescription{
  my ($this ) = @_;
  return $this->{$ErrorDescription};
}





sub getParams{
  my ($this) = @_;
  my $myparams =  
    {
     "RESPONSEFILEINSTALL" => [
                               sub {$this->setResponseFileInstall($_[0]);},
                               sub {return $this->getResponseFileInstall();},
                               \&defaultFormatter,
                               "PATH",
                               $this->getName(),
                               \&voidValidate,
                               {"priority" => 0 
                               }
                              ],


     "RESPONSEFILEUNINSTALL" => [
                                 sub {$this->setResponseFileUninstall($_[0]);},
                                 sub {return $this->getResponseFileUninstall();},
                                 \&defaultFormatter,
                                 "PATH",
                                 $this->getName() ,
                                 \&voidValidate,
                                 {"priority" => 0 
                                 }
                                ],

     "LOGFILEINSTALL" => [
                          sub {$this->setLogFileInstall($_[0]);},
                          sub {return $this->getLogFileInstall();},
                          \&defaultFormatter,
                          "PATH",
                          $this->getName() ,
                          \&voidValidate,
                          {"priority" => 0 
                          }
                         ],

     "LOGFILEUNINSTALL" => [
                            sub {$this->setLogFileUninstall($_[0]);},
                            sub {return $this->getLogFileUninstall();},
                            \&defaultFormatter,
                            "PATH",
                            $this->getName() ,
                            \&voidValidate,
                            {"priority" => 0 
                            }
                           ],

     "VERSIONKEY" => [
                      sub {$this->setVersionKey($_[0]);},
                      sub {return $this->getVersionKey();},
                      \&defaultFormatter,
                      "STRING",
                      $this->getName() ,
                      \&voidValidate,
                      {"priority" => 0 
                      }
                     ],

     "INSTALLABLE" => [
                       sub {$this->setExecutable($_[0]);},
                       sub {return $this->getExecutable();},
                       \&defaultFormatter,
                       "PATH",
                       $this->getName() ,
                       \&voidValidate,
                       {"priority" => 0 
                       }
                      ],

     "VERSION" => [
                   sub {$this->setVersion($_[0]);},
                   sub {return $this->getVersion();},
                   \&defaultFormatter,
                   "STRING",
                   $this->getName() ,
                   \&voidValidate,
                   {"priority" => 0 
                   }
                  ],

     "RELEASE" => [
                   sub {$this->setAdvicedRelease($_[0]);},
                   sub {return $this->getRelease();},
                   \&defaultFormatter,
                   "STRING",
                   $this->getName() ,
                   \&voidValidate,
                   {"priority" => 0 
                   }
                  ],
     "DIRNAME" => [
                   sub {$this->setDirName($_[0]);},
                   sub {return $this->getDirName();},
                   \&defaultFormatter,
                   "PATH",
                   $this->getName() ,
                   \&voidValidate,
                   {"priority" => 0 
                   }
                  ],

     "ARCHIVE" => [
                   sub {$this->setAdvicedArchive($_[0]);},
                   sub {return $this->getArchive();},
                   \&defaultFormatter,
                   "PATH",
                   $this->getName() ,
                   \&voidValidate,
                   {"priority" => 0 
                   }
                  ],
     "ARCHIVEDINSTALLABLE" => [
                               sub {$this->setArchivedInstallable($_[0]);},
                               sub {return $this->getArchivedInstallable();},
                               \&defaultFormatter,
                               "PATH",
                               $this->getName() ,
                               \&voidValidate,
                               {"priority" => 0 
                               }
                              ],
     "INSTALLDIR" => [
                      sub {$this->setInstallDir($_[0]);},
                      sub {return $this->getInstallDir();},
                      \&defaultFormatter,
                      "PATH",
                      $this->getName() ,
                      \&voidValidate,
                      {"priority" => 0 
                      }
                     ]

    };
  return $myparams;
}


#+sub(String:fileName):void:Parse input file and configure
sub configureFromFile($$){
  my ($this,$file) = @_;
  open CONFIG, $file;
  $this->configureFromStream(*CONFIG);
  close CONFIG;
}



sub writedbm75UninstallResponseFile{
  my ($this, $aOut ) =@_;
  my $aName = $this->getName();
  my $installDir = $this->getInstallDir();

  my $aRelease = $this->getRelease();


  print $aOut <<AEND;
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[\{38E77B11-839B-4138-B002-598026C48116\}-DlgOrder]
Dlg0=\{38E77B11-839B-4138-B002-598026C48116\}-SdWelcomeMaint-0
Count=3
Dlg1=\{38E77B11-839B-4138-B002-598026C48116\}-SprintfBox-0
Dlg2=\{38E77B11-839B-4138-B002-598026C48116\}-SdFinish-0
[\{38E77B11-839B-4138-B002-598026C48116\}-SdWelcomeMaint-0]
Result=303
[\{38E77B11-839B-4138-B002-598026C48116\}-SprintfBox-0]
Result=1
[Application]
Name=Database Manager
Version=$aRelease
Company=SAP AG
Lang=0009
[\{38E77B11-839B-4138-B002-598026C48116\}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[\{765537B8-DA4C-480E-917E-E25CAA26CE99\}-DlgOrder]
Count=0
[\{B464AF4C-EFBF-4488-82DA-E1CF26531D34\}-DlgOrder]
Count=0
[\{9829D65A-1914-418C-98B6-48008C42FFD2\}-DlgOrder]
Count=0
[\{4E709CE6-A288-462A-AE8B-E76CB9AD0342\}-DlgOrder]
Count=0
AEND
}

sub writesqlstudio75InstallResponseFile{
  my ($this, $aOut ) =@_;
  my $installDir = $this->getInstallDir();
  my $aRelease = $this->getRelease();
  print $aOut <<AEND;
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-DlgOrder]
Dlg0=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdWelcome-0
Count=5
Dlg1=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdAskDestPath-0
Dlg2=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdSelectFolder-0
Dlg3=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdStartCopy-0
Dlg4=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdFinish-0
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdWelcome-0]
Result=1
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdAskDestPath-0]
szDir=$installDir
Result=1
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdSelectFolder-0]
szFolder=MySQL MaxDB
Result=1
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdStartCopy-0]
Result=1
[Application]
Name=SQL Studio
Version=$aRelease
Company=SAP AG
Lang=0009
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[\{BDF33DF1-74DD-42F7-B1E4-5FB009F322CC\}-DlgOrder]
Count=0
[\{40314691-03B7-4446-BD07-455BB5065FB8\}-DlgOrder]
Count=0
[\{1266F848-EDF7-482A-AC08-8572932505A1\}-DlgOrder]
Count=0
[\{3002A6A5-63C2-41ED-85DE-3438A737AF4C\}-DlgOrder]
Count=0
[\{F2AEC4C5-45F1-46D6-9E04-28C59D5CAA30\}-DlgOrder]
Count=0
[\{B75C64BC-0006-4437-9F51-641B18904F8B\}-DlgOrder]
Count=0
[\{05F72A77-4A2A-4B9E-B5E7-E53AE156FDF3\}-DlgOrder]
Count=0
AEND
}                               #writesqlstudio75InstallResponseFile




sub writesqlstudio75UninstallResponseFile{
  my ($this, $aOut ) =@_;
  my $aRelease = $this->getRelease();
  print $aOut <<AEND;
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-DlgOrder]
Dlg0=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdWelcomeMaint-0
Count=3
Dlg1=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SprintfBox-0
Dlg2=\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdFinish-0
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdWelcomeMaint-0]
Result=303
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SprintfBox-0]
Result=1
[Application]
Name=SQL Studio
Version=$aRelease
Company=SAP AG
Lang=0009
[\{6A729C9E-BC1A-4F20-9EA8-95E1570545F1\}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[\{BDF33DF1-74DD-42F7-B1E4-5FB009F322CC\}-DlgOrder]
Count=0
[\{40314691-03B7-4446-BD07-455BB5065FB8\}-DlgOrder]
Count=0
[\{1266F848-EDF7-482A-AC08-8572932505A1\}-DlgOrder]
Count=0
[\{3002A6A5-63C2-41ED-85DE-3438A737AF4C\}-DlgOrder]
Count=0
[\{F2AEC4C5-45F1-46D6-9E04-28C59D5CAA30\}-DlgOrder]
Count=0
[\{B75C64BC-0006-4437-9F51-641B18904F8B\}-DlgOrder]
Count=0
[\{05F72A77-4A2A-4B9E-B5E7-E53AE156FDF3\}-DlgOrder]
Count=0
AEND
}                               #writesqlstudio75UninstallResponseFile

sub writesqlstudio75UpdateResponseFile{
  my ($this, $aOut ) =@_;
  my $aRelease = $this->getRelease();
  print $aOut <<AEND;
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[{6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-DlgOrder]
Dlg0={6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdWelcome-0
Count=3
Dlg1={6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdStartCopy-0
Dlg2={6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdFinish-0
[{6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdWelcome-0]
Result=1
[{6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdStartCopy-0]
Result=1
[Application]
Name=SQL Studio
Version=$aRelease
Company=SAP AG
Lang=0009
[{6A729C9E-BC1A-4F20-9EA8-95E1570545F1}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[{CDEE62EA-7780-438C-BEE2-BEFF1C4E0D90}-DlgOrder]
Count=0
[{C5813181-443D-4786-8E7A-8942D357B7F0}-DlgOrder]
Count=0
[{74D357CA-ACF2-4BBD-AD30-9F873A8C4E6B}-DlgOrder]
Count=0
[{28488D74-4BBF-463F-9F83-10E38E5EFB10}-DlgOrder]
Count=0
[{E5DDC44F-C6A7-421B-AEC1-0AD3C883EB64}-DlgOrder]
Count=0
[{D6BA7E1B-4B0B-48C8-ACB4-4E5CC3754376}-DlgOrder]
Count=0
[{19CD7770-1B50-49BA-9025-72586042A74D}-DlgOrder]
Count=0
AEND
}                               #writesqlstudio75UninstallResponseFile

sub writedbm75UpdateResponseFile{
  my ($this, $aOut ) =@_;
  my $aRelease = $this->getRelease();
  print $aOut <<AEND;
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[{38E77B11-839B-4138-B002-598026C48116}-DlgOrder]
Dlg0={38E77B11-839B-4138-B002-598026C48116}-SdWelcome-0
Count=3
Dlg1={38E77B11-839B-4138-B002-598026C48116}-SdStartCopy-0
Dlg2={38E77B11-839B-4138-B002-598026C48116}-SdFinish-0
[{38E77B11-839B-4138-B002-598026C48116}-SdWelcome-0]
Result=1
[{38E77B11-839B-4138-B002-598026C48116}-SdStartCopy-0]
Result=1
[Application]
Name=Database Manager
Version=$aRelease
Company=SAP AG
Lang=0009
[{38E77B11-839B-4138-B002-598026C48116}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[{AAE16ED0-5751-4059-A3AC-C8CEEA9EC206}-DlgOrder]
Count=0
[{CC2821B5-2907-4B12-AA15-73F545F171D2}-DlgOrder]
Count=0
[{21D3C994-574E-4D5C-8B5A-13A8CB209AAB}-DlgOrder]
Count=0
[{38C6A71B-4176-49C2-814E-D078808CCF5B}-DlgOrder]
Count=0
AEND
}






sub writedbm75InstallResponseFile{
  my ($this, $aOut ) =@_;
  my $installDir = $this->getInstallDir();
  my $aRelease = $this->getRelease();
  print $aOut <<AEND;
[InstallShield Silent]
Version=v6.00.000
File=Response File
[File Transfer]
OverwrittenReadOnly=NoToAll
[\{38E77B11-839B-4138-B002-598026C48116\}-DlgOrder]
Dlg0=\{38E77B11-839B-4138-B002-598026C48116\}-SdWelcome-0
Count=5
Dlg1=\{38E77B11-839B-4138-B002-598026C48116\}-SdAskDestPath-0
Dlg2=\{38E77B11-839B-4138-B002-598026C48116\}-SdSelectFolder-0
Dlg3=\{38E77B11-839B-4138-B002-598026C48116\}-SdStartCopy-0
Dlg4=\{38E77B11-839B-4138-B002-598026C48116\}-SdFinish-0
[\{38E77B11-839B-4138-B002-598026C48116\}-SdWelcome-0]
Result=1
[\{38E77B11-839B-4138-B002-598026C48116\}-SdAskDestPath-0]
szDir=${installDir}
Result=1
[\{38E77B11-839B-4138-B002-598026C48116\}-SdSelectFolder-0]
szFolder=MySQL MaxDB
Result=1
[\{38E77B11-839B-4138-B002-598026C48116\}-SdStartCopy-0]
Result=1
[Application]
Name=Database Manager
Version=$aRelease
Company=SAP AG
Lang=0009
[\{38E77B11-839B-4138-B002-598026C48116\}-SdFinish-0]
Result=1
bOpt1=0
bOpt2=0
[\{765537B8-DA4C-480E-917E-E25CAA26CE99\}-DlgOrder]
Count=0
[\{B464AF4C-EFBF-4488-82DA-E1CF26531D34\}-DlgOrder]
Count=0
[\{9829D65A-1914-418C-98B6-48008C42FFD2\}-DlgOrder]
Count=0
[\{4E709CE6-A288-462A-AE8B-E76CB9AD0342\}-DlgOrder]
Count=0
AEND
}

sub writeLogAndDisplayStatus{
  my ($this,$message) = @_;
  my $aLogCallback = $this->getLogCallback();
  my $aStatusPutter = $this->getStatusCallback();
  &$aStatusPutter($message);
  &$aLogCallback($message);
}


sub getInstalledVersion(){
  my ($this) = @_;
  my $aVersionKey = $this->getVersionKey();
  my $installedVersion = $this->getInstalledVersionForRegistryKey($aVersionKey);
  if(not $installedVersion){
    my $r74=$aVersionKey;
    $r74 =~ s/\d\.\d$/7\.4/;
    my $r75 = $aVersionKey;
    $r75 =~ s/\d\.\d$/7\.5/;
    my $r76 = $aVersionKey ;
    $r76 =~ s/\d\.\d$/7\.6/;
    foreach my $key ($r74,$r75,$r76){
      unless($key eq $aVersionKey){
        $installedVersion = $this->getInstalledVersionForRegistryKey($key);
        if($installedVersion){
          last;
        }
      }
    }
  }
  return $installedVersion;
}
sub getInstalledVersionForRegistryKey(){
  my ($this,$aRegistryKey) = @_;
  my  %aValueHash=readValues(0,$aRegistryKey);
  my $installedVersion = $aValueHash{'Version'};
  return $installedVersion;
}


#ifdef sub:willSkip
#<p>Returns <code>1</code>, if the installation will be skipped, else <code>0</code></p>
#<p>The installation will be skipped, iff:</p>
#  <ol>
#  <li>A newer release is installed</li>
#  <li>There same release is installed</li>
#  </ol>
#
#endif
#+sub():int:Will the installation be skipped
sub willSkip{
  my ($this) = @_;
  my $installedVersion=$this->getInstalledVersion();
  my $aVersion= $this->getVersion();
  my $retVal = 0;
  if ( $installedVersion and $aVersion le $installedVersion) {
      $retVal=1;
  }
  return $retVal;
}


#+sub():void:Install the software
sub install{
  my ($this) = @_;
  my $aName = $this->getName();
  my $aVersion= $this->getVersion();
  my $installedVersion = $this->getInstalledVersion();
  my $aResponseFile = $this->getResponseFileInstall();
  my $aResponseDir = dirname($aResponseFile);
  unless(-d $aResponseDir){
    mkpath($aResponseDir);
    unless (-d $aResponseDir){
      $this->writeLogAndDisplayStatus("ERROR: Could not create directory $aResponseDir.\n");
      return -3;
    }
  }
  my $aLogfile = $this->getLogFileInstall();
  my $aLogDir = dirname($aLogfile);
  unless(-d $aLogDir){
    mkpath($aLogDir);
    unless (-d $aLogDir){
      $this->writeLogAndDisplayStatus("ERROR: Could not create directory $aLogDir.\n");
      return -4;
    }
  }

  if ( $installedVersion) {
    if ( $aVersion eq $installedVersion) { 
      $this->writeLogAndDisplayStatus
        ("ERROR: $aName $installedVersion already installed (Skipping)\n");
      return -1;
    } elsif ( $aVersion lt $installedVersion) {
      $this->writeLogAndDisplayStatus
        ("ERROR: $aName $installedVersion already installed, will not install $aVersion (Older!)\n");
      return -1;
    } else {
      $this->writeLogAndDisplayStatus
        ("INFO: $aName $installedVersion already installed, " .
         "trying to update to newer version  $aVersion.\n");
      unless (open RESPONSEFILE, ">" . $aResponseFile){
        $this->writeLogAndDisplayStatus( "Cannot open $aResponseFile for write\n");
        return -2;
      } 
      $this->writeUpdateResponseFile(*RESPONSEFILE);
      close RESPONSEFILE;
    }
  } else {
    unless (open RESPONSEFILE, ">" . $aResponseFile){
      $this->writeLogAndDisplayStatus( "Cannot open $aResponseFile for write\n");
      return -2;
    } 
    $this->writeInstallResponseFile(*RESPONSEFILE);
    close RESPONSEFILE;
  }

  my $result = $this->runExecutable("installing",$aResponseFile,$aLogfile);
  # result nicht definiert oder 0 bedeutet Fehler!
  if($result){
    $this->writeLogAndDisplayStatus("Installed $aName\n");
    return 1;
  }
  else{
    return -5;
  }
  #redundant code:
  return 1;
}
sub installNew{
}
#+sub():void:Uninstall the software
sub uninstall{
  my ($this) = @_;
  my $aVersion= $this->getVersion();
  my $aName = $this->getName();
  my $installedVersion = $this->getInstalledVersion();
  my $aResponseFile = $this->getResponseFileUninstall();

  my $aResponseDir = dirname($aResponseFile);
  unless(-d $aResponseDir){
    mkpath($aResponseDir);
    unless (-d $aResponseDir){
      $this->writeLogAndDisplayStatus("ERROR: Could not create directory $aResponseDir.\n");
      return -3;
    }
  }

  my $aLogfile = $this->getLogFileUninstall();

  my $aLogDir = dirname($aLogfile);
  unless(-d $aLogDir){
    mkpath($aLogDir);
    unless (-d $aLogDir){
      $this->writeLogAndDisplayStatus("ERROR: Could not create directory $aLogDir.\n");
      return -4;
    }
  }

  unless( $installedVersion){
    $this->writeLogAndDisplayStatus("ERROR: $aName $aVersion not installed (Skipping)\n");
    return -1;
  }
  if ( defined $installedVersion and not $installedVersion  eq $aVersion) {
    $this->writeLogAndDisplayStatus("ERROR: $aName $aVersion not installed (Version $installedVersion is installed). Will do nothing!\n");
    return -2;
  }
  unless (open RESPONSEFILE, ">" . $aResponseFile){
    writeLogAndDisplayStatus "ERROR: Cannot open $aResponseFile for write: $!\n";
    return -5;
  } 
  $this->writeUnInstallResponseFile(*RESPONSEFILE);
  close RESPONSEFILE;
  my $result = $this->runExecutable("uninstalling",$aResponseFile,$aLogfile);
  # result nicht definiert oder 0 bedeutet Fehler!
  if($result){
    $this->writeLogAndDisplayStatus("Uninstalled $aName\n");
    return 1;
  }
  else{
    return -6;
  }
  return 1;
}



sub configure{
  my ($this) = @_;
  my @configResult = (1);
  my $retVal = $this->initTempFiles();
  if ($retVal) {
    @configResult = (0, $this->getErrorMessage(),
                     $this->getErrorDescription()? $this->getErrorDescription() : "");
  }
  return @configResult;
}



sub initTempFiles{
  my ($this) = @_;
  my $retVal=0;
  my $untgz = $this->getArchiveUntgz();
  my $archive= $this->getArchive();
  if (defined $untgz or defined $archive) {
    $retVal = $this->unpackArchive();
  }
  my $tempDir = undef;
  unless($retVal){
    $tempDir = $this->getTempDir();
    unless(defined $tempDir){
      $retVal=1;
    }
  }
  unless($retVal){
    my  $aResponseFileInstall = catfile($tempDir,"install_". $this->getDirName() . ".iss");
    my  $aResponseFileUnInstall = catfile($tempDir,"uninstall_" . $this->getDirName() . ".iss" );
    my  $aLogFileInstall = catfile($tempDir,"install_" . $this->getDirName() . ".log" );
    my  $aLogFileUnInstall = catfile($tempDir,"uninstall_" . $this->getDirName() . ".log" );
    $this->setResponseFileInstall($aResponseFileInstall);
    $this->setResponseFileUninstall($aResponseFileUnInstall);
    $this->setLogFileInstall($aLogFileInstall);
    $this->setLogFileUninstall($aLogFileUnInstall);
  }
  return $retVal;
}


sub unpackArchive{
  my ($this) = @_;
  my $retVal=0;

  my $untgz = $this->getArchiveUntgz();
  unless(defined $untgz){
    my $archive= $this->getArchive();
    if (defined $archive) {
      $untgz = SAPDB::Install::Untgz::new ();
      my $archiveSuccess = $untgz->Open($archive);
      unless (defined $archiveSuccess 
              && $archiveSuccess == 0){
        $retVal = 1;
        $this->setErrorMessage("Cannot open archive \"$archive\"");
      }
    } else {
      $retVal=5;
      $this->setErrorMessage("Cannot open archive: Archive istnot known");
    }
  }
  my $realArchivedInstallable=undef;
  my $archivedInstallable = $this->getArchivedInstallable();
  unless($retVal){
    $untgz->Rewind();
    my @hrd = $untgz->Next();
    while (1) {
      unless(defined $hrd[0]){
        last;
      }
      my $maybeArchivedInstallable = $hrd[3]; 
      if ( (uc $archivedInstallable) eq (uc $maybeArchivedInstallable)) {
        $realArchivedInstallable = $maybeArchivedInstallable;
        last;
      }
      @hrd = $untgz->Next();
    }
  }

  unless(defined $realArchivedInstallable){
    $retVal = 2;
    $this->setErrorMessage("Did not find installable  $archivedInstallable in archive");
  }
  my $tempDir = $this->getTempDir();
  unless(defined $tempDir){
    $retVal=3;
  }
  unless($retVal){
    $untgz->SetDestDir($tempDir);
    $untgz->SetReadOnly(0);
    $untgz->ExtractFile();
    if ($untgz->GetErr()) {
      $retVal=4;
      $this->setErrorMessage($untgz->GetErr());
    } else {
      $this->setExecutable(catfile($tempDir,$realArchivedInstallable));
    }
  }
  #   0 bedeutet richtig schreiben, 1 bedeutet Testlauf
  return $retVal;
}




sub runExecutable{
  my ($this,$action,$aResponse,$aLog) = @_;
  my $aExe = $this->getExecutable();

  unless (-f $aExe){
    my $name = $this->getName();
    $this->writeLogAndDisplayStatus("Cannot find file \"$aExe\" to uninstall or install $name\n");
    return 0;
  }
  my $aLogCallback = $this->getLogCallback();
  my $aStatusPutter = $this->getStatusCallback();
  my $aName = $this->getName();
  my $command = $aExe . " -s -a -s  -f1" . $aResponse . " -f2" . $aLog;
  my $aVersionKey = $this->getVersionKey();

  &$aLogCallback("Running \"$command\"\n");
  #  system "$command";
  callsystem($command, 
             sub{
               my $line = $_[0];
               return 1;
             },
             1);
  &$aLogCallback("Finished \"$command\"\n");


  unless(-f $aLog){
    $this->writeLogAndDisplayStatus("Error $action $aName.\n(No more information available, no logfile found).\n");
    return 0;
  }                             #unless
  open LOGFILE, $aLog;
  my $anError;
  my $section;
  while ( <LOGFILE>) {
    my ($key,$value,$type);
    parseConfigLine($_,
                    sub{ $section =$_[0];},
                    sub {$key=$_[0];
                         $value=$_[1];
                         if (defined $_[2]) {
                           $type=$_[2];
                         }
                       }
                   );
    if ( defined $key and $key eq "ResultCode" and defined $section and $section eq "ResponseResult") {
      $anError = $value;
    }                           #if
  }                             #while
  if ( $anError) {
    $this->writeLogAndDisplayStatus("Error $action $aName. (Log is  \"$aLog\").\n");
    return 0;
  }                             #if
  return 1;
}                               #runExecutable


sub new{
  my $this = {};

  bless $this;
  return $this;
}
#new


sub setPriorities{
  my ($this) = @_;
  my $params = $this->getParams();
}

#+sub():WindowsInstallerRef:Konstruktor for an Object to install "Database Manager"
sub DBM{
  my $this = new();
  $this->setName("Database Manager");
  $this->setDirName("DBM");
  $this->setVersion("7.5.00.18");
  $this->addAfterAdvice("releaseKey",sub{
                          my ($this) = @_;
                          my $aRelease = $this->getRelease();
                          $this->setVersionKey("SOFTWARE\\SAP AG\\Database Manager\\$aRelease");
                        }
                       );
  $this->setAdvicedRelease("7.5");
  $this->setInstallResponseFileWriter(\&writedbm75InstallResponseFile);
  $this->setUpdateResponseFileWriter(\&writedbm75UpdateResponseFile);
  $this->setUninstallResponseFileWriter(\&writedbm75UninstallResponseFile);
  $this->setStatusCallback(sub{print "STATUS: " . $_[0];});
  $this->setLogCallback(sub{print "LOG: " . $_[0];});
  $this->setExecutable("dbm75.exe");
  $this->setLogFileInstall("install.log");
  $this->setLogFileUninstall("install.log");
  $this->setResponseFileInstall("install.iss");
  $this->setResponseFileUninstall("remove.iss");
  $this->setInstallDir("c:\\sdb\\programs\\DBM");
  $this->setVersionKey("SOFTWARE\\SAP AG\\Database Manager\\" . $this->getRelease());
  $this->setArchivedInstallable("dbm75.exe");
  $this->setPriorities();
  return $this;
}

#+sub():WindowsInstallerRef:Konstruktor for an Object to install "SQL Studio"
sub SQLSTUDIO{
  my $this = new();
  $this->setName("SQL Studio");
  $this->setDirName("SQLStudio");
  $this->setVersion("7.5.00.18");
  $this->addAfterAdvice("releaseKey",sub{
                          my ($this) = @_;
                          my $aRelease = $this->getRelease();
                          $this->setVersionKey("SOFTWARE\\SAP AG\\SQL Studio\\$aRelease");
                        }
                       );
  $this->setAdvicedRelease("7.5");
  $this->setInstallResponseFileWriter(\&writesqlstudio75InstallResponseFile);
  $this->setUpdateResponseFileWriter(\&writesqlstudio75UpdateResponseFile);
  $this->setUninstallResponseFileWriter(\&writesqlstudio75UninstallResponseFile);
  $this->setStatusCallback(sub{print "STATUS: " . $_[0];});
  $this->setLogCallback(sub{print "LOG: " . $_[0];});
  $this->setExecutable("sqlstd75.exe");
  $this->setLogFileInstall("install_studio.log");
  $this->setLogFileUninstall("uninstall_studio.log");
  $this->setResponseFileInstall("install_studio.iss");
  $this->setResponseFileUninstall("uninstall_studio.iss");
  $this->setInstallDir("c:\\sdb\\programs\\SQLStudio");
  $this->setArchivedInstallable("sqlstd75.exe");
  $this->setPriorities();
  return $this;
}

##+sub(StreamRef:Reference to a output stream):void:Writes the update response file
sub writeUpdateResponseFile{
  my ($this,$output) = @_;
  &{$this->getUpdateResponseFileWriter()}($this,$output);
}

##+sub(StreamRef:Reference to a output stream):void:Writes the installation response file
sub writeInstallResponseFile{
  my ($this,$output) = @_;
  &{$this->getInstallResponseFileWriter()}($this,$output);
}

##+sub(StreamRef:Reference to a output stream):void:Writes the uninstallation response file
sub writeUnInstallResponseFile{
  my ($this,$output) = @_;
  &{$this->getUninstallResponseFileWriter()}($this,$output);
}

sub writeConfig($$){
  my ($this,$write) = @_;
  &$write("[". $this->getName() . "]\n");
  my $paramsRef = $this->getParams();
  defaultWriteParams($paramsRef,$this->getName(),$write);
}
#end writeConfig();

sub wiframestart{
  return 1;
}

sub wipreinstall{
  my ($paths,$config) = @_;
  my ($indep_data,$indep_prog)=readIndepPath(); # try to get independent paths from earlier installation
  my $path = $paths->[0];
  my $default =   $path->{'default'};
  $path->{'value'}=catfile($indep_prog,$default);
  return 1;
}
sub wipostprepare{
  return 1;
}


sub createInstanceForInstall{
  my ($paths,$config) = @_;
  my $tgz =   $config->{'tgz_handle'};
  my $name =   $config->{'name'};
  my $archive =   $config->{'archive_name'};

  my $aVersion =   $config->{'version'};
  my $archivedInstallable =   $config->{'archived_installable'};
  my $installer;
  if ($name eq 'SQL Studio') {
    $installer = SQLSTUDIO();
  } elsif ($name eq 'Database Manager') {
    $installer =  DBM();
  }
  $installer->setArchiveUntgz($tgz);
  $installer->setStatusCallback(\&status);
  $installer->setLogCallback(\&logger);
  if (defined $aVersion) {
    $installer->setVersion($aVersion);
  }
  if ( $archivedInstallable) {
    $installer->setArchivedInstallable($archivedInstallable);
  }
  if ( $archive) {
    $installer->setArchive($archive);
  }
  return $installer;
}#createInstanceForInstall


sub staticWillSkip{
  my ($paths,$config) = @_;
  my $instance = createInstanceForInstall($paths,$config);
  return $instance->willSkip();
}


sub wipostuninstall{
  return doit(@_,'uninstall');
}#wipostuninstall




sub doit{
  my ($paths,$config,$action) = @_;
  my $installer  = createInstanceForInstall($paths,$config);
  my ($configResult) = $installer->configure();
  my $name =  $installer->getName();
  my $msg ; 
  if ( $configResult == 0) {
    my $m = $installer->getErrorMessage();
    $msg = "Error in configuration for $name: $m\n";
    logger($msg);
    status($msg);
#because SDBINST
    diesoft ("Failed to install/deinstall $name: $m\n");
#endbecause
    return undef;
  }
  $installer->configureFromIndepPath();
  my $retVal = 0;
  if ( $action eq 'install') {
    $retVal = $installer->install();
#because SDBINST
    if($retVal < 0){
      diesoft ("Failed to install $name: " . $retVal[1] . "\n");
    }
#endbecause
  } elsif ($action eq 'uninstall') {
    $retVal = $installer->uninstall();
  }
  logger("WindowsInstaller returned " . $retVal . "\n");
  return $retVal;
}                               #doit


sub wiregister{
  return 1;
}

sub wiframestop{
  return 1;
}
sub wiunregister{
}
sub wipreuninstall{
  return wipreinstall(@_);
}
sub wipostinstall{
  return doit(@_,'install');
}



sub wiverify{
}
sub logger{
#because SDBINST
  $SAPDB::Install::Values::log->SetMsg($_[0]); 
#endbecause
  #ifdef SCRIPTING
#  print "LOG: ". $_[0];
  #endif
}

sub status{
#because SDBINST
  print2stdout("\n" . $_[0]);
#endbecause
  #ifdef SCRIPTING
#  print "STATUS: ". $_[0];
  #endif
}



1;
# Local Variables: ***
# perl-dbg-flags:"-w      -I../../../../../../wrk/script -I../../../sdbrun  " ***
# sdbinst-defines: "-define SCRIPTING=YES -define PERL5.6.1=YES" ***
# sdbinst-testfile: "WindowsInstallerTest.pl" ***
# sdbinst-script-directory:  "sys/wrk/script/SAPDB/Install/Templates" ***
# End: ***



