package SAPDB::Install::Templates2::Archiver;

use SAPDB::Install::System;
use SAPDB::Install::StdIO;
use SAPDB::Install::Tools;
use SAPDB::Install::Values;
use SAPDB::Install::Trace;
use SAPDB::Install::SysInfo;
use SAPDB::Install::Untgz;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::Debug;




our @ISA;
if($SCRIPTING){
  require Exporter;
  push @ISA,qw(Exporter);
}
else{
  require SDB::Common::Exporter;
  push @ISA,qw(SDB::Common::Exporter);
}
push  @ISA, qw( SAPDB::Install::Templates2::Properties);
our @EXPORT = qw (getPackageData getAllPackageData);

my $sysinfo = GetSystemInfo();

#sub reads package signatur and return it as hash
sub readPackageData{
	my %values = %SAPDB::Install::Values::package_interface;
	my ($untgz)=@_;
	my $datastring = readFileInTGZ($untgz,'PACKAGEDATA');
	$datastring or return undef;
	my @filedata=split("\n",$datastring);
	@filedata or print("WRN: package \"$tgzfile\" has no package signature\n") ;
	foreach my $line (@filedata){
		foreach my $key (keys(%values)){
			if($line =~ /^\s*$key\s*=\s*"(.*)"\s*$/){
				$values{$key} = $1;
				last;
			}
		}
	}
	return \%values;
}


sub removeElementFromList{
	my ($delete,@list)=@_;
	my @newlist;
	foreach my $element (@list){
		$element == $delete or push @newlist,$element;
	}
	return @newlist;
}

# parse require strings like "Packegename 1 = 7.2.5.3 , Packegename 2 >= 7.3.0.0"

sub parseRequire{
	my ($reqString)=@_;
	my @ops=('>=','<=','=','>','<'); # operators in search order
	my $delimiter = ',';
	my @packages = split($delimiter,$reqString);
	my @returnvalue;
	my $first = 1;
	foreach my $package (@packages){
		$_ = $package;
		my %required_package;
		foreach my $operator (@ops){
			if(/$operator/){
				if(/^\s*(\S.*\S)\s*$operator\s*([0-9]{1,}\.[0-9]{1,}\.[0-9]{1,}\.[0-9]{1,})\s*(\d*)\s*/){
					# found valid operator and release -> set hash 
					$required_package{'name'} = $1;
					$required_package{'operator'} = $operator;
					$required_package{'release'} = $2;
					$required_package{'bit'} = $3 if ($3 == 32 or $3 == 64); 		
					last;
				}

				# found operand without valid release -> ignore operator and wrong release
				s/\s*$operator.*$//; 
				s/^\s*//;
				$required_package{'name'} = $_;
				last;
			}
		}
		$required_package{'first'} = $first;
		$first = 0;
		unless(exists $required_package{'name'}){
			s/^\s*//;
			s/\s*$//;
			$required_package{'name'} = $_;	
		}
		push(@returnvalue,\%required_package);
	}
	return @returnvalue; # return an array of hash references; hashes with keys: name (required), operator (optional) and release (optional) 
}


sub checkRequired{
	my (%args) = @_;
	my %required = %{$args{'required'}};
	my $pack_ok = 0;
	if($args{'package_name'} eq $required{'name'}){
		if(exists $required{'operator'}){
			TraceMsg("TEST WITH OTHER PACKAGE ".release2num($args{'version'})." $required{'operator'} ".release2num($required{'release'})."\n",4,\$DEBUG); 
			$_=$required{'operator'};
			SWITCH:{
				/^=$/ && do{
								release2num($args{'version'}) == release2num($required{'release'}) or last SWITCH;
								if($required{'bit'}){
									$required{'bit'} eq $args{'mode'} or TraceMsg("object mode dont match\n",4,\$DEBUG) and last SWITCH;
								}
								$pack_ok=1;
								last SWITCH;
							};	
				/^>$/ && do{
								release2num($args{'version'}) > release2num($required{'release'}) or last SWITCH;
								if($required{'bit'}){
									$required{'bit'} eq $args{'mode'} or TraceMsg("object mode dont match\n",4,\$DEBUG) and last SWITCH;
								}
								$pack_ok=1;
								last SWITCH;
							};
				/^<$/ && do{
								release2num($args{'version'}) < release2num($required{'release'}) or last SWITCH;
								if($required{'bit'}){
									$required{'bit'} eq $args{'mode'} or TraceMsg("object mode dont match\n",4,\$DEBUG) and last SWITCH;
								}
								$pack_ok=1;
								last SWITCH;
							};
				/^>=$/ && do{
								release2num($args{'version'}) >= release2num($required{'release'}) or last SWITCH;
								if($required{'bit'}){
									$required{'bit'} eq $args{'mode'} or TraceMsg("object mode dont match\n",4,\$DEBUG) and last SWITCH;
								}
								$pack_ok=1;
								last SWITCH;
							};
				/^<=$/ && do{
								release2num($args{'version'}) <= release2num($required{'release'}) or last SWITCH;
								if($required{'bit'}){
									$required{'bit'} eq $args{'mode'} or TraceMsg("object mode dont match\n",4,\$DEBUG) and last SWITCH;
								}
								$pack_ok=1;
								last SWITCH;
							};
			}
		}
		else{
			
			$pack_ok = 1;
		}
	}
	return $pack_ok;
}



#resolve dependencies

sub resolve{
	my ($instRegistry_ref,$packages_ref,$dependency_ref)=@_;
	my $instRegistry = $$instRegistry_ref;
	local *packages = $packages_ref; #array of hash references
	local *dependency=$dependency_ref; # hash, keys: name (required), operator (optional) and release (optional)
	exists $dependency{'name'} or return;
	my $pack_ok=0;
	my @nextdependencies;
	my $found_package;
	foreach my $package_ref (@packages){
		local *package=$package_ref; #hash, keys: see INTERFACE in package Values
		
		$pack_ok = checkRequired('mode' => $package{'MODE'},
								'version' => $package{'SOFTWARE_VERSION'},
								'package_name' => $package{'PACKAGE_NAME'},
								'required' => $dependency_ref);
		if($pack_ok){
				@nextdependencies=parseRequire($package{'REQUIRE'});	
				$found_package=$package_ref;
				TraceMsg("DEPENDENCY CHECK WITH OTHER PACKAGES: package \"$dependency{'name'}\" ok\n",4,\$DEBUG);
				$SAPDB::Install::Values::log->SetMsg("dependencies for package \"$dependency{'name'}\" resolved\n");
				if($#nextdependencies == -1){
					#TraceMsg("package \"$dependency{'name'}\" has no dependencies\n",4,\$DEBUG);
					$SAPDB::Install::Values::log->SetMsg("package \"$dependency{'name'}\" has no dependencies\n");
					return (\%package);
				}				
			last;
		}
	}
	if($pack_ok){
		my @returnlist;
		foreach my $next_one (@nextdependencies){
			push(@returnlist,resolve($instRegistry_ref,$packages_ref,$next_one));
		}
		push @returnlist,$found_package;
		return @returnlist;
	}
	elsif(defined $instRegistry){
		
		my %releases = $instRegistry->getInstallPathes($dependency{'name'},1);
		my $pack_ok=0;
		foreach my $install_path (keys(%releases)){
			#print "$install_path -> $releases{$install_path}\n";
			
			
			my %data = ();
			
			if($dependency{'bit'} == 32 or $dependency{'bit'} == 64){
				%data = $instRegistry->getPackageData($dependency{'name'},$install_path,1);
			}
			
			$pack_ok = checkRequired(
							'required' => \%dependency,
							'package_name' => $dependency{'name'},
							'version' => $releases{$install_path},	
							'mode' => $data{'Mode'}
			);
			
			$pack_ok and last; 
		}
		if($pack_ok){
			TraceMsg("DEPENDENCY CHECK IN REGISTRY: package \"$dependency{'name'}\" ok\n",4,\$DEBUG);
			$SAPDB::Install::Values::log->SetMsg("reqired package $dependency{'name'} $dependency{'operator'} $dependency{'release'} in package registry found\n");
			return ();
		}
		
		# unresolved dependency - no such package collected or installed 
		
	}
	print2stderr("unresolved dependency - package $dependency{'name'} $dependency{'operator'} $dependency{'release'} not found\n");
	diesoft($SAPDB::Install::Values::diemsg) unless $SAPDB::Install::Values::GUI_MODE;
	die('dependency error');
}


sub removeDoubles{
	my (@input)=@_;
	my @return;
	foreach my $element (@input){
		return undef unless defined $element;  
		my $check=0;
		foreach my $inner (@return){
			$inner eq $element and $check=1 and last;
		}
		$check or push @return,$element;
	}
	return @return;
}



# resolve dependencies and sort packages in install order

sub sortpackages{
	my ($instRegistry_ref,@hashref)=@_;
	my $instRegistry=$$instRegistry_ref;
	my @sortlist;
	foreach my $pack_ref (@hashref){
		local *package=$pack_ref;
		my @depends=parseRequire($package{'REQUIRE'});
		foreach my $dep (@depends){
			push @sortlist,resolve($instRegistry_ref,\@hashref,$dep);
		}
		push @sortlist,$pack_ref;
	}
	@sortlist=removeDoubles(@sortlist);
	return @sortlist;
}



sub getAllPackageData{
	my ($dir) = @_;
	my @tgzfiles;
        my $retVal = 0;
	opendir(DH,$dir) or return ("Cannot open $dir: $!",undef);
	@tgzfiles = grep { /\.tgz$/i && -f "$dir/$_" && ! /sdbinst.tgz|wxperl.tgz|resources.tgz|wxperl.tgz|sqldbc-man.tgz/i } readdir(DH);
	closedir(DH);
	($#tgzfiles == -1) and return ("No packages found in $dir",undef); #no tgz file found
	my @profiles;
	my @packages;
	foreach my $file (@tgzfiles){
#			print("Found archive \"$file\" in directory $dir\n");
			my $untgz = SAPDB::Install::Untgz::new();
			$untgz->Open(catfile($dir,$file));
			if($untgz->GetErr){
				print("cannot open archive $file: ".join(': ',$untgz->GetErr)."\n");
				next;
			}
			my $packdataRef = readPackageData($untgz);
                        if($packdataRef){
                          $packdataRef->{'UNTGZ'}=$untgz;
                          $packdataRef->{'ARCHIVE_NAME'}=$file;
                          $packdataRef->{'ARCHIVE_DIR'}=$dir;
                          $packdataRef->{'PACKAGE_NAME'} or print "No package name found for $file";
                          my %packdata = %$packdataRef;
                          push @packages,\%packdata ;
                           my $top = $packdata{'IS_TOP_OF'};
                           if($top){
                             @profiles = union(\@profiles,[$top]) ;
                           }
                        }
                        else{
                          print "No package data found for $file";
                        }
	}
	return ($retVal,{'profiles' => \@profiles, 'packages' => \@packages});
}

sub getPackageData{
  my ($dir,$file) = @_;
  my $untgz = SAPDB::Install::Untgz::new();
  my $retVal=0;
  $untgz->Open(catfile($dir,$file));
  $retVal = $untgz->GetErr and return $retVal;
  my $packdataRef = readPackageData($untgz);
  if ($packdataRef) {
    $packdataRef->{'UNTGZ'}=$untgz;
    $packdataRef->{'ARCHIVE_NAME'}=$file;
    $packdataRef->{'ARCHIVE_DIR'}=$dir;
    $packdataRef->{'PACKAGE_NAME'} or print "No package name found for $file";
    my %packdata = %$packdataRef;
  } else {
    return ( "No package data found for $file",undef);
  }
  return ($retVal,$packdataRef);
}
1;
