package  SAPDB::Install::Templates2::test::DatabaseInstanceTest;
use SAPDB::Install::Templates2::InstUnit;
use SAPDB::Install::Templates2::InstTemplate;
use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Templates2::Util;
use SAPDB::Install::Templates2::test::DatabaseInstanceGeneric;
use strict;

if($SCRIPTING){
  require Data::Dumper; 
  import Data::Dumper;
}



our @ISA;
if($SCRIPTING){
  require Exporter;
  push @ISA,qw(Exporter);
}
else{
  require SDB::Common::Exporter;
  push @ISA,qw(SDB::Common::Exporter);
}
our @EXPORT = qw (allTests);


my ($db); 
sub setUp{
  $db = SAPDB::Install::Templates2::test::DatabaseInstanceGeneric::new() or die;
  my $template = $db->getInstTemplate();
  push @{$template->GetListeners('log')},(sub {Logit($_[0]);});
  push @{$template->GetListeners('warning')},(sub {Logit( "WARNING: " . $_[0]);});
  push @{$template->GetListeners('debug')},(sub{Debugit(@_);});
  push @{$template->GetListeners('async_status')},(sub{Statusit(@_);});
}


sub tearDown{
  undef $db;
}

sub testGetDomainPass{
  my $p = $db->getDomainPass();
  defined $p  or return "No domain password";
  $p eq 'domain' or return "Wrong domain password: '$p'";
  return 0;
}

sub testLoadSampleSchema{
  my $loadSampleSchema = $db->getLoadSampleSchema();
  return 0;
}
sub testGetDBVolume{
  my $vols = $db->getDBVolume();
  foreach(@$vols){
    my ($num,$type,$path,$kind,$size)= split/;/; 
  }
#  print Dumper($vols);
  return 0;
}

sub  testGetVolStructs{
  my $vols = $db->GetVolStructs();
  return 0;
}


sub  testVolumeFromStruct{
  my $vols = $db->GetVolStructs();
  foreach(@$vols){
    my $av = $db->volumeFromStruct($_,1,'DATA');
    $av or return "transformation of volume struct failed";
  }
  return 0;
}

sub testClearVolumes{
my $newVols =   $db->clearVolumes('DATA');
my $v1 = $newVols->[0];
$v1 or return "not enough volumes";
my ($num,$art) = split(/;/,$v1);
$art eq 'LOG' or return "Wrong art $art";
  return 0;
}

sub testReplaceVolumesByStruct{
  my $vols = $db->GetVolStructs();
  $vols = $db->replaceVolumesByStruct($vols,'DATA','LOG');
  return 0;
}
sub testValidateDBVolume{
  my $vols = $db->getDBVolume();
  my @res = my ($res,$txt,$msg1,$msg2) = $db->guiValidateDBVolume($vols);
  $res ==1 or return "$msg1\n$msg2";
  return 0;
}


sub testFileSystemUsageForVolumes{
  my $vols = $db->getDBVolume();
  my $res =  $db->fileSystemUsageForVolumes($vols);
  return 0;
}

sub testGuiValidateVolstructArray{
  my $vols = $db->GetVolStructs('LOG');
  my @valRes =  $db->guiValidateVolstructArray($vols,'LOG');
  return 0;
}



sub testSetParallelConnections{
  $db->setParallelConnections(3);
  my $ps = $db->getDBParams();
  my $cs = $ps->{'MAXUSERTASKS'};
  $cs or return "No MAXUSERTASKS";
  $cs eq 3 or return "Wrong MAXUSERTASK: $cs";
  return 0;
}

sub testSetDBMUser{
  $db->setDBMUser('sdf');
  my $dbm = $db->getDBManager();
  my ($user,$passwd) = split(/,/,$dbm);
  $user eq 'sdf' or return "Wrong user $user";
  return 0;
}

sub testSetDBAUser{
  $db->setDBAUser('sdf');
  my $dba = $db->getDBAdmin();
  my ($user,$passwd) = split(/,/,$dba);
  $user eq 'sdf' or return "Wrong user $user";
  return 0;
}

sub testSetDBAPass{
  $db->setDBAPass('sdf');
  my $dba = $db->getDBAdmin();
  my ($user,$passwd) = split(/,/,$dba);
  $passwd eq 'sdf' or return "Wrong password $passwd";
  return 0;
}

sub testSetDBMPass{
  $db->setDBMPass('sdf');
  my $dbm = $db->getDBManager();
  my ($user,$passwd) = split(/,/,$dbm);
  $passwd eq 'sdf' or return "Wrong password $passwd";
  return 0;
}

sub testSetDBManager{
  $db->setDBManager('user,pwd');
  my $p = $db->getDBMPass();
  my $u = $db->getDBMUser();
  $u eq 'user' or return "Wrong user $u";
  $p eq 'pwd' or return "Wrong password $p";
  return 0;
}

sub testSetDBAdmin{
  $db->setDBAdmin('user,pwd');
  my $p = $db->getDBAPass();
  my $u = $db->getDBAUser();
  $u eq 'user' or return "Wrong user $u";
  $p eq 'pwd' or return "Wrong password $p";
  return 0;
}

sub testValidateDBAdmin{
  my $user = 'dfsdfsdf';
  $db->ValidateKey($user,'dBAdmin')
    or return "Validator failed.";
    ;
  return 0;
}
sub testSetLoadSampleSchema{
  $db->setLoadSampleSchema(1);
  my $s = $db->getLoadSampleSchema();
  $s == 1 or return "Wrong value $s";
  return 0;
}

sub testCreateVolumeArray{
 my $a = $db->createVolumeArray('LOG'); 
 @$a ==1 or return "Wrong number of log volumes";
 return 0;
}

sub testCreateVolumeParameterDescriptionArray{
 my $a = $db->createVolumeParameterDescriptionArray('LOG'); 
 @$a ==1 or return "Wrong number of log volumes";
 return 0;
}

sub testCreateParameterDescriptionArray{
 my $a = $db->createParameterDescriptionArray('dataVolumes'); 
 @$a ==1 or return "Wrong number of log volumes";
 return 0;
}



sub testModifyStatusList{
  my $sl = $db->getStatusList();
  my $aa = $db->GetStatusArray();
  $sl = $db->modifyStatusList($sl,'checkVolumes','RUNNING',undef,undef,"Skipped");
#  print "'".$sl."'";
  $aa = $db->GetStatusArray();
  $sl = $db->getStatusList();
#  print "'".$sl."'";
  return 0;
}

sub testStartStatus{
   $db->startStatus('checkVolumes');
  my $sl = $db->getStatusList();
#  print $sl;
  return 0;
}

sub testEndStatusError{
   $db->startStatus('checkVolumes');
   $db->endStatusError('checkVolumes');
  my $sl = $db->getStatusList();
  return 0;
}

sub testEndStatusSuccess{
   $db->startStatus('checkVolumes');
   $db->endStatusSuccess('checkVolumes');
  my $sl = $db->getStatusList();
  return 0;
}
sub testGetStatusArray{
  my $aa = $db->GetStatusArray();
#  print Dumper($aa);
  return 0;
}

sub testDefaultDBMUser{
  my $s = $db->defaultDBMUser();
#  print "User: $s\n";
  return 0;
}

sub testValidateDBName(){
  my $s = $db->validateDBName('J2EE');
#  print "validation result: $s\n";
  return 0;
}


sub allTests{
  unitTest(__PACKAGE__);
}
1;

# Local Variables: ***
# eval: (setq my-sdbrun (format "%s\\sys\\src\\install\\perl\\sdbrun" "c:\\SapDevelop\\aw")) ***
# eval: (setq my-script (format "%s\\sys\\wrk\\script" "c:\\SapDevelop\\aw")) ***
# eval: (setq my-src (format "%s\\sys\\src\\install\\perl" (sdb-get-current-own))) ***
# eval: (setq perl-dbg-flags (format "-w -I%s -I%s\\readkey -I%s -I%s" my-sdbrun my-sdbrun my-script my-src )) ***
# End: ***
