package SAPDB::Install::Templates2::test::InstanceDesktopTest;

use SAPDB::Install::Templates2::Debug;
use SAPDB::Install::Templates2::InstTemplate;
use SAPDB::Install::Templates2::Util;
use strict;

our @ISA;
if($SCRIPTING){
  require Exporter;
  push @ISA,qw(Exporter);
  require Data::Dumper;
  import Data::Dumper;
}
else{
  require SDB::Common::Exporter;
  push @ISA,qw(SDB::Common::Exporter);
}


require Exporter;
push @ISA,qw(Exporter);
our @EXPORT = qw ( allTests );





my ($db,$tmpl);
sub setUp{
  $tmpl = SAPDB::Install::Templates2::InstTemplate::new(undef,'desktop') or die;
  $db = $tmpl->getDBInstance();
}


sub tearDown{
  undef $db;
}



sub testUsers{
  my $p;
  $p = $db->getDBMUser();
  $p eq 'DBM' or return "Wrong dbm user: $p";
  $p= $db->getDBMPass();
  $p eq 'SECRET' or return "Wrong dbm password: $p";

  $p = $db->getDBAUser();
  $p  eq 'DBADMIN' or return "Wrong dbauser user $p";
  $p = $db->getDBAPass();
  $p  eq 'SECRET' or return "Wrong dbapass $p";
  
  $p = $db->getDBAdmin();
  $p eq 'DBADMIN,SECRET' or return "Wrong dbadmin: $p";
  $p = $db->getDBManager();
  $p eq 'DBM,SECRET' or return "Wrong dbmanager: $p";
#  print "p: $p\n";
  return 0;
}



sub testGetDBUser{
  my $val = $db->getDBUser();
  exists $val->{'Role'} or return "No role";
  my $role = $val->{'Role'};
  $role eq 'globaluser' or return "Wrong role,expected 'globaluser', got '$role'";
  exists $val->{'Name'} or return "No name";
  exists $val->{'Description'} or return "No Description";
  return 0;
}

sub testSetDBUser{
  my $val = {
             'Name' => 'JUSTDBA',
             'Password' => 'MYPASS',
             'ConfirmPassword' => 'MYCONFIRM'
            };
  $db->setDBUser($val);
  $val = $db->getDBUser();
  my $dba = $db->getDBAUser();
  $dba eq 'JUSTDBA' or return "Wrong dba name: $dba";
  return 0;
}
sub testGetAutoRestart{
  my $is = $db->getAutoRestart();
  $is or return "Wrong value for auto restart!";
  return 0;
}


sub testDBUserValidator{
  my $param =   $db->createParameterDescriptionById('dBUser');
  my $val = $param->{'Validator'};
  $val or return "No validator";
  my $getter = $param->{'Getter'};
  my $cv = &$getter();
  $cv or return "No value from getter";
   my ($res,$msg1,$msg2,$s)=&$val($cv);
#  $res and return "Wrong validation result: $res: $msg1,$msg2,$s";
#  print Dumper(\@valres);
  return 0;
}






sub testDBAdmin{
  my $s = $db->getDBAdmin();
  $s = $db->GetLineForKey('dBAdmin');
  $s = $db->GetConfigurationLinesForKey('dBAdmin');
  $s = $db->GetConfigurationLines();
#  print "$s\n";
  return 0;
}


sub  testGetVolStructSimple{
  my $vols = $db->getVolStructSimple();
  @$vols eq 2 or return "Wrong number of volumes";
  return 0;
}

sub  testSetVolStructSimple{
  my $vols = $db->getVolStructSimple();
  $vols->[0]->{'Location'}="c:\\schrott\\MAXDB1";
  $vols->[1]->{'Location'}="c:\\schrot2\\MAXDB1";
  $db->setVolStructSimple($vols);
  my $ss = $db->getDBVolume();
  #print Dumper($ss);
  $ss = $db->Get('dataPath','no can','no replace');
  #print Dumper($ss);
  return 0;
}



sub testVolumePathConvention{
  my $vols = $db->getVolStructSimple();
  my $vol1 = $vols->[0];
  my $res = $vol1->{'DBInstance'}->volumePathConvention($vol1,'C:');
#  print Dumper($res);
  $res = $vol1->{'DBInstance'}->volumePathConvention($vol1,'F');
#  print Dumper($res);
  $res = $vol1->{'DBInstance'}->volumePathConvention($vol1,'C');
#  print Dumper($res);
  return 0;
}



sub testSetIndepProg{
  $db->setIndepProg("sdfSDFSDF12");
  return 0;
}



sub testSetSWDrive{
  $^O =~ /mswin/i or return 0;
  $db->getInstTemplate()->setSWDrive("F");
  my $drv = $db->getInstTemplate()->getSWDrive();
  $drv eq 'F' or return "Wrong drive: $drv";
  unless($tmpl->getIsIndepProgRegistered()){
    my $indepProg = $tmpl->getIndepProg();
    $indepProg =~ /^F:\\sdb\\programs/ or return "Wrong indep prog: $indepProg";
  }
  unless($tmpl->getIsIndepDataRegistered()){
    my $indepData = $tmpl->getIndepData();
    $indepData =~ /^F:\\sdb\\data/ or return "Wrong indep data: $indepData";
  }
  my $depPath = $db->getDependendPath();
  my $indepProg = $db->getInstTemplate()->getIndepProg();
  my $indepData = $db->getInstTemplate()->getIndepData();
  $depPath =~ /^F/ or return "Wrong dependend data path: $depPath";
#  print "depPath: ",$depPath , "\n";
#  print "indepProg: ",$indepProg , "\n";
#  print "indepData: ",$indepData , "\n";
  $db->getInstTemplate()->setSWDrive("C");
  $depPath = $db->getDependendPath();
  $indepProg = $db->getInstTemplate()->getIndepProg();
  $indepData = $db->getInstTemplate()->getIndepData();
#  print "depPath: ",$depPath , "\n";
#  print "indepProg: ",$indepProg , "\n";
#  print "indepData: ",$indepData , "\n";
  return 0;
}

sub testValidateSWDrive{
  my $val = $db->getInstTemplate()->validateSWDrive('F');
  return 0;
}


sub testSetDataDrive{
  $db->setDataDrive('F');
  my $ss = $db->getDBVolume();
  $ss = $db->getVolStructSimple();
  return 0;
}


sub testSWDrive{
  my $d =   $db->GetDefault('sWDrive');
  my $l =   $db->GetLineForKey('sWDrive',$d);
#  print  "$d '$l'\n";
  return 0;
}

sub allTests{
  unitTest(__PACKAGE__);
}

1;


# Local Variables: ***
# eval: (setq my-sdbrun (format "%s\\sys\\src\\install\\perl\\sdbrun" "c:\\SapDevelop\\aw")) ***
# eval: (setq my-script (format "%s\\sys\\wrk\\script" "c:\\SapDevelop\\aw")) ***
# eval: (setq my-src (format "%s\\sys\\src\\install\\perl" (sdb-get-current-own))) ***
# eval: (setq perl-dbg-flags (format "-w -I%s -I%s\\readkey -I%s -I%s" my-sdbrun my-sdbrun my-script my-src )) ***
# End: ***
