package SAPDB::Install::Window::Browser;

use Wx::Event qw(EVT_BUTTON EVT_KEY_DOWN EVT_CLOSE); 
use Wx qw( wxADJUST_MINSIZE wxALL wxTOP
		   wxALIGN_CENTER wxWANTS_CHARS wxALIGN_LEFT
		   wxWHITE wxEXPAND wxBOLD wxLEFT wxRIGHT wxBOTTOM
		   wxVERTICAL wxHORIZONTAL wxNORMAL wxALIGN_RIGHT) ;

use SAPDB::Install::Control::Link;
use SAPDB::Install::Control::HyperLink;
use SAPDB::Install::Resources;

our @ISA = qw(Wx::Panel);

sub new{
	my($this ) = shift->SUPER::new(@_);
	$this->SetWindowStyleFlag(wxWANTS_CHARS);
	$this->SetBackgroundColour(wxWHITE);
	$this->{'desc'} = SAPDB::Install::Control::StaticText->new($this,-1,[0,0],[300,70]);
	$this->{'desc'}->{'font'} = Wx::Font->new(10,wxNORMAL,wxNORMAL, wxBOLD, 0 );	
	$this->{'desc'}->{'text_colour'} = Wx::Colour->new(100,100,100);
	EVT_KEY_DOWN($this,\&OnKeyDown);
	EVT_CLOSE($this,\&OnClose);
	return $this;
}

sub initDesk{
	my ($self,$link_defs) = @_;
	my $rc = SAPDB::Install::Resources->new('RESOURCES.TGZ');
	my @link_defs = @$link_defs;
	my @needed_imgs;
	foreach my $def (@link_defs){
		push @needed_imgs,$def->[1];
	} 
	my $imgs = $rc->getImages(@needed_imgs);
	#$self->{'link_sizer'} = Wx::BoxSizer->new(wxVERTICAL);
	$self->{'link_sizer'} = Wx::GridSizer->new(2,2);
	my $top_sizer = Wx::BoxSizer->new(wxVERTICAL);
	my $space_sizer = Wx::BoxSizer->new(wxHORIZONTAL);
	my @links;
	my ($xmax,$ymax) = (0,0);
	foreach my $linkdef (@link_defs){
		my $link = SAPDB::Install::Control::Link->new($self,-1);
		$link->setImg($imgs->{$linkdef->[1]});
		$link->setText($linkdef->[0]);
		$link->{'description'} = $linkdef->[2];
		my ($x,$y) = $self->{'desc'}->getTextExtend($link->{'description'});
		$xmax = $x if($x > $xmax);
		$ymax = $y if($y > $ymax);
		$link->{'cmd_func'} = $linkdef->[3];
		$link->{'link'} = $linkdef->[4];
		push @links,$link->GetId;
		#$link->SetSize([80,20]);
		$self->{'link_sizer'}->Add($link,1, wxEXPAND | wxALIGN_CENTER | wxADJUST_MINSIZE | wxALL,10);
		EVT_BUTTON($self,$link->GetId,\&OnLinkSelect);
	}
	$self->{'links'} = \@links;
	$self->{'desc'}->SetSize([$xmax,$ymax]);
	$space_sizer->Add($self->{'desc'},1,wxALIGN_CENTER | wxLEFT,30);
	$space_sizer->Add($self->{'link_sizer'},0,wxEXPAND | (wxALL) & (~wxBOTTOM), 50);
	$self->SetAutoLayout(1);
	$top_sizer->Add($space_sizer,1,wxEXPAND);
	my $exit_link = SAPDB::Install::Control::HyperLink->new($self,-1,[0,0],[40,15]);
	$exit_link->SetLabel('Exit');
	$exit_link->SetFunc(sub{Wx::GetTopLevelParent($self)->Close(1)});
	$exit_link->{'description'} = "Exit Installation\nManager";
	push @links,$exit_link->GetId;
	EVT_BUTTON($self,$exit_link->GetId,\&OnLinkSelect);
	$top_sizer->Add($exit_link,0,wxALIGN_RIGHT | (wxALL) & (~wxTOP),30);
	$self->SetSizer($top_sizer);
	$self->Fit;
	$self->Layout;
	$self->SetFocus();
}

sub OnLinkSelect{
	my ($self,$event) = @_;
	return if defined $self->{'selected'} and
			 $self->{'selected'} == $event->GetId;
	$self->FindWindow($self->{'selected'})->unselect
			if defined $self->{'selected'};
	$self->{'selected'} = $event->GetId;
	$self->{'desc'}->setText($self->FindWindow($self->{'selected'})->{'description'});
}


sub OnKeyDown{
	my ($self,$event) = @_;
	if(	$event->GetKeyCode == 9 || 
		$event->GetKeyCode == 318 ||
		$event->GetKeyCode == 316
		){
		my $next;
		my $index = 0; 
		foreach my $id (@{$self->{'links'}}){
			if($self->{'selected'} == $id){
				$next = $index; 
			}
			$index++;
		}

		my @links = @{$self->{'links'}};
		if(($event->GetKeyCode == 9 && $event->ShiftDown) || $event->GetKeyCode == 316){
			$next--;
			$next = $#links if $next < 0;
		}
		else{
			$next++;
			$next = 0 if $next > $#links;
		}
		$self->FindWindow($self->{'links'}->[$next])->select;
	}
	elsif($event->GetKeyCode == 13){
		$self->FindWindow($self->{'selected'})->Call;
	}
	elsif($event->GetKeyCode == 65){
		require SAPDB::Install::Dialog::About;
		SAPDB::Install::Dialog::About->new($self,-1);
	}
	elsif($event->GetKeyCode == 82){
		require SAPDB::Install::App::RegistryViewer;
		SAPDB::Install::App::RegistryViewer::main();
	}

}

sub OnClose{
	my ($self,$evt) = @_;
	print ">>>>> closing $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


sub Destroy{
	my ($self) = @_;
	
	$self->GetSizer->Remove($self->{'desc'});
	$self->{'desc'}->Destroy;
	delete $self->{'desc'};
	foreach my $link_id (@{$self->{'links'}}){
		$self->FindWindow($link_id)->Destroy;
	}
	while($self->GetSizer->Remove(0)){
	
	}
	delete $self->{'link_sizer'}; 
	$self->SUPER::Destroy;

}


sub DESTROY{
	my ($self) = @_;
	print ">>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}



1;