#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Wizard/Dialog/DBHandling.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc: start SAP DB installation/update/patch/instanceupgrade,
#       called by SDBINST/SDBUPD
#

package SAPDB::Install::Wizard::Dialog::DBHandling;

use Wx::Event qw (EVT_RADIOBUTTON);
use Wx qw (
	wxICON_ERROR wxCENTRE
	wxVERTICAL wxALIGN_CENTER wxBOTTOM wxEXPAND
	wxALL wxLEFT wxTOP wxRIGHT);

use base SAPDB::Install::Wizard::Dialog::Base;
use SAPDB::Install::Wizard::Dialog::Template;

my $types = {
	'create' =>	{
				'desc' =>  "Installs the software and creates a database instance. ",
				'label' => 'Install software and &create database instance',
				'seq' => 0
			},
	'update' => {
				'desc' =>   "Installs the software and upgrades an existing database instance.",
				'label' => 'Install software and &upgrade existing database instance',
				'seq' => 1

			},
	'software' => {
				'desc' =>   "Installs or updates the database software.\nUse this option if you want to create a new database instance later.",
				'label' => 'Install/update &software only',
				'seq' => 2

		}

};


my $default_type = 'create';


sub new{
	my($this ) = shift->SUPER::new(@_);
	my $topsizer = Wx::BoxSizer->new(wxVERTICAL);
	my @types =  sort {$types->{$a}->{'seq'} <=> $types->{$b}->{'seq'}} keys(%$types);
	my $cb_sizer = Wx::BoxSizer->new(wxVERTICAL);
	$this->{'sb'} = Wx::StaticBox->new($this,-1,'Installation Type Description');
	my $sb_sizer = Wx::StaticBoxSizer->new($this->{'sb'},wxVERTICAL);
	if($^O =~ /mswin/i){
		$this->{'text_desc'} = Wx::StaticText->new($this,-1,'');
	}
	else{
			$this->{'text_desc'} = SAPDB::Install::Control::StaticText->new($this,-1,[0,0],[150,50]);
			$this->{'text_desc'}->{'auto_split'} = 1;
			$this->{'text_desc'}->{'auto_resize'} = 0;
	}
	
	foreach my $type (@types){
		$this->{'rb_'.$type} = Wx::RadioButton->new($this,-1, $types->{$type}->{'label'});
		$this->{'id_offset'} = $this->{'rb_'.$type}->GetId unless(defined $this->{'id_offset'});
		if($type eq $default_type){
			$this->{'rb_'.$type}->SetValue(1);
			$this->{'text_desc'}->SetLabel($types->{$type}->{'desc'});
		}

		EVT_RADIOBUTTON($this,$this->{'rb_'.$type}->GetId,\&OnRBClick);	
		$cb_sizer->Add($this->{'rb_'.$type},1,wxEXPAND | wxLEFT,50);
	}
	
	#$sb_sizer->SetMinSize([250,150]);
	
	$sb_sizer->Add($this->{'text_desc'},1,wxEXPAND | wxALL,30);
	$sb_sizer->SetItemMinSize($this->{'text_desc'},150,50);
	my $sizer = Wx::BoxSizer->new(wxVERTICAL);
	$sizer->Add($sb_sizer,1,wxEXPAND | wxTOP | wxBOTTOM,20);
	$this->{'sb_sizer'} = $sb_sizer;	
	$topsizer->Add($cb_sizer,1,wxEXPAND | wxLEFT | wxTOP | wxALIGN_CENTER,30);
	$topsizer->Add($sizer,1,wxEXPAND | wxLEFT | wxRIGHT |  wxALIGN_CENTER ,80);
	$this->SetSizer($topsizer);
	$this->SetAutoLayout(1);
	$this->Layout;
	return $this;
}

sub init(){
	my ($self,$config) = @_;
	$config->{'collector'}->GenInstanceIndex($config->{'container'}->InstReg);
	my @dbs;
	if (defined $config->{'collector'}->instance_index ){
		foreach my $db (keys(%{$config->{'collector'}->instance_index})){
			#if($config->{'collector'}->instance_index->{$db}->{'ready_to_update'}){
				push @dbs,$db;
			#}
		}
	}
	
	if($#dbs == -1){
		$self->{'rb_update'}->Enable(0);
	}
	
	$self->{'initialized'} = 1;
	return {}; 
}

sub Process{
	my ($self,$config) = @_;
	my $selection;
	foreach my $type (keys(%$types)){
		if($self->{'rb_'.$type}->GetValue){
			$selection = $type;
			last;
		}
	}
	
	if($selection eq 'create'){
		my $step = {
			'name' => 'Select database configuration',
			'dlg' => SAPDB::Install::Wizard::Dialog::Template->new($self->GetWizard,-1)
		};

		return {'AddNewSteps' => [
									[$step,undef]
								 ]};		
	}
	elsif($selection eq 'update'){
		require SAPDB::Install::Wizard::Dialog::UpdateSelect;
		my $step = {
			'name' => 'Select a database',
			'dlg' => SAPDB::Install::Wizard::Dialog::UpdateSelect->new($self->GetWizard,-1)
		};

		return {'AddNewSteps' => [
									[$step,undef]
								 ]};			
	}
	else{
		return {};
	}
}

sub OnRBClick{
	my ($self,$ev) = @_;
	my $selection = $self->FindWindow($ev->GetId)->GetLabel;
	$selection =~ s/&//g;
	foreach my $type (keys(%$types)){
		my $label = $types->{$type}->{'label'};
		$label =~ s/&//g;
		if($label eq $selection){
			$self->{'text_desc'}->SetLabel($types->{$type}->{'desc'});
			$self->GetWizard->LayoutDlg;
			last;
		}
	}
}

sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	foreach my $type (keys(%$types)){
		if(defined $self->{'rb_'.$type}){
			$self->{'rb_'.$type}->Destroy;
			delete $self->{'rb_'.$type};
		}
	}
	if(defined $self->{'sb'}){
		$self->{'sb'}->Destroy;
		delete $self->{'sb'};
	}
	if(defined $self->{'text_desc'}){
		$self->{'text_desc'}->Destroy;
		delete $self->{'text_desc'};
	}
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}



1;
