#!/usr/bin/perl
#
# $Header$
# $DateTime$
# $Change$
#
# Desc:
#

package SAPDB::Install::Wizard::Dialog::Finalize;

use Wx qw (
	wxNORMAL wxBOLD wxTE_MULTILINE
	wxTE_READONLY wxEXPAND wxVERTICAL wxTOP wxALL
	wxBOTTOM wxALIGN_CENTER);

use Wx::Event qw(EVT_BUTTON);

use SAPDB::Install::Control::StaticText;
use SAPDB::Install::Registry;
use SAPDB::Install::Tools;
use SAPDB::Install::Window::LogFileViewer;
use base SAPDB::Install::Wizard::Dialog::Base;

sub new{
	my ($this ) = shift->SUPER::new(@_);
	$this->{'caption'} = SAPDB::Install::Control::StaticText->new($this,-1);
	$this->{'text'} = SAPDB::Install::Control::StaticText->new($this,-1);
	$this->{'caption'}->{'font'} = Wx::Font->new(14,wxNORMAL, wxNORMAL, wxBOLD, 0 );
	$this->{'text'}->{'font'} = Wx::Font->new(10,wxNORMAL, wxNORMAL, wxBOLD, 0 ); 
	$this->{'logwin'} = SAPDB::Install::Window::LogFileViewer->new($this,-1);
	$this->{'logwin'}->Show(0);
	$this->{'log_is_shown'} = 0;
	$this->{'show_self_defined'} = 1;
	return $this;
}
sub init{
	my ($self,$config) = @_;
	if($^O =~ /mswin/i and $config->{'error_state'} != 1  and defined $config->{'container'}->Template){
		$self->{'tool_buttons'} = 1;
	}
		
	
	
	
	$self->{'config'} = $config;
	$self->{'initialized'} = 1;
	unless($^O =~ /mswin/i){
		$self->{'caption'}->SetSize(300,30);
		$self->{'text'}->SetSize(300,20);
	}
	if($config->{'error_state'} == 1){
		$self->{'caption'}->Show(1);
		$self->{'caption'}->setText("Sorry, an ERROR has occurred!");
		$self->{'text'}->setText("Please look into logfile for detailed information!");
	}
	else{
		$self->{'caption'}->Show(0);
		if($self->{'uninst_mode'}){
				$self->{'text'}->setText("Your MaxDB uninstallation was finished successfully!");
		}
		elsif(defined $config->{'dbupdate'}){
			$self->{'text'}->setText("Upgrade of database ".
				$config->{'dbupdate'}->{'dbname'}." was finished successfully!");
		}	
		else{
			$self->{'text'}->setText("Your MaxDB installation was finished successfully!");
		}
	}
	my $fsteps_index; 
	
	if(	$config->{'error_state'} != 1 and
		$^O !~ /mswin/i and
		defined $config->{'container'}->Template){
		my $depend = $config->{'container'}->Template->getDBInstance()->getDependendPath;
		if(-f "$depend/doc/FirstSteps/index.html"){
			$fsteps_index = "$depend/doc/FirstSteps/index.html";
			$fsteps_index =~ s/\//\\/g if $^O =~ /mswin/i; 
			$self->{'firststeps_button'} = 1;
		}		
	}
	
	my $top_sizer = Wx::BoxSizer->new(wxVERTICAL);
	$top_sizer->Add($self->{'caption'},0,wxALIGN_CENTER | wxTOP,80);
	$top_sizer->Add($self->{'text'},0,wxALIGN_CENTER | wxTOP,20);
	if($SAPDB::Install::Config{'SysPathChanged'}){
		$self->{'reboot_msg'} = Wx::StaticText->new($self,-1,
			'Rebooting ensures that the MaxDB software can be started from any command line window.',[0,0],[-1,-1]);
		$top_sizer->Add($self->{'reboot_msg'},0, wxALIGN_CENTER | wxTOP,20);
	}
	
	my $max_button_width = -1;	
	my ($w,$h);
	if($self->{'tool_buttons'}){
		$self->{'dbmgui_button'} = Wx::Button->new($self,-1,'Start &Database Manager ...',[0,0],[-1,-1]);
		($w,$h) = $self->{'dbmgui_button'}->GetSizeWH;
		$max_button_width = $w if $w > $max_button_width;  
		$self->{'sqlstudio_button'} = Wx::Button->new($self,-1,'Start &SQL Studio ...',[0,0],[-1,-1]);
		($w,$h) = $self->{'sqlstudio_button'}->GetSizeWH;
		$max_button_width = $w if $w > $max_button_width;
		my $dbmgui = SAPDB::Install::Registry::GetDBMGui();
		my $sqlstudio = SAPDB::Install::Registry::GetSQLStudio();
		my $home_dir = $ENV{'HOMEDRIVE'}.$ENV{'HOMEPATH'};
		my $cwd;
		if($dbmgui =~ /\S/ and -f $dbmgui){
			my $dbname = $config->{'container'}->Template->getDBInstance()->getDBName();
			my $user_password;
			$user_password = $config->{'container'}->Template->getDBInstance()->getDBMUser();
			$user_password .= ','.$config->{'container'}->Template->getDBInstance()->getDBMPass();
			my $options = "-d $dbname -u $user_password";
			EVT_BUTTON($self,$self->{'dbmgui_button'}->GetId,
				sub{
					if(-d $home_dir){
						require SAPDB::Install::Cwd;
						$cwd = SAPDB::Install::Cwd::getcwd();	
						chdir($home_dir);
					}
					system('rundll32.exe shell32.dll,ShellExec_RunDLLA "'.$dbmgui.'" '.$options);			
					if($cwd =~ /\S/){
						chdir($cwd);
					}
				}
			);
		}
		else{
			$self->{'dbmgui_button'}->Enable(0);
		}
		if($sqlstudio =~ /\S/ and -f $sqlstudio){
			my $dbname = $config->{'container'}->Template->getDBInstance()->getDBName();
			my $user_password;
			if($config->{'container'}->Template->getDBInstance()->getLoadSampleSchema){
				$user_password = 'mona,red';
			}
			else{
				$user_password = $config->{'container'}->Template->getDBInstance()->getDBAUser();
				$user_password .= ','.$config->{'container'}->Template->getDBInstance()->getDBAPass();
			
			}
			 
			my $options = "-d $dbname -u $user_password";
			
			EVT_BUTTON($self,$self->{'sqlstudio_button'}->GetId,
					sub{
						if(-d $home_dir){
							require SAPDB::Install::Cwd;
							$cwd = SAPDB::Install::Cwd::getcwd();	
							chdir($home_dir);
						}
						system('rundll32.exe shell32.dll,ShellExec_RunDLLA "'.$sqlstudio.'" '.$options);
						if($cwd =~ /\S/){
							chdir($cwd);
						}
					}
			);
		}
		else{
			$self->{'sqlstudio_button'}->Enable(0);
		}
	}
	if($self->{'firststeps_button'}){
		$self->{'fsteps_button'} = Wx::Button->new($self,-1,'Launch &First Steps ...',[0,0],[-1,-1]);
		($w,$h) = $self->{'fsteps_button'}->GetSizeWH;
		$max_button_width = $w if $w > $max_button_width;
		EVT_BUTTON($self,$self->{'fsteps_button'}->GetId,
			sub{
				if($^O =~ /mswin/i){
					system("rundll32 url.dll,FileProtocolHandler ".$fsteps_index);
				}
				else{
					my $browser = getBrowser();
					unless($browser =~ /\S/){
						$self->SetError('Browser missing','Cannot detect a html browser');
						return undef;
					}
					system("$browser $fsteps_index &");
				}
			}
		);
	}

	if($self->{'tool_buttons'}){
		$self->{'dbmgui_button'}->SetSize([$max_button_width,$h]);
		$self->{'sqlstudio_button'}->SetSize([$max_button_width,$h]);
		$top_sizer->Add($self->{'dbmgui_button'},0,wxALIGN_CENTER | wxTOP,30);
		$top_sizer->Add($self->{'sqlstudio_button'},0,wxALIGN_CENTER | wxTOP,20);
	}
	if($self->{'firststeps_button'}){
		$self->{'fsteps_button'}->SetSize([$max_button_width,$h]);
		$top_sizer->Add($self->{'fsteps_button'},0,wxALIGN_CENTER | wxTOP,20);	
	}

	
	$self->SetSizer($top_sizer);
	$self->SetAutoLayout(1);
	$self->Layout;
	$self->Show(1);
	&{$SAPDB::Install::Values::RefreshGui}();


	#
	# freeing InstallRegistry and Package objects here, but 
	# save log object to get all the destuct messages
	#
	
	my $log = $config->{'container'}->Log;
	foreach my $member (keys(%$config)){
		delete $config->{$member};
	}	
	my $level = 0;
	while(1){
		my @all_info_caller_package = (caller($level));
		$#all_info_caller_package == -1  and last;
		$level++;
		my $package = $all_info_caller_package[0];
		if(defined ${"${package}::cleaner"} and	ref(${"${package}::cleaner"}) eq 'SAPDB::Install::Cleaner'){ 
			${"${package}::cleaner"}->DESTROY;
			last;
		}	
	}
	
	# 
	#	set messages into text control
	#
	
	my $logmsgs = $log->Messages;
	$self->{'logwin'}->SetValue(join('',@$logmsgs));
	
	#
	#	delete last log reference
	#   -> writes log file
	#
	
	$log = undef;
	return {'auto_next' => 1};
}


sub ShowLog{
	my ($self,$true) = @_;
	if(	($self->{'log_is_shown'} and $true != 0) or 
		(!$self->{'log_is_shown'} and $true == 0)){
	
		return undef;
	}
	
	if($true == 0){
		$self->{'logwin'}->Show(0);
		$self->GetSizer->Remove($self->{'logwin'});
		$self->GetSizer->Add($self->{'caption'},0,wxALIGN_CENTER | wxTOP,80);
		$self->GetSizer->Add($self->{'text'},0,wxALIGN_CENTER | wxTOP,30);
		if($self->{'reboot_msg'}){
			$self->{'reboot_msg'}->Show(1);
			$self->GetSizer->Add($self->{'reboot_msg'},0, wxALIGN_CENTER | wxTOP,20);
		}
		if($self->{'tool_buttons'}){
			$self->GetSizer->Add($self->{'dbmgui_button'},0,wxALIGN_CENTER | wxTOP,30);
			$self->GetSizer->Add($self->{'sqlstudio_button'},0,wxALIGN_CENTER | wxTOP,20);
		}
		if($self->{'firststeps_button'}){
			$self->GetSizer->Add($self->{'fsteps_button'},0,wxALIGN_CENTER | wxTOP,20);
		}
		$self->{'caption'}->Show(1);
		$self->{'text'}->Show(1);
		$self->Layout();
		$self->{'log_is_shown'} = 0;
	}
	else{
		$self->{'caption'}->Show(0);
		$self->{'text'}->Show(0);
		$self->GetSizer->Remove($self->{'caption'});
		$self->GetSizer->Remove($self->{'text'});
		if($self->{'reboot_msg'}){
			$self->{'reboot_msg'}->Show(0);
			$self->GetSizer->Remove($self->{'reboot_msg'});
		}
		if($self->{'tool_buttons'}){
			$self->GetSizer->Remove($self->{'dbmgui_button'});
			$self->GetSizer->Remove($self->{'sqlstudio_button'});
		}
		if($self->{'firststeps_button'}){
			$self->GetSizer->Remove($self->{'fsteps_button'});
		}
		$self->GetSizer->Add($self->{'logwin'},1,wxEXPAND | wxALL,10);
		$self->{'logwin'}->Show(1);
		$self->Layout();
		$self->{'log_is_shown'} = 1;
	
	}
}


sub OnViewLog{
	my ($wizard,$evt) = @_;
	my $self = $wizard->{'current_step'}->{'dlg'};
	$self->ShowLog(1);
	$wizard->LogButton('C&lose Log',\&OnCloseLog);
	return 1;
}

sub OnCloseLog{
	my ($wizard,$evt) = @_;
	my $self = $wizard->{'current_step'}->{'dlg'};
	$self->ShowLog(0);
	$wizard->LogButton('&View Log',\&OnViewLog);
	return 1;
}

sub Process{
	my ($self) = @_;
	$self->GetWizard->LayoutDlg();
	$self->GetWizard()->KillPrevSteps();
	$self->GetWizard->LogButton('&View Log',\&OnViewLog);
	$self->GetWizard->{'cbutton'}->SetFocus;
	return 1;
}


sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	if($self->{'tool_buttons'}){
		$self->{'dbmgui_button'}->Destroy;
		delete $self->{'dbmgui_button'};
		$self->{'sqlstudio_button'}->Destroy;
		delete $self->{'sqlstudio_button'};
	}
	if($self->{'firststeps_button'}){
		$self->{'fsteps_button'}->Destroy;
		delete $self->{'fsteps_button'};
	}
	if($self->{'reboot_msg'}){
		$self->{'reboot_msg'}->Destroy;
		delete $self->{'reboot_msg'};
	}
	$self->{'logwin'}->Destroy;
	delete $self->{'logwin'};
	$self->{'text'}->Destroy;
	delete $self->{'text'};
	$self->{'caption'}->Destroy;
	delete $self->{'caption'};
	$self->SUPER::Destroy;
}



sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
