#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Wizard/Dialog/Prepare.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc: start SAP DB installation/update/patch/instanceupgrade,
# called by SDBINST/SDBUPD
#

package SAPDB::Install::Wizard::Dialog::Prepare;

use Wx qw (
	wxVERTICAL wxEXPAND wxALIGN_CENTER
	wxTOP wxLEFT wxICON_ERROR wxCENTRE
	wxYES_NO wxID_YES wxICON_QUESTION);

use base SAPDB::Install::Wizard::Dialog::Base;
use SAPDB::Install::Control::StaticText;
use SAPDB::Install::Window::Progress;
use SAPDB::Install::Window::Parameter;
use SAPDB::Install::FSUsage;
use SAPDB::Install::Prepare;
use SAPDB::Install::Registry;
use SAPDB::Install::System;

sub new{
	my($this ) = shift->SUPER::new(@_);
	$this->{'progresswin'} = SAPDB::Install::Window::Progress->new($this,-1);
	#$this->{'caption'} = SAPDB::Install::Control::StaticText->new($this,-1,[0,0],[500,30]);
	$this->{'caption'} = Wx::StaticText->new($this,-1,'Preparing MaxDB package ...',[0,0],[500,15]);
	#$this->{'caption'}->setText("Preparing MaxDB Package ...");
	my $top_sizer = Wx::BoxSizer->new(wxVERTICAL);
	$top_sizer->Add($this->{'caption'},0,wxTOP | wxLEFT ,60);
	$this->{'progresswin'}->SetSize([100,80]);
	$top_sizer->Add($this->{'progresswin'},0,wxEXPAND | wxALIGN_CENTER | wxTOP | wxLEFT, 60);
	$this->{'progress_enabled'} = 1;
	$this->SetSizer($top_sizer);
	$this->SetAutoLayout(1);
	$this->Layout;
	return $this;
}



sub init{
	my ($self,$config) = @_;
	if(defined $config->{'container'}->InstReg){
		$config->{'container'}->InstReg->RefreshAllFilesList;
	}
	$self->{'hFileSystemInfo'} = {};
	$self->{'current_package'} = undef;
	$self->{'update_selection_pending'} = undef;
	$self->{'param_selection_pending'} = undef;
	$self->enableProgressWin(1);
	$self->GetWizard()->KillPrevSteps();
	return {'auto_next' => 1};
}


sub enableProgressWin{
	my ($self,$enable) = @_;
	if($enable == 0){
		return unless($self->{'progress_enabled'});
		$self->{'progresswin'}->Show(0);
		$self->GetSizer->Remove($self->{'progresswin'});
		$self->{'progress_enabled'} = 0;
	}
	else{
		return if($self->{'progress_enabled'});
		if(defined $self->{'update_rbox'}){
			$self->GetSizer->Remove($self->{'update_rbox'});
			$self->{'update_rbox'}->Destroy;
			$self->{'update_rbox'} = undef;
		}
		if(defined $self->{'paramwin'}){
			$self->GetSizer->Remove($self->{'paramwin'});
			$self->{'paramwin'}->Destroy;
			$self->{'paramwin'} = undef;			
		}
		$self->GetSizer->Add($self->{'progresswin'},0,wxEXPAND | wxALIGN_CENTER | wxTOP | wxLEFT, 60);
		$self->Layout;
		$self->{'progresswin'}->Show(1);
		$self->{'progress_enabled'} = 1;
	}
}

sub Process{
	my ($self,$config) = @_;

	if($self->{'retry'}){
		$self->GetWizard->SetRetryMode(0);
	}

	my $eval_rc = eval{
	my $index = 0;
	foreach my $package (@{$config->{'packages'}}){
		++$index and next if defined $self->{'current_package'} and $index < $self->{'current_package'}; 
		$self->{'current_package'} = $index++;
		next if (${($package->Skip)});

		#$self->{'caption'}->setText("Preparing MaxDB Package ".$package->DispName);
		$self->{'caption'}->SetLabel("Preparing MaxDB Package ".$package->DispName);
		
		if($package->Update){
			unless(defined $package->RegData){
				$package->RegData($config->{'container'}->InstReg->getPackage($package->Name,$package->MainPath->{'value'}));
			}
			$config->{'container'}->SetRefInstReg(\$package);
			$config->{'container'}->SetRefInstReg(\($package->RegData));
		}
		else{
			$package->RegData($config->{'container'}->InstReg->newPackage($package->Name,$package->MainPath->{'value'}));
			$config->{'container'}->SetRefInstReg(\$package);
			$config->{'container'}->SetRefInstReg(\($package->RegData));
		}
		
		
		my ($rc,$details) = checkPackage($package,{'callback_progress' => sub {$self->{'progresswin'}->SetProgress(@_)},
								'callback_initprogress' => sub {$self->{'progresswin'}->init(@_)} });
		if($rc == 0){
			if($details->{'test_extract_failed'}){
				#
				# check running programs
 				#
				
				my $known_components = {};
				
				foreach my $process (@{$details->{'info'}->{'process_names'}}){
					if($^O =~ /mswin/i and $process =~ /serv.exe/ or 
						$^O !~ /mswin/i and $process =~ /vserver/ or
						$^O !~ /mswin/i and $process =~ /niserver/){
							$known_components->{'communication'} = 1;
					}
					elsif($process =~ /kernel/){
						my $arg = 'stop_all';
						if($package->Name ne 'Base'){
								$arg = $package->MainPath->{'value'};
						} 
						$known_components->{'database'} = $arg;
					}
				}
				
				my $all_known_services_stopped = 1;
				my $found_a_known_service = 0;
				my $found_zombie_service = 0;
				
				if(exists $known_components->{'database'}){
					$found_a_known_service = 1;				
					unless(exists $config->{'comm'} and defined $config->{'comm'}){
						require SAPDB::Install::Service::Communication;
						$config->{'comm'} = SAPDB::Install::Service::Communication->new();
					}
					my $comm_started = 0;
					unless($config->{'comm'}->state->{'is_running'}){
						$comm_started = 1;	
						unless($config->{'comm'}->start){
							$self->SetError('Cannot start communication server',$config->{'comm'}->last_error);
							return {'errstate' => 1};
						}					
						$comm_started = 1;
					}
					
					require SAPDB::Install::Instance::Base;
					require SAPDB::Install::DBMCmd;
					require SAPDB::Install::Instance::Check::Instances;
					my $icheck = SAPDB::Install::Instance::Base::new();
					bless($icheck,'SAPDB::Install::Instance::Check::Instances');
					$icheck->{'instancetype'} = 'Dummy'; # avoid call of conditions()
					$icheck->{'dbm'} = SAPDB::Install::DBMCmd->new('localhost');
					#$icheck->{'msg0'} = sub {print ">>>>> 0: $_[0]\n"};
					#$icheck->{'msg1'} = sub {print ">>>>> 1: $_[0]\n"};
					$icheck->instances();
					my @stop_databases;
					if($package->Name eq 'Base'){
						@stop_databases = @{$icheck->{'running_instances'}};
					}	
					else{
						my %instances = getInstances();
						foreach my $running_db (@{$icheck->{running_instances}}){
							my $path = $instances{$running_db};
							if($^O =~ /mswin/i){
								if(normalizePath($path) eq $package->MainPath->{'value'}){
									push @stop_databases,$running_db;
								}	
							}else{
								if($path eq $package->MainPath->{'value'}){
									push @stop_databases,$running_db;
								}
							}	
						}							
					}
					
					if($#stop_databases == -1){
					
						#
						# kernel is running, but cannot find a running database
						#
						
						$found_zombie_service = 1;		
						$all_known_services_stopped = 0;	
					}
					
					foreach my $db (@stop_databases){
						my $msgbox = Wx::MessageDialog->new($self,"To install the new software you have to stop the database $db.\n".
																"Stop MaxDB instance $db now?","MaxDB instance $db is running",
																wxYES_NO | wxICON_QUESTION);
						if($msgbox->ShowModal == wxID_YES){
							$msgbox->Destroy;
							require SAPDB::Install::Dialog::DBStop;
							my $dlg = SAPDB::Install::Dialog::DBStop->new({'dbname' => $db,
											'log' => $config->{'container'}->Log},$self,-1);
							my $rc = $dlg->ShowModal;
							$dlg->Destroy;
							unless($rc){
								$all_known_services_stopped = 0;
							}
						}
						else{
							$all_known_services_stopped = 0;
							$msgbox->Destroy;
						}
					}
					
					if($comm_started){
						unless($config->{'comm'}->stop){
							$self->SetError('Cannot stop communication server',$config->{'comm'}->last_error);
							return {'errstate' => 1};	
						}
					}
					
				}

				if(exists $known_components->{'communication'}){
					$found_a_known_service = 1;
					my $msgbox = Wx::MessageDialog->new($self,"To install the new software you have to stop the communication service.\n".
																"Stop MaxDB communication now?",'MaxDB communication service is running',
																wxYES_NO | wxICON_QUESTION);
					if($msgbox->ShowModal == wxID_YES){
						$msgbox->Destroy;
						unless(exists $config->{'comm'} and defined $config->{'comm'}){
							require SAPDB::Install::Service::Communication;
							$config->{'comm'} = SAPDB::Install::Service::Communication->new();
						}
						
						unless($config->{'comm'}->stop){
							$self->SetError('Cannot stop communication server',$config->{'comm'}->last_error);
							$msgbox->Destroy;
							return {'errstate' => 1};	
						}
						$config->{'comm_stopped_by_software_update'} = 1;
 					}
					else{
						$msgbox->Destroy;
						$all_known_services_stopped = 0;
					}
					
				}
				
				if(not $found_a_known_service or $found_zombie_service){
						$self->SetError("Error: Checking package ".$package->Name." failed!",
							$details->{'errmsg'}.($details->{'errdetails'} =~ /\S/ ?
							":\n".$details->{'errdetails'} : ''));
						$self->GetWizard->SetRetryMode(1);
						$self->{'retry'} = 1;
						$self->{'current_package'} == undef;
						$self->{'hFileSystemInfo'} = {};
						return undef;					
				}
				if($all_known_services_stopped){
					return {'auto_retry' => 1}
				}
				$self->GetWizard->SetRetryMode(1);
				$self->{'retry'} = 1;
				$self->{'current_package'} == undef;
				$self->{'hFileSystemInfo'} = {};
				return undef;			
			}
			
			$self->SetError("Error: Checking package ".$package->Name." failed!",
							$details->{'errmsg'}.($details->{'errdetails'} =~ /\S/ ?
							":\n".$details->{'errdetails'} : ''));
			
			return {'errstate' => 1};
		}
		
		foreach my $fsinfo (@{$details->{'fs_infos'}}){
			if(exists $self->{'hFileSystemInfo'}->{$fsinfo->{'FSNAME'}}){
				$self->{'hFileSystemInfo'}->{$fsinfo->{'FSNAME'}} = { 
						'AVAIL' => $fsinfo->{'AVAIL'},
						'NEEDED' => ($fsinfo->{'NEEDED'} + $self->{'hFileSystemInfo'}->{$fsinfo->{'FSNAME'}}->{'NEEDED'}),
						'USED' => ($fsinfo->{'USED'} + $self->{'hFileSystemInfo'}->{$fsinfo->{'FSNAME'}}->{'USED'})
					 };
			}
			else{
				$self->{'hFileSystemInfo'}->{$fsinfo->{'FSNAME'}} = { 
							'AVAIL' => $fsinfo->{'AVAIL'} ,
							'NEEDED' => $fsinfo->{'NEEDED'},
							'USED' => $fsinfo->{'USED'}
					};
			} 	
		}

		
		if($package->IsSubPackage){
			my $parent_id = $package->ParentPackage;
			my $pack;  
			if(exists $config->{'collector'}->package_index->{$parent_id->[0]}){
				for(my $i = 0; $i < $index;$i++){
					if($config->{'packages'}->[$i]->Name eq $parent_id->[0]){
						$pack = $config->{'packages'}->[$i]->RegData; 			
						last;
					}
				}				
			}
			elsif(exists $config->{'collector'}->regpackage_index->{$parent_id->[0]}->{$parent_id->[1]}){
				$pack = $config->{'container'}->InstReg->getPackage(@$parent_id);
			}
			
			unless(defined $pack){
				return undef;
			}
			my @subpackages = @{$pack->SubPackages};
			my $found = 0;
		
			foreach my $subpackage (@subpackages){
				if($package->Name eq ${@$subpackage}[0] and $package->MainPath->{'value'} eq ${@$subpackage}[1]){
					$found =1;
					last;
				}	
			}
			unless($found){
				push @subpackages,[$package->Name,$package->MainPath->{'value'}];
				$pack->SubPackages(\@subpackages);
				$pack->setValues(); # dump registry data to disk
			}
			unless(exists $config->{'collector'}->package_index->{$parent_id->[0]}){
					$SAPDB::Install::InstallRegistry::Package::SELFOBJ[$pack->ID] = undef; 			
			}
		}

		if(defined $package->postprepare){
			$config->{'container'}->Log->SetMsg("MSG: running postprepare of package ".$package->Name."\n");
			eval{
				&{$package->postprepare}();
			};
			
			die($@) if($@ =~ /^SDB_closing/);
			
			if($@){
				$self->SetError("Error in postprepare() of package ".$package->DispName,
								$@);
				return {'errstate' => 1};
			}
		}		
	}
	1;
	};
	die($@) if($@ =~ /^SDB_closing/);
	if($@ =~ /fatal\serror/){
		return {'errstate' => 1};	
	}
	elsif($@){
		$self->SetError("Runtime Error: ",$@);
		return {'errstate' => 1};
	}
	
	unless($eval_rc == 1){
		return $eval_rc;
	}

	if($self->{'diskspace_retry'}){
		foreach my $fs (keys(%{$self->{'hFileSystemInfo'}})){
			my ($total,$avail) = SAPDB::Install::FSUsage::GetFSSize(fs);
			if($avail < 
				$self->{'hFileSystemInfo'}->{$fs}->{'NEEDED'}){
				$self->SetError("Error: Filesystem Full!",
							"Filesystem $fs needs at least ".
							$self->{'hFileSystemInfo'}->{$fs}->{'NEEDED'}.
							"KB space.\n"."At the moment there are ".
							$self->{'hFileSystemInfo'}->{$fs}->{'AVAIL'}."KB available.");
				return undef;
			}
			else{
				$self->GetWizard->SetRetryMode(0);
				delete $self->{'diskspace_retry'};
			}
		}	
	}
	else{
		foreach my $fs (keys(%{$self->{'hFileSystemInfo'}})){
			if($self->{'hFileSystemInfo'}->{$fs}->{'AVAIL'} < 
				$self->{'hFileSystemInfo'}->{$fs}->{'NEEDED'}){
				$self->SetError("Error: Filesystem Full!",
							"Filesystem $fs needs at least ".
							$self->{'hFileSystemInfo'}->{$fs}->{'NEEDED'}.
							"KB space.\n"."At the moment there are ".
							$self->{'hFileSystemInfo'}->{$fs}->{'AVAIL'}."KB available.");
				$self->{'current_package'}++;
				$self->GetWizard->SetRetryMode(1);
				$self->{'diskspace_retry'} = 1;
				return undef;
			} 
		}
	}
	return 1;
}


sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	if(defined $self->{'progresswin'}){
		$self->{'progresswin'}->Destroy;
		delete $self->{'progresswin'};
	}
	if(defined $self->{'caption'}){
		$self->{'caption'}->Destroy;
		delete $self->{'caption'};
	}
	$self->SUPER::Destroy;
}



sub DESTROY{
	my ($self) = @_;
	print ">>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
