#!/usr/bin/perl
#
# $Header: //sapdb/V76/c_00/b_34/sys/src/install/perl/SAPDB/Install/Wizard/Dialog/UpdateCheck.pm#1 $
# $DateTime: 2006/09/04 17:37:38 $
# $Change: 134683 $
#
# Desc:
#

package SAPDB::Install::Wizard::Dialog::UpdateCheck;

use Wx::Event qw(EVT_TEXT);

use Wx qw (	wxVERTICAL wxHORIZONTAL wxEXPAND
			wxALL wxTOP wxRIGHT wxYES_NO wxICON_QUESTION
			wxTE_READONLY wxTE_MULTILINE wxID_YES);


use SAPDB::Install::Tools;
use SAPDB::Install::System;
use base SAPDB::Install::Wizard::Dialog::Base;

sub new{
	my($this ) = shift->SUPER::new(@_);
	return $this;
}


sub init(){
	my ($self,$config) = @_;
	my $sizer = Wx::BoxSizer->new(wxVERTICAL);
	$self->{'label'} = Wx::StaticText->new($self,-1,'Checking database '.$config->{'dbupdate'}->{'dbname'});
	$self->{'text'} = Wx::TextCtrl->new($self,-1,'',[0,0],[150,100],wxTE_READONLY | wxTE_MULTILINE);
	$sizer->Add($self->{'label'},0,wxALL,20);
	$sizer->Add($self->{'text'},1,wxEXPAND | wxALL,20);
	$self->SetAutoLayout(1);
	$self->SetSizerAndFit($sizer);
	$self->{'initialized'} = 1;
	return {'auto_next' => 1}; 

}



sub Process{
	my ($self,$config) = @_;
	my $instance_data = $config->{'dbupdate'}->{'registry_instancedata'};
	
	if(	defined $instance_data and
		$instance_data->{'INSTANCE_CHECK_OK'}){
		$config->{'container'}->Log->SetMsg("MSG: last instance check finished successfully\n");
		unless($instance_data->{'MIGRATION_STRATEGY'} =~ /\S/){
			$self->SetError('Cannot resume broken database update!','Migration strategy is unkown');
			return undef;
		}
		if(not $instance_data->{'SOFTWARE_OK'}){
			if(($^O =~ /mswin/i or not $config->{'comm'}->state->{'is_updatable'})
			    and $config->{'comm'}->state->{'is_running'}){
				unless($config->{'comm'}->stop){
					$self->SetError('Cannot stop communication server',$config->{'comm'}->last_error);
					return undef;
				}
			}	
		}
		return 1;
	}
	$config->{'container'}->Log->SetMsg("MSG: starting new instance check\n");
	
	require SAPDB::Install::Instance;
	my $instance = SAPDB::Install::Instance->new ();
	
	$instance->{'msg0'}= sub {
			my ($text) = @_;
			$self->{'text'}->AppendText($text);
			&{$SAPDB::Install::Values::RefreshGui}();
	};
	
	$instance->{'msg1'}= 
		sub {
			$config->{'container'}->Log->SetMsg(@_);
			&{$SAPDB::Install::Values::RefreshGui}(); 
		};
	
	my $release = $config->{'collector'}->package_index->
				  {'Database Kernel'}->{'SOFTWARE_VERSION'};  
	
	my $apo_release = $SAPDB::Install::Values::APO_release;
			
	$apo_release =~ s/\.(\d+)$/\ Build\ $1/; 
	
	
	#
	#	check APO COM version 
	#
	
	if(	defined $instance_data and
		exists $instance_data->{'APO_TARGET_RELEASE'} and
		$instance_data->{'APO_TARGET_RELEASE'} =~ /\S/ and
		$apo_release =~ /\S/ and 
		$instance_data->{'APO_TARGET_RELEASE'} ne $apo_release){
		$self->SetError('Start pending liveCache update failed',"cannot resume update: wrong APO COM version [$apo_release vs. ".
						$instance_data->{'APO_TARGET_RELEASE'}."]");
		return undef;
	
	}
	
	
	
	my $rc = $instance->session (
		'apo_target_release' => $apo_release,
		'dbname' => $config->{'dbupdate'}->{'dbname'},
		'dbhost' => 'localhost',
		'userpwd' => $config->{'dbupdate'}->{'user_passwd'},
		'target_release' => $release,
		'additional' => $instance_data->{'ADDITIONAL'},
		'logpos' => $instance_data->{'LOGPOS'},
		'backup_logpos' => $instance_data->{'BACKUP_LOGPOS'},
		'default_code' => $instance_data->{'DEFAULT_CODE'},
		'mcod' => $instance_data->{'MCOD'},
		'lcowner' => $instance_data->{'LCOWNER'}
                                    );
		
	
	unless($rc){
		$self->SetError('Error','Cannot create db session.');
		return undef;
	}
		
	$instance->prepare;	
	
	my %new_instance_data = (
			'APO_STARTING_RELEASE' => $instance->{'apo_starting_release'},
			'APO_TARGET_RELEASE' => $apo_release,
			'TYPE' => $instance->{'instancetype'},
			'MIGRATION_STRATEGY' => $instance->{'migration_strategy'},
			'STARTING_RELEASE' => $config->{'dbupdate'}->{'version'},
			'TARGET_RELEASE' => $release,
			'INSTANCE_CHECK_OK' => ($instance->get_errorstate () eq 'OK' ? 1 : 0),
			'ADDITIONAL' => $instance->{'additional'},
			'LOGPOS' => $instance->{'logpos'},
			'BACKUP_LOGPOS' => $instance->{'backup_logpos'},
			'DEFAULT_CODE' => $instance->{'default_code'}
	);
		
	$config->{'container'}->InstReg->setInstanceData($config->{'dbupdate'}->{'dbname'},\%new_instance_data);	
	$config->{'container'}->InstReg->DenyWrite(0);
	$config->{'container'}->InstReg->dump;
	$config->{'container'}->InstReg->DenyWrite(1);
	
	
	unless(defined $config->{'dbupdate'}->{'registry_instancedata'}){
		my %instance_data = $config->{'container'}->InstReg->
				getInstanceData($config->{'dbupdate'}->{'dbname'});
		$config->{'dbupdate'}->{'registry_instancedata'} =
			\%instance_data;
	}
	
	
	if($instance->get_errorstate () ne 'OK'){
		$self->{'text'}->AppendText('ERROR: '.$instance->{msgtxt});
		$instance->disconnect() if $instance->can ('disconnect');
		$self->SetError($config->{'dbupdate'}->{'dbname'}.' not ready for update',$instance->{msgtxt});
		return undef;
	}
	
	
	if($^O =~ /mswin/i){
		my $other_running_instances = $instance->get_running_instances();
	
		if(defined $other_running_instances && $#{@$other_running_instances} >= 0){
			my $dlg = Wx::MessageDialog->new($self,'Stop other databases now?','Found Running Databases',wxYES_NO | wxICON_QUESTION);			
			unless($dlg->ShowModal == wxID_YES){
				$dlg->Destroy;
				return undef;
			}
			$dlg->Destroy;
			
			require SAPDB::Install::Dialog::DBStop;	
			
			foreach my $running_instance (@$other_running_instances){
				my $dlg = SAPDB::Install::Dialog::DBStop->new({'dbname' => $running_instance},$self,-1);
				my $rc = $dlg->ShowModal;
				$dlg->Destroy;
				unless($rc){
					return undef;
				}
			}
		}
	}

	#shutdown
	if($instance->shutdown()){
		$instance->disconnect() if $instance->can ('disconnect');
		$self->SetError('Error',"cannot stop db instance \n");
		return undef
	}

	$instance->disconnect() if $instance->can ('disconnect');
	
	if($^O =~ /mswin/i or not $config->{'comm'}->state->{'is_updatable'}){
		unless($config->{'comm'}->stop){
			$self->SetError('Cannot stop communication server',$config->{'comm'}->last_error);
			return undef;
		}
	}	
	return 1;

		
	
	
}

sub BackInit{
	my ($self,$config) = @_;
	return 1;
}
	

sub Destroy{
	my ($self) = @_;
	if(defined $self->GetSizer){
		while($self->GetSizer->Remove(0)){}
	}
	$self->SUPER::Destroy;
}

sub DESTROY{
	my ($self) = @_;
	print ">>>>> destroying $self\n"
		if $SAPDB::Install::DEBUG::DESTRUCT;
}


1;
